@@ -5,44 +5,46 @@ __DOC__ = '''
 Functions for Input/Output, especially reading specific types
 of scientific data files.
 '''
+
 from .fileutils import (increment_filename, new_filename, new_dirname,
                         fix_filename, fix_varname, pathOf, unixpath,
                         winpath, nativepath, strip_quotes, get_timestamp,
-                        asciikeys)
+                        gformat, asciikeys)
 
 from .columnfile import (read_ascii, write_ascii, write_group, set_array_labels,
-                         guess_filereader, look_for_nans)
+                         guess_filereader, look_for_nans, read_fdmnes)
 from .xdi import read_xdi, XDIFile, XDIFileException
 from .mda import read_mda
 from .hdf5group import h5file, h5group, netcdf_file, netcdf_group
-
 from .xsp3_hdf5 import read_xsp3_hdf5
 from .xrf_netcdf import read_xrf_netcdf
 from .xrd_netcdf import read_xrd_netcdf
 from .xrd_hdf5 import read_xrd_hdf5
-
 from .xdi import read_xdi
-
 from .gse_escan import gsescan_group, gsescan_deadtime_correct
 from .gse_xdiscan import read_gsexdi, gsexdi_deadtime_correct, is_GSEXDI
 from .gse_mcafile import gsemca_group, GSEMCA_File
-from .save_restore import save, restore
+
+from .save_restore import (save_session, load_session, read_session,
+                           clear_session, is_larch_session_file,
+                           save_groups, read_groups)
+
+from . import tifffile
 from .tifffile import TIFFfile
-from .athena_project import (is_athena_project, read_athena, AthenaProject, create_athena,
-                             extract_athenagroup, make_hashkey)
+from .athena_project import (is_athena_project, read_athena, AthenaProject,
+                             create_athena, extract_athenagroup,
+                             make_hashkey)
 
 from .xafs_beamlines import guess_beamline
-
 from .csvfiles import groups2csv, read_csv
 from .export_modelresult import export_modelresult
 from .mergegroups import merge_groups
-from .specfile_reader import (HAS_SPECFILE, spec_getscan2group,
-                              spec_getmap2group, spec_getmrg2group,
-                              str2rng_larch, read_specfile, open_specfile,
-                              is_specfile)
 
+from .specfile_reader import (str2rng_larch, read_specfile, open_specfile,
+                              is_specfile)
 from .stepscan_file import read_stepscan
-from . import tifffile
+
+
 def read_tiff(fname, _larch=None, *args, **kws):
     """read image data from a TIFF file as an array"""
     return tifffile.imread(fname, *args, **kws)
@@ -82,8 +84,12 @@ __exports__ = dict(increment_filename=increment_filename,
                    read_stepscan=read_stepscan,
                    read_tiff=read_tiff,
                    merge_groups=merge_groups,
-                   save=save,
-                   restore=restore,
+                   save_session=save_session,
+                   clear_session=clear_session,
+                   load_session=load_session,
+                   read_session=read_session,
+                   save_groups=save_groups,
+                   read_groups=read_groups,
                    read_xrd_hdf5=read_xrd_hdf5,
                    read_xrd_netcdf=read_xrd_netcdf,
                    read_xrf_netcdf=read_xrf_netcdf,
@@ -92,17 +98,10 @@ __exports__ = dict(increment_filename=increment_filename,
                    h5file=h5file,
                    netcdf_file=netcdf_file,
                    netcdf_group=netcdf_group,
+                   str2rng=str2rng_larch,
+                   read_specfile=read_specfile,
+                   specfile=open_specfile,
+                   read_fdmnes=read_fdmnes
                    )
 
-
-
-
-if HAS_SPECFILE:
-    __exports__.update(dict(read_specfile_scan=spec_getscan2group,
-                            read_specfile_map=spec_getmap2group,
-                            read_specfile_mrg=spec_getmrg2group,
-                            str2rng=str2rng_larch,
-                            read_specfile=read_specfile,
-                            specfile=open_specfile))
-
 _larch_builtins = {'_io':__exports__}
