@@ -5,6 +5,7 @@ Code to read and write Athena Project files
 """
 
 import os
+import io
 import sys
 import time
 import json
@@ -18,12 +19,16 @@ from numpy.random import randint
 
 from larch import Group
 from larch import __version__ as larch_version
-from larch.utils.strutils import bytes2str, str2bytes, fix_varname
+from larch.utils.strutils import bytes2str, str2bytes, fix_varname, asfloat
+
 from xraydb import guess_edge
+import asteval
+
+hexopen = '\\x{'
+hexclose = '}'
 
 alist2json = str.maketrans("();'\n", "[] \" ")
 
-
 def plarray2json(text):
     return json.loads(text.split('=', 1)[1].strip().translate(alist2json))
 
@@ -34,13 +39,6 @@ def parse_arglist(text):
     return json.loads(txt.translate(alist2json))
 
 
-def asfloat(x):
-    """try to convert value to float, or fail gracefully"""
-    try:
-        return float(x)
-    except (ValueError, TypeError):
-        return x
-
 
 ERR_MSG = "Error reading Athena Project File"
 
@@ -269,7 +267,136 @@ def clean_fft_params(grp):
     return grp
 
 
+def text2list(text):
+    key, txt = [a.strip() for a in text.split('=', 1)]
+    if txt.endswith('\n'):
+        txt = txt[:-1]
+    if txt.endswith(';'):
+        txt = txt[:-1]
+    txt = txt.replace('=>', ':').replace('\n', ' ').replace('\r', ' ').replace('\t', ' ')
+    # re-cast unicode stored by perl (\x{e34} -> 0xe4)
+    if hexopen in txt:
+        w = []
+        k = 0
+        for i in range(len(txt)-3):
+            if txt[i:i+3] == hexopen:
+                j = txt[i:i+8].find(hexclose)
+                if j > 0:
+                    w.extend((txt[k:i], chr(int('0x' + txt[i+3:i+j], base=16))))
+                    k = i+j+1
+        w.append(txt[k:])
+        txt = ''.join(w)
+    return txt
+
+
 def parse_perlathena(text, filename):
+    """
+    parse old athena file format to Group of Groups
+    """
+    aout = io.StringIO()
+    aeval = asteval.Interpreter(minimal=True, writer=aout, err_writer=aout,
+                                max_statement_length=12543000)
+
+    lines = text.split('\n')
+    athenagroups = []
+    raw = {'name':''}
+    vline = lines.pop(0)
+    if  "Athena project file -- " not in vline:
+        raise ValueError("%s '%s': invalid Athena File" % (ERR_MSG, filename))
+    major, minor, fix = '0', '0', '0'
+    if 'Demeter' in vline:
+        try:
+            vs = vline.split("Athena project file -- Demeter version")[1]
+            major, minor, fix = vs.split('.')
+        except:
+            raise ValueError("%s '%s': cannot read version" % (ERR_MSG, filename))
+    else:
+        try:
+            vs = vline.split("Athena project file -- Athena version")[1]
+            major, minor, fix = vs.split('.')
+        except:
+            raise ValueError("%s '%s': cannot read version" % (ERR_MSG, filename))
+
+    header = [vline]
+    journal = ['']
+    is_header = True
+    ix = 0
+    for t in lines:
+        ix += 1
+        if t.startswith('#') or len(t) < 2 or 'undef' in t:
+            if is_header:
+                header.append(t)
+            continue
+        is_header = False
+        key = t.split(' ')[0].strip()
+        key = key.replace('$', '').replace('@', '').replace('%', '').strip()
+        if key == 'old_group':
+            raw['name'] = aeval(text2list(t))
+        elif key == '[record]':
+            athenagroups.append(raw)
+            raw = {'name':''}
+        elif key == 'journal':
+            try:
+                journal = aeval(text2list(t))
+            except ValueError:
+                pass
+            if len(aeval.error) > 0:
+                print(f" warning: may not read journal from '{filename:s}' completely")
+                journal = text2list(t)
+
+        elif key == 'args':
+            raw['args'] = aeval(text2list(t))
+        elif key == 'xdi':
+            raw['xdi'] = t
+        elif key in ('x', 'y', 'i0', 'signal', 'stddev'):
+            raw[key] = np.array([float(x) for x in aeval(text2list(t))])
+        elif key in ('1;', 'indicator', 'lcf_data', 'plot_features'):
+            pass
+        else:
+            print(" do not know what to do with key '%s' at '%s'" % (key, raw['name']))
+
+    out = Group()
+    out.__doc__ = """XAFS Data from Athena Project File %s""" % (filename)
+    out.journal = '\n'.join(journal)
+    out.group_names = []
+    out.header = '\n'.join(header)
+    for dat in athenagroups:
+        label = dat.get('name', 'unknown')
+        this = Group(energy=dat['x'], mu=dat['y'],
+                     athena_params=Group(id=label, bkg=Group(), fft=Group()))
+
+        if 'i0' in dat:
+            this.i0 = dat['i0']
+        if 'signal' in dat:
+            this.signal = dat['signal']
+        if 'stddev' in dat:
+            this.stddev = dat['stddev']
+        if 'args' in dat:
+            for i in range(len(dat['args'])//2):
+                key = dat['args'][2*i]
+                val = dat['args'][2*i+1]
+                if key.startswith('bkg_'):
+                    setattr(this.athena_params.bkg, key[4:], asfloat(val))
+                elif key.startswith('fft_'):
+                    setattr(this.athena_params.fft, key[4:], asfloat(val))
+                elif key == 'label':
+                    label = this.label = val
+                elif key in ('valence', 'lasso_yvalue', 'epsk', 'epsr'):
+                    setattr(this, key, asfloat(val))
+                elif key in ('atsym', 'edge'):
+                    setattr(this, key, val)
+                else:
+                    setattr(this.athena_params, key, asfloat(val))
+        this.__doc__ = """Athena Group Name %s (key='%s')""" % (label, dat['name'])
+        name = fix_varname(label)
+        if name.startswith('_'):
+            name = 'd' + name
+        setattr(out, name, this)
+        out.group_names.append(name)
+    return out
+
+
+def parse_perlathena_old(text, filename):
     """
     parse old athena file format to Group of Groups
     """
@@ -329,10 +456,9 @@ def parse_perlathena(text, filename):
     out.header = '\n'.join(header)
     for dat in athenagroups:
         label = dat.get('name', 'unknown')
-        this = Group(athena_id=label, energy=dat['x'], mu=dat['y'],
-                     bkg_params=Group(),
-                     fft_params=Group(),
-                     athena_params=Group())
+        this = Group(energy=dat['x'], mu=dat['y'],
+                     athena_params=Group(id=label, bkg=Group(), fft=Group()))
+
         if 'i0' in dat:
             this.i0 = dat['i0']
         if 'signal' in dat:
@@ -344,15 +470,14 @@ def parse_perlathena(text, filename):
                 key = dat['args'][2*i]
                 val = dat['args'][2*i+1]
                 if key.startswith('bkg_'):
-                    setattr(this.bkg_params, key[4:], asfloat(val))
+                    setattr(this.athena_params.bkg, key[4:], asfloat(val))
                 elif key.startswith('fft_'):
-                    setattr(this.fft_params, key[4:], asfloat(val))
+                    setattr(this.athena_params.fft, key[4:], asfloat(val))
                 elif key == 'label':
                     label = this.label = val
-                elif key in ('valence', 'lasso_yvalue',
-                             'epsk', 'epsr', 'importance'):
+                elif key in ('valence', 'lasso_yvalue', 'epsk', 'epsr'):
                     setattr(this, key, asfloat(val))
-                elif key in ('atsym', 'edge', 'provenance'):
+                elif key in ('atsym', 'edge'):
                     setattr(this, key, val)
                 else:
                     setattr(this.athena_params, key, asfloat(val))
@@ -391,10 +516,9 @@ def parse_jsonathena(text, filename):
         dat = jsdict[name]
         x = np.array(dat['x'], dtype='float64')
         y = np.array(dat['y'], dtype='float64')
-        this = Group(athena_id=name, energy=x, mu=y,
-                     bkg_params=Group(),
-                     fft_params=Group(),
-                     athena_params=Group())
+        this = Group(energy=x, mu=y,
+                     athena_params=Group(id=name, bkg=Group(), fft=Group()))
+
         if 'i0' in dat:
             this.i0 = np.array(dat['i0'], dtype='float64')
         if 'signal' in dat:
@@ -404,11 +528,15 @@ def parse_jsonathena(text, filename):
         if 'args' in dat:
             for key, val in dat['args'].items():
                 if key.startswith('bkg_'):
-                    setattr(this.bkg_params, key[4:], asfloat(val))
+                    setattr(this.athena_params.bkg, key[4:], asfloat(val))
                 elif key.startswith('fft_'):
-                    setattr(this.fft_params, key[4:], asfloat(val))
+                    setattr(this.athena_params.fft, key[4:], asfloat(val))
                 elif key == 'label':
                     label = this.label = val
+                elif key in ('valence', 'lasso_yvalue', 'epsk', 'epsr'):
+                    setattr(this, key, asfloat(val))
+                elif key in ('atsym', 'edge'):
+                    setattr(this, key, val)
                 else:
                     setattr(this.athena_params, key, asfloat(val))
         this.__doc__ = """Athena Group Name %s (key='%s')""" % (label, name)
@@ -520,7 +648,8 @@ class AthenaProject(object):
             if sname is not None:
                 signal = athena_array(group, sname)
 
-        hashkey = getattr(group, 'athena_id', None)
+        apars = getattr(group, 'athena_params', None)
+        hashkey = getattr(group, 'id', None)
         if hashkey is None or hashkey in self.groups:
             hashkey = make_hashkey()
             while hashkey in self.groups:
@@ -532,7 +661,7 @@ class AthenaProject(object):
         group.args = make_athena_args(group, hashkey)
 
         # fix parameters that are incompatible with athena
-        group.args['bkg_nnorm'] = max(1, min(3, int(group.args['bkg_nnorm'])))
+        group.args['bkg_nnorm'] = max(0, min(3, int(group.args['bkg_nnorm'])))
 
         _elem, _edge = guess_edge(group.e0)
         group.args['bkg_z'] = _elem
@@ -578,7 +707,6 @@ class AthenaProject(object):
         buff.extend(["", "@journal = {};", "", "1;", "", "",
                      "# Local Variables:", "# truncate-lines: t",
                      "# End:", ""])
-
         fopen =open
         if use_gzip:
             fopen = GzipFile
@@ -586,8 +714,8 @@ class AthenaProject(object):
         fh.write(str2bytes("\n".join([bytes2str(t) for t in buff])))
         fh.close()
 
-    def read(self, filename=None, match=None, do_preedge=True, do_bkg=True,
-             do_fft=True, use_hashkey=False):
+    def read(self, filename=None, match=None, do_preedge=True, do_bkg=False,
+             do_fft=False, use_hashkey=False):
         """
         read Athena project to group of groups, one for each Athena dataset
         in the project file.  This supports both gzipped and unzipped files
@@ -597,8 +725,8 @@ class AthenaProject(object):
             filename (string): name of Athena Project file
             match (string): pattern to use to limit imported groups (see Note 1)
             do_preedge (bool): whether to do pre-edge subtraction [True]
-            do_bkg (bool): whether to do XAFS background subtraction [True]
-            do_fft (bool): whether to do XAFS Fast Fourier transform [True]
+            do_bkg (bool): whether to do XAFS background subtraction [False]
+            do_fft (bool): whether to do XAFS Fast Fourier transform [False]
             use_hashkey (bool): whether to use Athena's hash key as the
                            group name instead of the Athena label [False]
         Returns:
@@ -619,8 +747,8 @@ class AthenaProject(object):
             1. read in all groups from a project file:
                cr_data = read_athena('My Cr Project.prj')
 
-            2. read in only the "merged" data from a Project, and don't do FFT:
-               zn_data = read_athena('Zn on Stuff.prj', match='*merge*', do_fft=False)
+            2. read in only the "merged" data from a Project, do BKG and FFT:
+               zn_data = read_athena('Zn on Stuff.prj', match='*merge*', do_bkg=True, do_fft=True)
         """
         if filename is not None:
             self.filename = filename
@@ -641,14 +769,14 @@ class AthenaProject(object):
 
         # decode JSON or Perl format
         data = None
-        try:
-            data = parse_jsonathena(text, self.filename)
-        except Exception:
-            #  try as perl format
+        if  '____header' in text[:500]:
             try:
-                data = parse_perlathena(text, self.filename)
+                data = parse_jsonathena(text, self.filename)
             except Exception:
-                print("Not perl-athena ", sys.exc_info())
+                pass
+
+        if data is None:
+            data = parse_perlathena(text, self.filename)
 
         if data is None:
             raise ValueError("cannot read file '%s' as Athena Project File" % (self.filename))
@@ -665,7 +793,7 @@ class AthenaProject(object):
             this = getattr(data, gname)
 
             if use_hashkey:
-                oname = this.athena_id
+                oname = this.athena_params.id
             is_xmu = bool(int(getattr(this.athena_params, 'is_xmu', 1.0)))
             is_chi = bool(int(getattr(this.athena_params, 'is_chi', 0.0)))
             is_xmu = is_xmu and not is_chi
@@ -675,7 +803,7 @@ class AthenaProject(object):
                 is_xmu = is_xmu and not val
 
             if is_xmu and (do_preedge or do_bkg) and (self._larch is not None):
-                pars = clean_bkg_params(this.bkg_params)
+                pars = clean_bkg_params(this.athena_params.bkg)
                 pre_edge(this,  e0=float(pars.e0),
                          pre1=float(pars.pre1), pre2=float(pars.pre2),
                          norm1=float(pars.nor1), norm2=float(pars.nor2),
@@ -688,7 +816,7 @@ class AthenaProject(object):
                            dk=float(pars.dk), clamp_lo=float(pars.clamp1),
                            clamp_hi=float(pars.clamp2))
                     if do_fft:
-                        pars = clean_fft_params(this.fft_params)
+                        pars = clean_fft_params(this.athena_params.fft)
                         kweight=2
                         if hasattr(pars, 'kw'):
                             kweight = float(pars.kw)
@@ -743,7 +871,7 @@ class AthenaProject(object):
         return out
 
 
-def read_athena(filename, match=None, do_preedge=True, do_bkg=True, do_fft=True,
+def read_athena(filename, match=None, do_preedge=True, do_bkg=False, do_fft=False,
                 use_hashkey=False,  _larch=None):
     """read athena project file
     returns a Group of Groups, one for each Athena Group in the project file
@@ -752,8 +880,8 @@ def read_athena(filename, match=None, do_preedge=True, do_bkg=True, do_fft=True,
         filename (string): name of Athena Project file
         match (string): pattern to use to limit imported groups (see Note 1)
         do_preedge (bool): whether to do pre-edge subtraction [True]
-        do_bkg (bool): whether to do XAFS background subtraction [True]
-        do_fft (bool): whether to do XAFS Fast Fourier transform [True]
+        do_bkg (bool): whether to do XAFS background subtraction [False]
+        do_fft (bool): whether to do XAFS Fast Fourier transform [False]
         use_hashkey (bool): whether to use Athena's hash key as the
                        group name instead of the Athena label [False]
 
@@ -775,8 +903,8 @@ def read_athena(filename, match=None, do_preedge=True, do_bkg=True, do_fft=True,
         1. read in all groups from a project file:
            cr_data = read_athena('My Cr Project.prj')
 
-        2. read in only the "merged" data from a Project, and don't do FFT:
-           zn_data = read_athena('Zn on Stuff.prj', match='*merge*', do_fft=False)
+        2. read in only the "merged" data from a Project, and do BKG and FFT:
+           zn_data = read_athena('Zn on Stuff.prj', match='*merge*', do_bkg=True, do_fft=True)
 
     """
     if not os.path.exists(filename):
