@@ -8,14 +8,13 @@ These methods are built on the methods from scikit-learn
 import numpy as np
 
 try:
-    from sklearn.cross_decomposition import PLSRegression, PLSCanonical, PLSSVD, CCA
-    from sklearn.model_selection import KFold, RepeatedKFold
-    from sklearn.linear_model import LassoLarsCV, LassoLars, LassoCV, Lasso
+    from sklearn.cross_decomposition import PLSRegression
+    from sklearn.model_selection import  RepeatedKFold
+    from sklearn.linear_model import LassoLarsCV, LassoLars, Lasso
     HAS_SKLEARN = True
 except ImportError:
     HAS_SKLEARN = False
 
-
 from .. import Group, isgroup
 
 from .utils import interp
@@ -144,7 +143,7 @@ def lasso_train(groups, varname='valence', arrayname='norm', alpha=None,
             (rounded to integer).  if cv_repeats is None, sqrt(len(groups))-1
             will be used (rounded).
      5.  alpha is the regularization parameter. if alpha is None it will
-         be set using LassoLarsSCV
+         be set using LassoLarsCV
     """
     if not HAS_SKLEARN:
         raise ImportError("scikit-learn not installed")
@@ -241,7 +240,7 @@ def lasso_predict(group, model):
              model.model.__repr__().startswith('Lasso'))
     if not valid:
         raise ValueError("lasso_predict needs a Lasso training model")
-    return _predict(group, mod)
+    return _predict(group, model)
 
 def pls_predict(group, model):
     """
