@@ -1,19 +1,17 @@
 #!/usr/bin/env python
-
+"""
+Larch command-line shell
+"""
 import cmd
 import os
 import sys
-import numpy
-import matplotlib
-
-from .symboltable import SymbolTable
 from .interpreter import Interpreter
-from .site_config import history_file, show_site_config
+
+from .site_config import history_file
 from .version import make_banner
-from .inputText import InputText
 from .larchlib import StdWriter
 from .utils import uname
-from .wxlib import LarchWxApp
+
 HAS_READLINE = False
 try:
     import readline
@@ -24,19 +22,20 @@ except ImportError:
 HAS_WXPYTHON = False
 try:
     import wx
+    from .wxlib import LarchWxApp
     HAS_WXPYTHON = True
 except ImportError:
     wx = None
 
 
-class shell(cmd.Cmd):
+class Shell(cmd.Cmd):
+    """command shell for Larch"""
     def __init__(self,  completekey='tab', debug=False, quiet=False,
                  stdin=None, stdout=None, banner_msg=None,
-                 maxhist=25000, with_wx=False, with_plugins=True):
+                 maxhist=25000, with_wx=False):
 
         self.debug  = debug
         cmd.Cmd.__init__(self,completekey='tab')
-        homedir = os.environ.get('HOME', os.getcwd())
 
         if stdin is not None:
             sys.stdin = stdin
@@ -49,10 +48,9 @@ class shell(cmd.Cmd):
             try:
                 readline.read_history_file(history_file)
             except IOError:
-                print('could not read history from %s' % history_file)
+                print(f'could not read history from {history_file}')
 
-        self.larch = Interpreter(with_plugins=with_plugins,
-                                 historyfile=history_file,
+        self.larch = Interpreter(historyfile=history_file,
                                  maxhistory=maxhist)
         self.larch.writer = StdWriter(_larch=self.larch)
 
@@ -60,22 +58,26 @@ class shell(cmd.Cmd):
             symtable = self.larch.symtable
 
             app = LarchWxApp(redirect=False, clearSigInt=False)
-            
+
             symtable.set_symbol('_sys.wx.wxapp', app)
             symtable.set_symbol('_sys.wx.force_wxupdate', False)
             symtable.set_symbol('_sys.wx.parent', None)
 
-            def onCtrlC(*args, **kws): return 0
+            def on_ctrl_c(*args, **kws):
+                return 0
 
             from .wxlib import inputhook
             symtable.set_symbol('_sys.wx.inputhook', inputhook)
-            symtable.set_symbol('_sys.wx.ping',   inputhook.ping)
-            inputhook.ON_INTERRUPT = onCtrlC
+            if uname == 'darwin':
+                symtable.set_symbol('_sys.wx.ping',   inputhook.ping_darwin)
+            else:
+                symtable.set_symbol('_sys.wx.ping',   inputhook.ping)
+            inputhook.ON_INTERRUPT = on_ctrl_c
             inputhook.WXLARCH_SYM = symtable
 
         self.prompt = self.larch.input.prompt
         writer = self.larch.writer
-        self.color_writer = uname != 'win' and hasattr(writer, 'set_textstyle')
+        self.color_writer = (uname != 'win' and hasattr(writer, 'set_textstyle'))
         if not quiet:
             if banner_msg is None:
                 banner_msg = make_banner(mods=[wx])
@@ -86,9 +88,11 @@ class shell(cmd.Cmd):
             if self.color_writer:
                 writer.set_textstyle('text')
 
+        self.larch_execute = self.default
         self.larch.run_init_scripts()
 
     def on_exit(self, text=None):
+        "exit"
         trim_last = False
         if text is not None:
             trim_last = text.strip() in ('quit', 'exit')
@@ -96,51 +100,52 @@ class shell(cmd.Cmd):
             self.larch.input.history.save(trim_last=trim_last)
         except PermissionError:
             print("Warning: could not save session history -- permission denied")
+        self.larch.symtable._plotter.close_all_displays()
         sys.exit()
 
-    def do_quit(self, text):
+    def do_exit(self, text):
+        "exit"
         self.on_exit(text=text)
 
-    def do_exit(self, text):
+    def do_quit(self, text):
+        "quit"
         self.on_exit(text=text)
 
     def emptyline(self):
         pass
 
-    def onecmd(self, txt):
-        return self.default(txt)
+    def onecmd(self, line):
+        "single command"
+        return self.default(line)
 
-    def do_help(self, txt):
-        if txt.startswith('(') and txt.endswith(')'):
-            txt = txt[1:-1]
-        elif txt.startswith("'") and txt.endswith("'"):
-            txt = txt[1:-1]
-        elif txt.startswith('"') and txt.endswith('"'):
-            txt = txt[1:-1]
-        self.default("help(%s)" % txt)
+    def do_help(self, arg):
+        "help"
+        if arg.startswith('(') and arg.endswith(')'):
+            arg = arg[1:-1]
+        elif arg.startswith("'") and arg.endswith("'"):
+            arg = arg[1:-1]
+        elif arg.startswith('"') and arg.endswith('"'):
+            arg = arg[1:-1]
+        self.default(f"help({arg})")
 
     def do_shell(self, txt):
+        "shell command"
         os.system(txt)
 
-    def larch_execute(self, text):
-        self.default(text)
-
-    def default(self, text):
-        if text.strip() in ('quit', 'exit', 'quit()', 'exit()', 'EOF'):
-            self.on_exit(text)
-        ret = self.larch.eval(text, fname='<stdin>', lineno=0)
+    def default(self, line):
+        "default handler"
+        if line.strip() in ('quit', 'exit', 'quit()', 'exit()', 'EOF'):
+            self.on_exit(line)
+        ret = self.larch.eval(line, fname='<stdin>', lineno=0)
         if self.larch.error:
             self.larch.input.clear()
             if self.color_writer:
                 self.larch.writer.set_textstyle('error')
             self.larch.show_errors()
             if self.color_writer:
-                self.larch.writer.set_textstyle('text')
+                self.larch.writer.set_textstyle('line')
         if ret is not None:
             self.larch.writer.write("%s\n" % repr(ret))
 
         self.larch.writer.flush()
         self.prompt = self.larch.input.next_prompt
-
-if __name__ == '__main__':
-    t = shell(debug=True).cmdloop()
