@@ -3,30 +3,27 @@
 site configuration for larch:
 
    init_files:  list of larch files run (in order) on startup
-   module_path: list of directories to search for larch code
    history_file:
 """
 from __future__ import print_function
 
 import sys
 import os
-from os.path import exists, abspath, join
 from .utils import uname, get_homedir, nativepath, unixpath
-from .version import __version__ as larch_version
-from .version import __release_version__ as release_version
-
+from .version import __version__, __release_version__
 
+larch_version = __version__
 # lists of recommended packages that are not installed by default
 # but may be installed if several of the larch apps are run.
 extras_wxgraph = set(['wxutils', 'wxmplot'])
 extras_epics = set(['pyepics', 'epicsapps', 'psycopg2-binary'])
-extras_xrd   = set(['fabio', 'pyfai', 'pycifrw', 'pymatgen'])
 extras_doc   = set(['pytest', 'sphinx', 'numpydoc',
                     'sphinxcontrib-bibtex', 'sphinxcontrib-argdoc'])
 extras_qtgraph = set(['pyqt5', 'pyqtwebengine', 'pyqtgraph'])
 
 def pjoin(*args):
-    return nativepath(join(*args))
+    "simple join"
+    return nativepath(os.path.join(*args))
 
 ##
 # set system-wide and local larch folders
@@ -68,28 +65,13 @@ if hasattr(sys, 'frozen'):
         toplevel, bindir = os.path.split(tdir)
         larchdir = pjoin(toplevel, 'Resources', 'larch')
 
-modules_path = []
-plugins_path = []
-_path = [user_larchdir]
-
-if 'LARCHPATH' in os.environ:
-    _path.extend([nativepath(s) for s in os.environ['LARCHPATH'].split(':')])
-
-for pth in _path:
-    mdir = pjoin(pth, 'modules')
-    if exists(mdir) and mdir not in modules_path:
-        modules_path.append(mdir)
-
-    pdir = pjoin(pth, 'plugins')
-    if exists(pdir) and pdir not in plugins_path:
-        plugins_path.append(pdir)
 
 # initialization larch files to be run on startup
 init_files = [pjoin(user_larchdir, 'init.lar')]
 
 if 'LARCHSTARTUP' in os.environ:
     startup = os.environ['LARCHSTARTUP']
-    if exists(startup):
+    if os.path.exists(startup):
         init_files = [nativepath(startup)]
 
 # history file:
@@ -103,25 +85,24 @@ def make_user_larchdirs():
              }
     subdirs = {'matplotlib': 'matplotlib may put files here',
                'dlls':       'put dlls here',
-               'feff':       'Feff files and folders here',
-               'modules':    'put custom larch or python modules here',
-               'plugins':    'put custom larch plugins here (deprecated!!)'}
+               'feff':       'Feff files and folders here'}
 
     def make_dir(dname):
-        if not exists(dname):
+        "create directory"
+        if not os.path.exists(dname):
             try:
                 os.mkdir(dname)
             except PermissionError:
-                print("no permission to create directory ", dname)
+                print(f'no permission to create directory {dname}')
             except (OSError, TypeError):
                 print(sys.exc_info()[1])
 
     def write_file(fname, text):
-        if not exists(fname):
+        "write wrapper"
+        if not os.path.exists(fname):
             try:
-                f = open(fname, 'w')
-                f.write('# %s\n' % text)
-                f.close()
+                with open(fname, 'w', encoding='utf-8') as fh:
+                    fh.write(f'# {text}\n')
             except:
                 print(sys.exc_info()[1])
 
@@ -136,22 +117,17 @@ def make_user_larchdirs():
 
 
 def show_site_config():
+    "print config"
     print( """===  Larch Configuration
-  larch release:        %s
-  larch version:        %s
-  sys executable:       %s
-  sys is frozen:        %s
-  users larch dir:      %s
-  users history_file:   %s
-  users startup files:  %s
-  modules search path:  %s
-  plugins search path:  %s (deprecated!!)
+  larch release:        {__release_version__}
+  larch version:        {__version__}
+  release version:        {__release_version__}
+  sys executable:       {sys.executable}
+  sys is frozen:        repr(getattr(sys, 'frozen', False))
+  users larch dir:      {user_larchdir}
+  users history_file:   {history_file}
 ========================
-""" % (larch_release, larch_version, sys.executable,
-       repr(getattr(sys, 'frozen', False)),
-       user_larchdir,
-       history_file, init_files,
-       modules_path, plugins_path))
+""")
 
 def system_settings():
     """set system-specific Environmental Variables, and make sure
