@@ -12,9 +12,11 @@ fileprompt       launch file browser to select files.
 
 '''
 
-import platform
-import os
+import locale
 
+from pyshortcuts import platform
+import os
+import sys
 HAS_WXPYTHON = False
 try:
     import wx
@@ -25,29 +27,43 @@ except (ImportError, AttributeError):
 _larch_name = '_sys.wx'
 _larch_builtins = {}
 
-FONTSIZE = 9
-if platform.system() == 'Windows':
+FONTSIZE = 11
+FONTSIZE_FW = 11
+if platform == 'win':
     FONTSIZE = 10
-if platform.system() == 'Darwin':
+    FONTSIZE_FW = 10
+    locale.setlocale(locale.LC_ALL, 'C')
+
+if platform == 'darwin':
     FONTSIZE = 12
+    FONTSIZE_FW = 12
 
 if HAS_WXPYTHON:
+    try:
+        import wxutils
+        import wxmplot
+    except:
+        from ..apps import install_extras
+        from ..site_config import extras_wxgraph
+        install_extras(extras_wxgraph)
+
 
     from . import larchframe
     from . import larchfilling
     from . import readlinetextctrl
 
-
     from wxutils import (set_sizer, pack, SetTip, Font, HLine, Check,
                          MenuItem, Popup, RIGHT, LEFT, CEN , LTEXT,
                          FRAMESTYLE, hms, DateTimeCtrl, Button, TextCtrl,
                          ToggleButton, BitmapButton, Choice, YesNo,
-                         SimpleText, LabeledTextCtrl, HyperText,
-                         get_icon, GUIColors, OkCancel,
-                         FileOpen, FileSave, SelectWorkdir, fix_filename,
-                         SavedParameterDialog, FileCheckList,
-                         FileDropTarget, GridPanel, RowPanel, make_steps,
-                         set_float, FloatCtrl)
+                         SimpleText, LabeledTextCtrl, HyperText, get_icon,
+                         OkCancel, FileOpen, FileSave, SelectWorkdir,
+                         fix_filename, SavedParameterDialog, GridPanel,
+                         RowPanel, make_steps, set_float, FloatCtrl,
+                         EditableListBox)
+
+    from .colors import COLORS, GUIColors, set_color
+    from .filechecklist import FileCheckList, FileDropTarget
 
     from .floats import NumericCombo, FloatSpin, FloatSpinWithPin
 
@@ -57,8 +73,10 @@ if HAS_WXPYTHON:
     from .columnframe import ColumnDataFileFrame, EditColumnFrame
     from .athena_importer import AthenaImporter
     from .specfile_importer import SpecfileImporter
-    from .reportframe import ReportFrame
-    from .gui_utils import gcd, databrowser, fileprompt, wx_update, EditableListBox
+    from .reportframe import ReportFrame, DictFrame, DataTableGrid
+    from .gui_utils import (gcd, databrowser, fileprompt, LarchWxApp,
+                            wx_update, show_wxsizes)
+
     from .larch_updater import LarchUpdaterDialog
     from .parameter import ParameterWidgets, ParameterPanel
     from .periodictable import PeriodicTablePanel
@@ -76,11 +94,12 @@ if HAS_WXPYTHON:
                           _plot_marker, _plot_arrow, _plot_setlimits,
                           _plot_axvline, _plot_axhline, _scatterplot,
                           _hist, _update_trace, _saveplot, _saveimg,
-                          _getDisplay, _closeDisplays, _getcursor,
+                          get_display, _closeDisplays, _getcursor,
                           last_cursor_pos, _imshow, _contour, _xrf_plot,
-                          _xrf_oplot, _fitplot, _redraw_plot)
+                          _xrf_oplot, _fitplot, _redraw_plot,
+                          get_zoomlimits, set_zoomlimits)
 
-    from .xrfdisplay import  XRFDisplayFrame
+    # from .xrfdisplay import  XRFDisplayFrame
 
     from . import xafsplots
     from .xafsplots import plotlabels
@@ -96,7 +115,7 @@ if HAS_WXPYTHON:
                                        update_trace=_update_trace,
                                        save_plot=_saveplot,
                                        save_image=_saveimg,
-                                       get_display=_getDisplay,
+                                       get_display=get_display,
                                        close_all_displays=_closeDisplays,
                                        get_cursor=_getcursor,
                                        last_cursor_pos=last_cursor_pos,
@@ -107,6 +126,7 @@ if HAS_WXPYTHON:
                                        redraw_plot=_redraw_plot)
 
     _larch_builtins['_xafs'] = dict(redraw=xafsplots.redraw,
+                                    plotlabels=xafsplots.plotlabels,
                                     plot_mu=xafsplots.plot_mu,
                                     plot_bkg=xafsplots.plot_bkg,
                                     plot_chie=xafsplots.plot_chie,
@@ -135,32 +155,7 @@ if HAS_WXPYTHON:
         _sys = _larch.symtable._sys
         if not hasattr(_sys, 'gui_apps'):
             _sys.gui_apps = {}
-        _sys.gui_apps['xrfviewer'] = ('XRF Spectrum Viewer', XRFDisplayFrame)
-
-
-    #############################
-    ## Hack System and Startfile on Windows totry to track down
-    ## weird error of starting other applications, like Mail
-    if platform.system() == 'Windows':
-        from os import system as os_system
-        from os import startfile as os_startfile
-        def my_system(command):
-            print("#@ os.system: ", command)
-            return os_system(command)
-
-        def my_startfile(filepath, operation=None):
-            print("#@ os.startfile: ", filepath, operation)
-            try:
-                if operation is None:
-                    return os_startfile(filepath)
-                else:
-                    return os_startfile(filepath, operation)
-            except WindowsError:
-                print("#@ os.startfile failed: ", filepath, operation)
-
-        os.system = my_system
-        os.startfile = my_startfile
-    #############################
+        # _sys.gui_apps['xrfviewer'] = ('XRF Spectrum Viewer', XRFDisplayFrame)
 
 else:
     def nullfunc(*args, **kws):
@@ -194,7 +189,3 @@ else:
                                        xrf_plot=nullfunc,
                                        xrf_oplot=nullfunc,
                                        fit_plot=nullfunc)
-
-
-
-    
