@@ -15,22 +15,25 @@ import wx
 import wx.lib.scrolledpanel as scrolled
 import wx.lib.agw.flatnotebook as fnb
 from wx.adv import AboutBox, AboutDialogInfo
+from matplotlib.ticker import FuncFormatter
 
 from wxmplot import PlotPanel
 from xraydb.chemparser import chemparse
+from xraydb import atomic_number
 
 import larch
 from larch import Group
 from larch.xafs import feff8l, feff6l
 from larch.xrd.cif2feff import cif_sites
+from larch.utils import read_textfile
 from larch.utils.paths import unixpath
 from larch.utils.strutils import fix_filename, unique_name, strict_ascii
 from larch.site_config import user_larchdir
-
+from .colors import set_color
 from larch.wxlib import (LarchFrame, FloatSpin, EditableListBox,
                          FloatCtrl, SetTip, get_icon, SimpleText, pack,
                          Button, Popup, HLine, FileSave, FileOpen, Choice,
-                         Check, MenuItem, GUIColors, CEN, LEFT, FRAMESTYLE,
+                         Check, MenuItem, CEN, LEFT, FRAMESTYLE,
                          Font, FONTSIZE, flatnotebook, LarchUpdaterDialog,
                          PeriodicTablePanel, FeffResultsPanel, LarchWxApp)
 
@@ -59,12 +62,11 @@ class CIFFrame(wx.Frame):
         if _larch is None:
             self.larch = larch.Interpreter()
         self.larch.eval("# started CIF browser\n")
-        self.larch.eval("if not hasattr('_main', '_feffruns'): _feffruns = {}")
+        self.larch.eval("if not hasattr('_sys', '_feffruns'): _sys._feffruns = {}")
 
         self.cifdb = get_amscifdb()
         self.all_minerals = self.cifdb.all_minerals()
         self.subframes = {}
-        self.plotframe = None
         self.has_xrd1d = False
         self.current_cif = None
         self.SetTitle(title)
@@ -104,6 +106,7 @@ class CIFFrame(wx.Frame):
         leftpanel = wx.Panel(splitter)
         self.ciflist = EditableListBox(leftpanel,
                                        self.onShowCIF, size=(300,-1))
+        set_color(self.ciflist, 'list_fg', bg='list_bg')
         self.cif_selections = {}
 
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -111,7 +114,7 @@ class CIFFrame(wx.Frame):
         pack(leftpanel, sizer)
 
         # right hand side
-        rightpanel = wx.Panel(splitter)
+        rightpanel = scrolled.ScrolledPanel(splitter)
         panel = wx.Panel(rightpanel)
         sizer = wx.GridBagSizer(2,2)
 
@@ -157,6 +160,9 @@ class CIFFrame(wx.Frame):
         wids['run_feff'] = Button(panel, ' Run Feff ',
                                   action=self.onRunFeff)
         wids['run_feff'].Disable()
+        wids['without_h'] = Check(panel, default=True, label='Remove H atoms',
+                                  action=self.onGetFeff)
+
 
         wids['central_atom'] = Choice(panel, choices=['<empty>'], size=(80, -1),
                                       action=self.onCentralAtom)
@@ -234,29 +240,39 @@ class CIFFrame(wx.Frame):
         sizer.Add(wids['cluster_size'], (ir, 1), (1, 1), LEFT, 3)
         sizer.Add(fverslab,             (ir, 2), (1, 1), LEFT, 3)
         sizer.Add(wids['feffvers'],     (ir, 3), (1, 1), LEFT, 3)
-        sizer.Add(wids['run_feff'],     (ir, 5), (1, 1), LEFT, 3)
+        sizer.Add(wids['without_h'],    (ir, 4), (1, 2), LEFT, 3)
 
         ir += 1
         sizer.Add(folderlab,             (ir, 0), (1, 1), LEFT, 3)
         sizer.Add(wids['run_folder'],    (ir, 1), (1, 4), LEFT, 3)
+        sizer.Add(wids['run_feff'],      (ir, 5), (1, 1), LEFT, 3)
 
         ir += 1
         sizer.Add(HLine(panel, size=(550, 2)), (ir, 0), (1, 6), LEFT, 3)
 
         pack(panel, sizer)
 
-        self.nb = flatnotebook(rightpanel, {}, drag_tabs=False,
-                               on_change=self.onNBChanged)
+        self.nb = flatnotebook(rightpanel, {}, on_change=self.onNBChanged)
 
         self.feffresults = FeffResultsPanel(rightpanel, _larch=self.larch)
 
-        self.plotpanel = PlotPanel(rightpanel)
+        def _swallow_plot_messages(s, panel=0):
+            pass
+
+        self.plotpanel = PlotPanel(rightpanel, messenger=_swallow_plot_messages)
+        try:
+            plotopts = self.larch.symtable._sys.wx.plotopts
+            self.plotpanel.conf.set_theme(plotopts['theme'])
+            self.plotpanel.conf.enable_grid(plotopts['show_grid'])
+        except:
+            pass
+
         self.plotpanel.SetMinSize((250, 250))
+        self.plotpanel.SetMaxSize((600, 450))
         self.plotpanel.onPanelExposed = self.showXRD1D
 
         cif_panel = wx.Panel(rightpanel)
-        wids['cif_text'] = wx.TextCtrl(cif_panel,
-                                       value='<CIF TEXT>',
+        wids['cif_text'] = wx.TextCtrl(cif_panel, value='<CIF TEXT>',
                                        style=wx.TE_MULTILINE|wx.TE_READONLY,
                                        size=(300, 350))
         wids['cif_text'].SetFont(Font(FONTSIZE+1))
@@ -303,6 +319,7 @@ class CIFFrame(wx.Frame):
         r_sizer.Add(panel, 0, LEFT|wx.GROW|wx.ALL)
         r_sizer.Add(self.nb, 1, LEFT|wx.GROW, 2)
         pack(rightpanel, r_sizer)
+        rightpanel.SetupScrolling()
         splitter.SplitVertically(leftpanel, rightpanel, 1)
 
     def get_nbpage(self, name):
@@ -392,6 +409,9 @@ class CIFFrame(wx.Frame):
         self.wids['central_atom'].Select(0)
 
         el0 = list(elems.keys())[0]
+        edge_val = 'K' if atomic_number(el0) < 60 else 'L3'
+        self.wids['edge'].SetStringSelection(edge_val)
+
         sites = cif_sites(cif.ciftext, absorber=el0)
         sites = ['%d' % (i+1) for i in range(len(sites))]
         self.wids['site'].Clear()
@@ -405,11 +425,17 @@ class CIFFrame(wx.Frame):
         cif  = self.current_cif
         if cif is None:
             return
-        sites = cif_sites(cif.ciftext, absorber=event.GetString())
-        sites = ['%d' % (i+1) for i in range(len(sites))]
-        self.wids['site'].Clear()
-        self.wids['site'].AppendItems(sites)
-        self.wids['site'].Select(0)
+        catom = event.GetString()
+        try:
+            sites = cif_sites(cif.ciftext, absorber=catom)
+            sites = ['%d' % (i+1) for i in range(len(sites))]
+            self.wids['site'].Clear()
+            self.wids['site'].AppendItems(sites)
+            self.wids['site'].Select(0)
+        except:
+            self.write_message(f"could not get sites for central atom '{catom}'")
+        edge_val = 'K' if atomic_number(catom) < 60 else 'L3'
+        self.wids['edge'].SetStringSelection(edge_val)
         self.onGetFeff()
 
     def onGetFeff(self, event=None):
@@ -421,12 +447,14 @@ class CIFFrame(wx.Frame):
         catom = self.wids['central_atom'].GetStringSelection()
         asite = int(self.wids['site'].GetStringSelection())
         csize = self.wids['cluster_size'].GetValue()
+        with_h = not self.wids['without_h'].IsChecked()
         mineral = cif.get_mineralname()
         folder = f'{catom:s}{asite:d}_{edge:s}_{mineral}_cif{cif.ams_id:d}'
-        folder = unique_name(folder, self.runs_list)
+        folder = unique_name(fix_filename(folder), self.runs_list)
 
         fefftext = cif.get_feffinp(catom, edge=edge, cluster_size=csize,
-                                    absorber_site=asite, version8=version8)
+                                   absorber_site=asite, version8=version8,
+                                   with_h=with_h)
 
         self.wids['run_folder'].SetValue(folder)
         self.wids['feff_text'].SetValue(fefftext)
@@ -438,6 +466,14 @@ class CIFFrame(wx.Frame):
         fefftext = self.wids['feff_text'].GetValue()
         if len(fefftext) < 100 or 'ATOMS' not in fefftext:
             return
+
+        ciftext = self.wids['cif_text'].GetValue()
+        cif  = self.current_cif
+        cif_fname = None
+        if cif is not None and len(ciftext) > 100:
+            mineral = cif.get_mineralname()
+            cif_fname = f'{mineral}_cif{cif.ams_id:d}.cif'
+
         # cc = self.current_cif
         # edge  = self.wids['edge'].GetStringSelection()
         # catom = self.wids['central_atom'].GetStringSelection()
@@ -448,7 +484,7 @@ class CIFFrame(wx.Frame):
         version8 = '8' == self.wids['feffvers'].GetStringSelection()
 
         fname = self.wids['run_folder'].GetValue()
-        fname = unique_name(fname, self.runs_list)
+        fname = unique_name(fix_filename(fname), self.runs_list)
         self.runs_list.append(fname)
         folder = unixpath(os.path.join(self.feff_folder, fname))
 
@@ -470,6 +506,11 @@ class CIFFrame(wx.Frame):
         with open(fname, 'w') as fh:
             fh.write(strict_ascii(fefftext))
 
+        if cif_fname is not None:
+            cname = unixpath(os.path.join(folder, cif_fname))
+            with open(cname, 'w') as fh:
+                fh.write(strict_ascii(ciftext))
+
         wx.CallAfter(self.run_feff, folder, version8=version8)
         # feffexe, folder=dirname, message_writer=self.feff_output)
 
@@ -483,8 +524,8 @@ class CIFFrame(wx.Frame):
 
         prog(folder=folder, message_writer=self.feff_output)
         self.larch.eval("## gathering results:\n")
-        self.larch.eval(f"_feffruns['{dname:s}'] = get_feff_pathinfo('{folder:s}')")
-        this_feffrun = self.larch.symtable._feffruns[f'{dname:s}']
+        self.larch.eval(f"_sys._feffruns['{dname:s}'] = get_feff_pathinfo('{folder:s}')")
+        this_feffrun = self.larch.symtable._sys._feffruns[f'{dname:s}']
         self.feffresults.set_feffresult(this_feffrun)
         ix, p = self.get_nbpage('Feff Results')
         self.nb.SetSelection(ix)
@@ -566,9 +607,8 @@ class CIFFrame(wx.Frame):
             fefftext = None
             _, fname = os.path.split(path)
             fname = fname.replace('.inp', '_run')
-            fname = unique_name(fname, self.runs_list)
-            with open(path, 'r') as fh:
-                fefftext = fh.read()
+            fname = unique_name(fix_filename(fname), self.runs_list)
+            fefftext = read_textfile(path)
             if fefftext is not None:
                 self.wids['feff_text'].SetValue(fefftext)
                 self.wids['run_folder'].SetValue(fname)
@@ -603,13 +643,23 @@ class CIFFrame(wx.Frame):
         mask = np.where(sfact.intensity>max_/10.0)[0]
         qval = sfact.q[mask]
         ival = sfact.intensity[mask]
-        title = '%s (cif %d)' % (self.cif_label, self.current_cif.ams_id)
-        self.plotpanel.plot(qval, ival, linewidth=0, marker='o', markersize=2,
-                            xlabel=r'$Q \rm\, (\AA^{-1})$',
-                            ylabel='Intensity (arb units)',
-                            title=title, titlefontsize=8)
-        self.plotpanel.axes.bar(qval, ival, 0.05, color='blue')
-        self.plotpanel.canvas.draw()
+        ival = ival/(1.0*ival.max())
+
+        def qd_formatter(q, pos):
+            qval = float(q)
+            dval = '\n[%.2f]' % (2*np.pi/max(qval, 1.e-6))
+            return r"%.2f%s" % (qval, dval)
+
+        qd_label = r'$Q\rm\,(\AA^{-1}) \,\> [d \rm\,(\AA)]$'
+        title = self.cif_label + '\n' + '(cif %d)' % (self.current_cif.ams_id)
+        ppan = self.plotpanel
+        ppan.plot(qval, ival, linewidth=0, marker='o', markersize=2,
+                  xlabel=qd_label, ylabel='Relative Intensity',
+                  title=title, titlefontsize=8)
+
+        ppan.axes.bar(qval, ival, 0.05, color='blue')
+        ppan.axes.xaxis.set_major_formatter(FuncFormatter(qd_formatter))
+        ppan.canvas.draw()
         self.has_xrd1d = True
 
     def onSelAll(self, event=None):
