@@ -4,6 +4,8 @@
 """
 import os
 import re
+from copy import deepcopy
+
 import numpy as np
 np.seterr(all='ignore')
 
@@ -21,16 +23,18 @@ from wxutils import (SimpleText, FloatCtrl, GUIColors, Button, Choice,
 import larch
 from larch import Group
 from larch.xafs.xafsutils import guess_energy_units
-from larch.utils.strutils import fix_varname, file2groupname
+from larch.utils.strutils import fix_varname, fix_filename, file2groupname
 from larch.io import look_for_nans
 from larch.utils.physical_constants import PLANCK_HC, DEG2RAD
+from larch.io.columnfile import guess_filereader
+
+from . import FONTSIZE
 
 CEN |=  wx.ALL
 FNB_STYLE = fnb.FNB_NO_X_BUTTON|fnb.FNB_SMART_TABS
 FNB_STYLE |= fnb.FNB_NO_NAV_BUTTONS|fnb.FNB_NODRAG
 
-XPRE_OPS = ('', 'log(', '-log(')
-YPRE_OPS = ('', 'log(', '-log(')
+YPRE_OPS = ('', 'log(', '-log(', '-')
 ARR_OPS = ('+', '-', '*', '/')
 
 YERR_OPS = ('Constant', 'Sqrt(Y)', 'Array')
@@ -105,7 +109,7 @@ class AddColumnsFrame(wx.Frame):
             cname = SimpleText(panel, label=' %s ' % name)
             csel = Check(panel, label='', default=False)
 
-            self.wids["col_%d" % i] = csel
+            self.wids["col%d" % i] = csel
 
             sizer.Add(cind,  (ir, 0), (1, 1), LEFT, 3)
             sizer.Add(csel,  (ir, 1), (1, 1), LEFT, 3)
@@ -124,8 +128,8 @@ class AddColumnsFrame(wx.Frame):
     def make_sum(self):
         sel =[]
         for name, wid in self.wids.items():
-            if name.startswith('col_') and wid.IsChecked():
-                sel.append(int(name[4:]))
+            if name.startswith('col') and wid.IsChecked():
+                sel.append(int(name[3:]))
         self.selected_columns = np.array(sel)
         narr, npts = self.group.raw.data.shape
         ydat = np.zeros(npts, dtype=np.float)
@@ -188,8 +192,8 @@ class AddColumnsFrame(wx.Frame):
                     pass
 
         for name, wid in self.wids.items():
-            if name.startswith('col_'):
-                wid.SetValue(int(name[4:]) in sel)
+            if name.startswith('col'):
+                wid.SetValue(int(name[3:]) in sel)
 
     def onSelRegex(self, event=None):
         pattern = self.wids['tc_regex'].GetValue().replace('*', '.*')
@@ -197,7 +201,7 @@ class AddColumnsFrame(wx.Frame):
         sel =[]
         for i, name in enumerate(self.group.array_labels):
             sel = re.search(pattern, name, flags=re.IGNORECASE) is not None
-            self.wids["col_%d" % i].SetValue(sel)
+            self.wids["col%d" % i].SetValue(sel)
 
 
 class EditColumnFrame(wx.Frame) :
@@ -295,7 +299,7 @@ class EditColumnFrame(wx.Frame) :
                 setter = wid.SetLabel
             else:
                 setter = wid.SetValue
-            setter("col_%d" % (int(val) +1))
+            setter("col%d" % (int(val) +1))
 
     def update(self, evt=None, index=-1):
         newval = fix_varname(self.wids["%d" % index].GetValue())
@@ -320,12 +324,14 @@ class EditColumnFrame(wx.Frame) :
 class ColumnDataFileFrame(wx.Frame) :
     """Column Data File, select columns"""
     def __init__(self, parent, filename=None, groupname=None,
-                 last_array_sel=None, read_ok_cb=None,
+                 last_array_sel=None, read_ok_cb=None, extra_sums=None,
                  edit_groupname=True, _larch=None):
         self.parent = parent
         self._larch = _larch
         self.path = filename
-        self.extra_sums = {}
+        if extra_sums is None:
+            extra_sums = {}
+        self.extra_sums = extra_sums
 
         group = self.initgroup = self.read_column_file(self.path)
 
@@ -349,7 +355,9 @@ class ColumnDataFileFrame(wx.Frame) :
         self.read_ok_cb = read_ok_cb
         self.array_sel = dict(xarr=None, yarr1=None, yarr2=None, yop='/',
                               ypop='', monod=3.1355316, en_units=en_units,
-                              yerror='constant', yerr_val=1, yerr_arr=None)
+                              yerror='constant', yerr_val=1, yerr_arr=None,
+                              yrpop='', yrop='/', yref1='', yref2='',
+                              has_yref=False)
         if last_array_sel is not None:
             self.array_sel.update(last_array_sel)
 
@@ -361,6 +369,18 @@ class ColumnDataFileFrame(wx.Frame) :
                 self.array_sel['yarr1'] = 'itrans'
             elif 'i1' in arr_labels:
                 self.array_sel['yarr1'] = 'i1'
+
+        if self.array_sel['yref1'] is None:
+            if 'iref' in arr_labels:
+                self.array_sel['yref1'] = 'iref'
+            elif 'irefer' in arr_labels:
+                self.array_sel['yref1'] = 'irefer'
+            elif 'i2' in arr_labels:
+                self.array_sel['yref1'] = 'i2'
+
+        if self.array_sel['yref2'] is None and 'i1' in arr_labels:
+            self.array_sel['yref2'] = 'i1'
+
         message = "Data Columns for %s" % group.filename
         wx.Frame.__init__(self, None, -1,
                           'Build Arrays from Data Columns for %s' % group.filename,
@@ -368,15 +388,18 @@ class ColumnDataFileFrame(wx.Frame) :
 
         x0, y0 = parent.GetPosition()
         self.SetPosition((x0+60, y0+60))
-        self.SetFont(Font(10))
 
+        self.SetFont(Font(FONTSIZE))
         panel = wx.Panel(self)
-        self.SetMinSize((600, 600))
+        self.SetMinSize((600, 700))
         self.colors = GUIColors()
 
+        def subtitle(s, fontsize=12, colour=wx.Colour(10, 10, 180)):
+            return SimpleText(panel, s, font=Font(fontsize),
+                           colour=colour, style=LEFT)
+
         # title row
-        title = SimpleText(panel, message, font=Font(12),
-                           colour=self.colors.title, style=LEFT)
+        title = subtitle(message, colour=self.colors.title)
 
         yarr_labels = self.yarr_labels = arr_labels + ['1.0', '0.0', '']
         xarr_labels = self.xarr_labels = arr_labels + ['_index']
@@ -390,33 +413,43 @@ class ColumnDataFileFrame(wx.Frame) :
         self.datatype = Choice(panel, choices=DATATYPES, action=self.onUpdate, size=(150, -1))
         self.datatype.SetStringSelection(self.workgroup.datatype)
 
-        self.datatype.SetStringSelection(self.workgroup.datatype)
-
         self.en_units = Choice(panel, choices=ENUNITS_TYPES,
                                action=self.onEnUnitsSelect, size=(150, -1))
 
-        self.ypop = Choice(panel, choices=YPRE_OPS, action=self.onUpdate, size=(150, -1))
-        self.yop =  Choice(panel, choices=ARR_OPS, action=self.onUpdate, size=(50, -1))
-        self.yerr_op = Choice(panel, choices=YERR_OPS, action=self.onYerrChoice, size=(150, -1))
+        self.ypop = Choice(panel, choices=YPRE_OPS, action=self.onUpdate, size=(100, -1))
+        self.yop =  Choice(panel, choices=ARR_OPS, action=self.onUpdate, size=(100, -1))
+        self.yerr_op = Choice(panel, choices=YERR_OPS, action=self.onYerrChoice, size=(100, -1))
         self.yerr_op.SetSelection(0)
 
-        self.yerr_val = FloatCtrl(panel, value=1, precision=4, size=(90, -1))
-        self.monod_val  = FloatCtrl(panel, value=3.1355316, precision=7, size=(90, -1))
+        self.yerr_val = FloatCtrl(panel, value=1, precision=4, size=(75, -1))
+        self.monod_val  = FloatCtrl(panel, value=3.1355316, precision=7, size=(75, -1))
 
-        xlab = SimpleText(panel, ' X array: ')
-        ylab = SimpleText(panel, ' Y array: ')
-        units_lab = SimpleText(panel, '  Units:  ')
-        yerr_lab = SimpleText(panel, ' Yerror: ')
+        xlab = SimpleText(panel, ' X array = ')
+        ylab = SimpleText(panel, ' Y array = ')
+        units_lab = SimpleText(panel, '  Units of X array:  ')
+        yerr_lab = SimpleText(panel, ' Y uncertainty = ')
         dtype_lab = SimpleText(panel, ' Data Type: ')
         monod_lab = SimpleText(panel, ' Mono D spacing (Ang): ')
         yerrval_lab = SimpleText(panel, ' Value:')
 
+        # yref
+        self.has_yref = Check(panel, label='data file includes energy reference data',
+                              default=self.array_sel['has_yref'],
+                              action=self.onYrefCheck)
+        refylab = SimpleText(panel, ' Refer array = ')
+        self.yref1 = Choice(panel, choices=yarr_labels, action=self.onUpdate, size=(150, -1))
+        self.yref2 = Choice(panel, choices=yarr_labels, action=self.onUpdate, size=(150, -1))
+        self.yrpop = Choice(panel, choices=YPRE_OPS, action=self.onUpdate, size=(100, -1))
+        self.yrop =  Choice(panel, choices=ARR_OPS, action=self.onUpdate, size=(100, -1))
+
         self.ysuf = SimpleText(panel, '')
-        self.message = SimpleText(panel, '', font=Font(11),
-                           colour=self.colors.title, style=LEFT)
+        self.message = subtitle(' ', colour=wx.Colour(0, 0, 0))
+
 
         self.ypop.SetStringSelection(self.array_sel['ypop'])
         self.yop.SetStringSelection(self.array_sel['yop'])
+        self.yrpop.SetStringSelection(self.array_sel['yrpop'])
+        self.yrop.SetStringSelection(self.array_sel['yrop'])
         self.monod_val.SetValue(self.array_sel['monod'])
         self.monod_val.SetAction(self.onUpdate)
         self.monod_val.Enable(self.array_sel['en_units'].startswith('deg'))
@@ -426,7 +459,8 @@ class ColumnDataFileFrame(wx.Frame) :
         if '(' in self.array_sel['ypop']:
             self.ysuf.SetLabel(')')
 
-        ixsel, iysel, iy2sel, iyesel = 0, 1, len(yarr_labels)-1, len(yarr_labels)-1
+        ixsel, iysel = 0, 1
+        iy2sel = iyesel = iyr1sel = iyr2sel = len(yarr_labels)-1
         if self.array_sel['xarr'] in xarr_labels:
             ixsel = xarr_labels.index(self.array_sel['xarr'])
         if self.array_sel['yarr1'] in arr_labels:
@@ -435,11 +469,30 @@ class ColumnDataFileFrame(wx.Frame) :
             iy2sel = yarr_labels.index(self.array_sel['yarr2'])
         if self.array_sel['yerr_arr'] in yarr_labels:
             iyesel = yarr_labels.index(self.array_sel['yerr_arr'])
+        if self.array_sel['yref1'] in arr_labels:
+            iyr1sel = arr_labels.index(self.array_sel['yref1'])
+        if self.array_sel['yref2'] in arr_labels:
+            iyr2sel = arr_labels.index(self.array_sel['yref2'])
 
         self.xarr.SetSelection(ixsel)
         self.yarr1.SetSelection(iysel)
         self.yarr2.SetSelection(iy2sel)
         self.yerr_arr.SetSelection(iyesel)
+        self.yref1.SetSelection(iyr1sel)
+        self.yref2.SetSelection(iyr2sel)
+
+        self.wid_filename = wx.TextCtrl(panel, value=fix_filename(group.filename),
+                                         size=(250, -1))
+        self.wid_groupname = wx.TextCtrl(panel, value=group.groupname,
+                                         size=(150, -1))
+        if not edit_groupname:
+            self.wid_groupname.Disable()
+        self.wid_reffilename = wx.TextCtrl(panel, value=fix_filename(group.filename + '_ref'),
+                                         size=(250, -1))
+        self.wid_refgroupname = wx.TextCtrl(panel, value=group.groupname + '_ref',
+                                         size=(150, -1))
+
+        self.onYrefCheck(has_yref=self.array_sel['has_yref'])
 
         bpanel = wx.Panel(panel)
         bsizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -459,17 +512,22 @@ class ColumnDataFileFrame(wx.Frame) :
         sizer.Add(title,     (0, 0), (1, 7), LEFT, 5)
 
         ir = 1
+        sizer.Add(subtitle(' X [ Energy ] Array:'),   (ir, 0), (1, 2), LEFT, 0)
+        sizer.Add(dtype_lab,       (ir, 3), (1, 1), RIGHT, 0)
+        sizer.Add(self.datatype,   (ir, 4), (1, 2), LEFT, 0)
+
+        ir += 1
         sizer.Add(xlab,      (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.xarr, (ir, 1), (1, 1), LEFT, 0)
-        sizer.Add(units_lab,     (ir, 2), (1, 2), RIGHT, 0)
+        sizer.Add(self.xarr, (ir, 1), (1, 2), LEFT, 0)
+        sizer.Add(units_lab,     (ir, 3), (1, 1), RIGHT, 0)
         sizer.Add(self.en_units,  (ir, 4), (1, 2), LEFT, 0)
 
         ir += 1
-        sizer.Add(dtype_lab,          (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.datatype,      (ir, 1), (1, 1), LEFT, 0)
-        sizer.Add(monod_lab,          (ir, 2), (1, 2), RIGHT, 0)
-        sizer.Add(self.monod_val,     (ir, 4), (1, 1), LEFT, 0)
+        sizer.Add(monod_lab,     (ir, 2), (1, 2), RIGHT, 0)
+        sizer.Add(self.monod_val,(ir, 4), (1, 1), LEFT, 0)
 
+        ir += 1
+        sizer.Add(subtitle(' Y [ \u03BC(E) ] Array:'),   (ir, 0), (1, 2), LEFT, 0)
         ir += 1
         sizer.Add(ylab,       (ir, 0), (1, 1), LEFT, 0)
         sizer.Add(self.ypop,  (ir, 1), (1, 1), LEFT, 0)
@@ -486,19 +544,29 @@ class ColumnDataFileFrame(wx.Frame) :
         sizer.Add(yerrval_lab,   (ir, 3), (1, 1), RIGHT, 0)
         sizer.Add(self.yerr_val, (ir, 4), (1, 2), LEFT, 0)
 
+        ir += 1
+        sizer.Add(SimpleText(panel, ' Display Name:'), (ir, 0), (1, 1), LEFT, 0)
+        sizer.Add(self.wid_filename,                  (ir, 1), (1, 2), LEFT, 0)
+        sizer.Add(SimpleText(panel, ' Group Name:'),   (ir, 3), (1, 1), RIGHT, 0)
+        sizer.Add(self.wid_groupname,                 (ir, 4), (1, 2), LEFT, 0)
 
-        self.wid_filename = wx.TextCtrl(panel, value=group.filename,
-                                         size=(250, -1))
-        self.wid_groupname = wx.TextCtrl(panel, value=group.groupname,
-                                         size=(150, -1))
-        if not edit_groupname:
-            self.wid_groupname.Disable()
+        ir += 2
+        sizer.Add(subtitle(' Reference [ \u03BC_ref(E) ] Array: '),
+                  (ir, 0), (1, 2), LEFT, 0)
+        sizer.Add(self.has_yref,   (ir, 2), (1, 2), LEFT, 0)
 
         ir += 1
-        sizer.Add(SimpleText(panel, 'Display Name:'), (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.wid_filename,                  (ir, 1), (1, 2), LEFT, 0)
-        sizer.Add(SimpleText(panel, 'Group Name:'),   (ir, 3), (1, 1), RIGHT, 0)
-        sizer.Add(self.wid_groupname,                 (ir, 4), (1, 2), LEFT, 0)
+        sizer.Add(refylab,    (ir, 0), (1, 1), LEFT, 0)
+        sizer.Add(self.yrpop, (ir, 1), (1, 1), LEFT, 0)
+        sizer.Add(self.yref1, (ir, 2), (1, 1), LEFT, 0)
+        sizer.Add(self.yrop,  (ir, 3), (1, 1), RIGHT, 0)
+        sizer.Add(self.yref2, (ir, 4), (1, 2), LEFT, 0)
+
+        ir += 1
+        sizer.Add(SimpleText(panel, ' Reference Name:'), (ir, 0), (1, 1), LEFT, 0)
+        sizer.Add(self.wid_reffilename,               (ir, 1), (1, 2), LEFT, 0)
+        sizer.Add(SimpleText(panel, ' Group Name:'),   (ir, 3), (1, 1), RIGHT, 0)
+        sizer.Add(self.wid_refgroupname,              (ir, 4), (1, 2), LEFT, 0)
         ir +=1
         sizer.Add(self.message,                     (ir, 1), (1, 4), LEFT, 0)
 
@@ -515,13 +583,21 @@ class ColumnDataFileFrame(wx.Frame) :
         self.nb.SetActiveTabTextColour(wx.Colour(80,0,0))
 
         self.plotpanel = PlotPanel(self, messenger=self.plot_messages)
+        try:
+            plotopts = self._larch.symtable._sys.wx.plotopts
+            self.plotpanel.conf.set_theme(plotopts['theme'])
+            self.plotpanel.conf.enable_grid(plotopts['show_grid'])
+        except:
+            pass
+
+
         self.plotpanel.SetMinSize((200, 200))
         textpanel = wx.Panel(self)
         ftext = wx.TextCtrl(textpanel, style=wx.TE_MULTILINE|wx.TE_READONLY,
                                size=(400, 250))
 
         ftext.SetValue(group.text)
-        ftext.SetFont(Font(10))
+        ftext.SetFont(Font(11))
 
         textsizer = wx.BoxSizer(wx.VERTICAL)
         textsizer.Add(ftext, 1, LEFT|wx.GROW, 1)
@@ -544,27 +620,12 @@ class ColumnDataFileFrame(wx.Frame) :
         self.set_energy_units()
         self.Show()
         self.Raise()
-        self.onUpdate(self)
+        self.onUpdate()
 
     def read_column_file(self, path):
         """read column file, generally as initial read"""
         parent, filename = os.path.split(path)
-        with open(path, 'rb') as fh:
-            text = fh.read().decode('utf-8').replace('\r\n', '\n').replace('\r', '\n')
-        lines = text.split('\n')
-        text = ''.join(lines)
-
-        line1 = lines[0].lower()
-
-        reader = 'read_ascii'
-        if 'epics scan' in line1:
-            reader = 'read_gsescan'
-        if 'xdi' in line1:
-            reader = 'read_xdi'
-            if 'epics stepscan file' in line1 :
-                reader = 'read_gsexdi'
-        elif ("#s" in line1) or ("#f" in line1):
-            reader = 'read_specfile'
+        reader, text = guess_filereader(path, return_text=True)
 
         if reader in ('read_xdi', 'read_gsexdi'):
             # first check for Nans and Infs
@@ -584,10 +645,11 @@ class ColumnDataFileFrame(wx.Frame) :
                 'has infs' in nan_result.message):
                 reader = 'read_ascii'
 
-        tmpname = '_tmp_file_'
+        tmpname = '_tmpfile_'
         read_cmd = "%s = %s('%s')" % (tmpname, reader, path)
         self.reader = reader
         _larch = self._larch
+
         if (not isinstance(_larch, larch.Interpreter) and
             hasattr(_larch, '_larch')):
             _larch = _larch._larch
@@ -595,7 +657,16 @@ class ColumnDataFileFrame(wx.Frame) :
             _larch.eval(read_cmd, add_history=True)
         except:
             pass
-        if _larch.error:
+        if len(_larch.error) > 0 and reader in ('read_xdi', 'read_gsexdi'):
+            read_cmd = "%s = %s('%s')" % (tmpname, 'read_ascii', path)
+            try:
+                _larch.eval(read_cmd, add_history=True)
+            except:
+                pass
+            if len(_larch.error) == 0:
+                self.reader = 'read_ascii'
+
+        if len(_larch.error) > 0:
             msg = ["Error trying to read '%s':" % path, ""]
             for err in _larch.error:
                 exc_name, errmsg = err.get_error()
@@ -604,8 +675,8 @@ class ColumnDataFileFrame(wx.Frame) :
             title = "Cannot read %s" % path
             r = Popup(self.parent, "\n".join(msg), title)
             return None
-        group = _larch.symtable.get_symbol(tmpname)
-        # _larch.symtable.del_symbol(tmpname)
+        group = deepcopy(_larch.symtable.get_symbol(tmpname))
+        _larch.symtable.del_symbol(tmpname)
 
         group.text = text
         group.path = path
@@ -666,11 +737,10 @@ class ColumnDataFileFrame(wx.Frame) :
     def onOK(self, event=None):
         """ build arrays according to selection """
         user_filename = self.wid_filename.GetValue()
-        if self.wid_groupname is not None:
-            groupname = fix_varname(self.wid_groupname.GetValue())
+        groupname = fix_varname(self.wid_groupname.GetValue())
 
         en_units = self.en_units.GetStringSelection()
-        dspace   = float(self.monod_val.GetValue())
+        monod    = float(self.monod_val.GetValue())
         xarr     = self.xarr.GetStringSelection()
         yarr1    = self.yarr1.GetStringSelection()
         yarr2    = self.yarr2.GetStringSelection()
@@ -709,8 +779,8 @@ class ColumnDataFileFrame(wx.Frame) :
 
         expr = self.expressions['xdat'].replace('%s', '{group:s}')
         if en_units.startswith('deg'):
-            buff.append(f"mono_dspace = {dspace:.9f}")
-            buff.append(f"{{group}}.xdat = PLANCK_HC/(2*mono_dspace*sin(DEG2RAD*({expr:s})))")
+            buff.append(f"monod = {monod:.9f}")
+            buff.append(f"{{group}}.xdat = PLANCK_HC/(2*monod*sin(DEG2RAD*({expr:s})))")
         elif en_units.startswith('keV'):
             buff.append(f"{{group}}.xdat = 1000.0*{expr:s}")
         else:
@@ -722,13 +792,53 @@ class ColumnDataFileFrame(wx.Frame) :
 
         if getattr(self.workgroup, 'datatype', 'raw') == 'xas':
             if self.reader == 'read_gsescan':
-                buff.append("{group}.energy = {group}.x")
-            else:
-                buff.append("{group}.energy = {group}.xdat")
+                buff.append("{group}.xdat = {group}.x")
+            buff.append("{group}.energy = {group}.xdat")
             buff.append("{group}.mu = {group}.ydat")
             buff.append("sort_xafs({group}, overwrite=True, fix_repeats=True)")
         else:
             buff.append("{group}.scale = 1./({group}.ydat.ptp()+1.e-16)")
+
+        array_desc = {}
+        array_desc['xdat'] = self.workgroup.plot_xlabel
+        array_desc['ydat'] = self.workgroup.plot_ylabel
+        array_desc['yerr'] = self.expressions['yerr'].replace('%s', '{group:s}')
+
+        ref_filename = None
+        ref_groupname = None
+        if self.has_yref.IsChecked():
+            yrname1 = self.yref1.GetStringSelection().strip()
+            yrname2 = self.yref2.GetStringSelection().strip()
+            iry1    = self.yref1.GetSelection()
+            iry2    = self.yref2.GetSelection()
+            yrpop   = self.yrpop.GetStringSelection().strip()
+            yrop    = self.yop.GetStringSelection().strip()
+
+            ref_filename = self.wid_reffilename.GetValue()
+            ref_groupname = fix_varname(self.wid_refgroupname.GetValue())
+            
+            buff.append("# reference group")           
+            buff.append("{refgroup} = %s" % read_cmd)
+            buff.append("{refgroup}.path = '{path}'")
+            buff.append("{refgroup}.is_frozen = False")
+            buff.append("{refgroup}.datatype = 'xas'")
+            buff.append("{refgroup}.plot_xlabel = 'energy'")
+            buff.append("{refgroup}.plot_ylabel = '%s'" % self.workgroup.yrlabel)
+            buff.append("{refgroup}.xdat =1.0*{group}.xdat")
+            buff.append("{refgroup}.energy = {refgroup}.xdat")
+
+            refexpr = self.expressions['yref'].replace('%s', '{group:s}')
+            array_desc['yref'] = self.workgroup.yrlabel
+
+            buff.append("{group}.energy_ref = '%s'" % (ref_groupname))
+            buff.append("{refgroup}.energy_ref = '%s'" % (ref_groupname))
+            buff.append("{refgroup}.ydat =  %s" % refexpr)
+            buff.append("{refgroup}.mu = {refgroup}.ydat")
+            buff.append("sort_xafs({refgroup}, overwrite=True, fix_repeats=True)")
+            buff.append("# end reference group")
+        else:
+            buff.append("{group}.energy_ref = '%s'" % (groupname))
+
         script = "\n".join(buff)
 
         self.array_sel['xarr'] = xarr
@@ -739,12 +849,16 @@ class ColumnDataFileFrame(wx.Frame) :
         self.array_sel['yerror'] = yerr_op
         self.array_sel['yerr_val'] = yerr_val
         self.array_sel['yerr_arr'] = yerr_arr
-        self.array_sel['monod'] = dspace
+        self.array_sel['monod'] = monod
         self.array_sel['en_units'] = en_units
 
         if self.read_ok_cb is not None:
             self.read_ok_cb(script, self.path, groupname=groupname,
                             filename=user_filename,
+                            ref_groupname=ref_groupname,
+                            ref_filename=ref_filename,
+                            array_desc=array_desc,
+                            extra_sums=self.extra_sums,
                             array_sel=self.array_sel)
 
         for f in self.subframes.values():
@@ -773,6 +887,16 @@ class ColumnDataFileFrame(wx.Frame) :
             self.yerr_arr.Enable()
         self.onUpdate()
 
+    def onYrefCheck(self, evt=None, has_yref=False):
+        if evt is not None:
+            has_yref = evt.IsChecked()
+        self.yref1.Enable(has_yref)
+        self.yref2.Enable(has_yref)
+        self.yrpop.Enable(has_yref)
+        self.yrop.Enable(has_yref)
+        self.wid_reffilename.Enable(has_yref)
+        self.wid_refgroupname.Enable(has_yref)
+
 
     def onXSelect(self, evt=None):
         ix  = self.xarr.GetSelection()
@@ -785,6 +909,7 @@ class ColumnDataFileFrame(wx.Frame) :
             workgroup.xdat = 1.0*np.arange(npts)
         else:
             workgroup.xdat = 1.0*rdata[ix, :]
+        self.onUpdate()
 
         self.monod_val.Disable()
         if self.datatype.GetStringSelection().strip().lower() == 'raw':
@@ -821,11 +946,8 @@ class ColumnDataFileFrame(wx.Frame) :
             elif eguess.startswith('keV'):
                 self.en_units.SetStringSelection('keV')
 
-    def onUpdate(self, value=None, evt=None):
+    def onUpdate(self, evt=None, **kws):
         """column selections changed calc xdat and ydat"""
-        # dtcorr = self.dtcorr.IsChecked()
-
-        dtcorr = False
         rawgroup = self.initgroup
         workgroup = self.workgroup
         rdata = self.initgroup.data
@@ -833,9 +955,13 @@ class ColumnDataFileFrame(wx.Frame) :
         ix  = self.xarr.GetSelection()
         xname = self.xarr.GetStringSelection()
 
-        exprs = dict(xdat=None, ydat=None, yerr=None)
+        exprs = dict(xdat=None, ydat=None, yerr=None, yref=None)
+
+        try:
+            ncol, npts = rdata.shape
+        except:
+            return
 
-        ncol, npts = rdata.shape
         workgroup.index = 1.0*np.arange(npts)
         if xname.startswith('_index') or ix >= ncol:
             workgroup.xdat = 1.0*np.arange(npts)
@@ -851,9 +977,9 @@ class ColumnDataFileFrame(wx.Frame) :
 
         xlabel = xname
         en_units = self.en_units.GetStringSelection()
+        monod = float(self.monod_val.GetValue())
         if en_units.startswith('deg'):
-            dspace = float(self.monod_val.GetValue())
-            workgroup.xdat = PLANCK_HC/(2*dspace*np.sin(DEG2RAD*workgroup.xdat))
+            workgroup.xdat = PLANCK_HC/(2*monod*np.sin(DEG2RAD*workgroup.xdat))
             xlabel = xname + ' (eV)'
         elif en_units.startswith('keV'):
             workgroup.xdat *= 1000.0
@@ -862,6 +988,8 @@ class ColumnDataFileFrame(wx.Frame) :
 
         def pre_op(opwid, arr):
             opstr = opwid.GetStringSelection().strip()
+            if opstr == '-':
+                return '', opstr, -arr
             suf = ''
             if opstr in ('-log(', 'log('):
                 suf = ')'
@@ -921,12 +1049,16 @@ class ColumnDataFileFrame(wx.Frame) :
         ysuf, ypop, workgroup.ydat = pre_op(self.ypop, workgroup.ydat)
         self.ysuf.SetLabel(ysuf)
         exprs['ydat'] = '%s%s%s' % (ypop, exprs['ydat'], ysuf)
+        ylabel = '%s%s%s' % (ypop, ylabel,ysuf)
 
+        # error
+        yerr_arr = self.yerr_arr.GetStringSelection()
         yerr_op = self.yerr_op.GetStringSelection().lower()
         exprs['yerr'] = '1'
+        yerr_val = self.yerr_val.GetValue()
         if yerr_op.startswith('const'):
-            yerr = self.yerr_val.GetValue()
-            exprs['yerr'] = '%f' % yerr
+            yerr = yerr_val
+            exprs['yerr'] = '%f' % yerr_val
         elif yerr_op.startswith('array'):
             iyerr = self.yerr_arr.GetSelection()
             yerr = rdata[iyerr, :]
@@ -935,11 +1067,67 @@ class ColumnDataFileFrame(wx.Frame) :
             yerr = np.sqrt(workgroup.ydat)
             exprs['yerr'] = 'sqrt(%s.ydat)'
 
+        # ref
+        has_yref = self.has_yref.IsChecked()
+        yrname1  = self.yref1.GetStringSelection().strip()
+        yrname2  = self.yref2.GetStringSelection().strip()
+        iry1    = self.yref1.GetSelection()
+        iry2    = self.yref2.GetSelection()
+        yrpop  = self.yrpop.GetStringSelection().strip()
+        yrop   = self.yop.GetStringSelection().strip()
+        yrlabel = ''
+        if has_yref:
+            yrlabel = yrname1
+            if len(yrname2) == 0:
+                yrname2 = '1.0'
+            else:
+                yrlabel = "%s%s%s" % (yrlabel, yrop, yrname2)
+
+            if yrname1 == '0.0':
+                yrarr1 = np.zeros(npts)*1.0
+                yrexpr1 = 'zeros(%i)' % npts
+            elif len(yrname1) == 0 or yrname1 == '1.0' or iry1 >= ncol:
+                yrarr1 = np.ones(npts)*1.0
+                yrexpr1 = 'ones(%i)' % npts
+            else:
+                yrarr1 = rdata[iry1, :]
+                yrexpr1 = '%%s.data[%i, : ]' % iry1
+
+            if yrname2 == '0.0':
+                yrarr2 = np.zeros(npts)*1.0
+                yrexpr2 = '0.0'
+            elif len(yrname2) == 0 or yrname2 == '1.0' or iry2 >= ncol:
+                yrarr2 = np.ones(npts)*1.0
+                yrexpr2 = '1.0'
+            else:
+                yrarr2 = rdata[iry2, :]
+                yrexpr2 = '%%s.data[%i, : ]' % iry2
+
+            workgroup.yref = yrarr1
+            exprs['yref'] = yrexpr1
+            if yrop in ('+', '-', '*', '/'):
+                exprs['yref'] = "%s %s %s" % (yrexpr1, yop, yrexpr2)
+                if yrop == '+':
+                    workgroup.yref = yrarr1.__add__(yrarr2)
+                elif yrop == '-':
+                    workgroup.yref = yrarr1.__sub__(yrarr2)
+                elif yrop == '*':
+                    workgroup.yref = yrarr1.__mul__(yarr2)
+                elif yrop == '/':
+                    workgroup.yref = yrarr1.__truediv__(yrarr2)
+
+            yrsuf, yprop, workgroup.yref = pre_op(self.yrpop, workgroup.yref)
+            exprs['yref'] = '%s%s%s' % (yrpop, exprs['yref'], yrsuf)
+            yrlabel = '%s%s%s' % (yrpop, yrlabel, yrsuf)
+            workgroup.yrlabel = yrlabel
 
         self.expressions = exprs
-        self.array_sel = {'xarr': xname,
-                          'ypop': ypop, 'yop': yop,
-                          'yarr1': yname1, 'yarr2': yname2}
+        self.array_sel = dict(xarr=xname, ypop=ypop, yop=yop, yarr1=yname1,
+                              yarr2=yname2, monod=monod, en_units=en_units,
+                              yerror=yerr_op, yerr_val=yerr_val,
+                              yerr_arr=yerr_arr, yrpop=yrpop, yrop=yrop,
+                              yref1=yrname1, yref2=yrname2,
+                              has_yref=has_yref)
 
         try:
             npts = min(len(workgroup.xdat), len(workgroup.ydat))
@@ -961,6 +1149,7 @@ class ColumnDataFileFrame(wx.Frame) :
         workgroup.npts        = npts
         workgroup.plot_xlabel = xlabel
         workgroup.plot_ylabel = ylabel
+        workgroup.plot_yrlabel = yrlabel
         workgroup.xdat        = np.array(workgroup.xdat[:npts])
         workgroup.ydat        = np.array(workgroup.ydat[:npts])
         workgroup.y           = workgroup.ydat
@@ -975,9 +1164,15 @@ class ColumnDataFileFrame(wx.Frame) :
         path, fname = os.path.split(workgroup.filename)
         popts = dict(marker='o', markersize=4, linewidth=1.5,
                      title=fname, ylabel=ylabel, xlabel=xlabel,
-                     label="%s: %s" % (fname, workgroup.plot_ylabel))
+                     label=ylabel)
 
         self.plotpanel.plot(workgroup.xdat, workgroup.ydat, **popts)
+        if has_yref:
+            self.plotpanel.oplot(workgroup.xdat, workgroup.yref,
+                                 y2label=yrlabel,
+                                 linewidth=2.0, color='#E08070',
+                                 label=yrlabel, zorder=-40, side='right')
+
 
         for i in range(self.nb.GetPageCount()):
             if 'plot' in self.nb.GetPageText(i).lower():
