@@ -10,7 +10,7 @@ import wx.dataview as dv
 
 import larch
 from larch.site_config import user_larchdir
-from larch.utils import unixpath
+from larch.utils import unixpath, read_textfile
 from larch.wxlib import (GridPanel, GUIColors, Button, pack, SimpleText,
                          FileOpen, FileSave, Font, LEFT, FRAMESTYLE,
                          FONTSIZE, MenuItem, EditableListBox, OkCancel,
@@ -20,8 +20,8 @@ from larch.wxlib import (GridPanel, GUIColors, Button, pack, SimpleText,
 from larch.xafs import get_feff_pathinfo
 from larch.xray import atomic_symbols
 
-ATSYMS = ['< All >'] + atomic_symbols
-EDGES  = ['< All >', 'K', 'L3', 'L2', 'L1', 'M5']
+ATSYMS = ['< All Atoms>'] + atomic_symbols
+EDGES  = ['< All Edges>', 'K', 'L3', 'L2', 'L1', 'M5']
 
 
 LEFT = LEFT|wx.ALL
@@ -51,7 +51,7 @@ class FeffPathsModel(dv.DataViewIndexListModel):
         else:
             for fp in self.feffpaths:
                 row = [fp.filename, '%.4f' % fp.reff,
-                       '%.0f' % fp.nleg, '%.0f' % fp.degeneracy,
+                       '%.0f' % fp.nleg, '%.0f' % fp.degen,
                        '%.3f' % fp.cwratio]
                 use = False
                 if self.with_use:
@@ -154,7 +154,7 @@ class FeffResultsPanel(wx.Panel):
         self.report_frame = None
 
         self.dvc = dv.DataViewCtrl(self, style=DVSTYLE)
-        self.dvc.SetMinSize((600, 350))
+        self.dvc.SetMinSize((650, 350))
 
         self.model = FeffPathsModel(None, with_use=callable(path_importer))
         self.dvc.AssociateModel(self.model)
@@ -167,13 +167,14 @@ class FeffResultsPanel(wx.Panel):
         bkws = dict(size=(175, -1))
         btn_header = Button(panel, "Show Full Header", action=self.onShowHeader, **bkws)
         btn_feffinp = Button(panel, "Show Feff.inp",   action=self.onShowFeffInp, **bkws)
+        btn_geom = Button(panel, "Show Path Geometries", action=self.onShowGeom, **bkws)
 
         if callable(self.path_importer):
             btn_import = Button(panel, "Import Paths",     action=self.onImportPath, **bkws)
             btn_above  = Button(panel, "Select All Above Current", action=self.onSelAbove,  **bkws)
             btn_none   = Button(panel, "Select None",      action=self.onSelNone, **bkws)
 
-        opts = dict(size=(400, -1), style=LEFT)
+        opts = dict(size=(475, -1), style=LEFT)
         self.feff_folder = SimpleText(panel, '',  **opts)
         self.feff_datetime = SimpleText(panel, '',**opts)
         self.feff_header = [SimpleText(panel, '', **opts),
@@ -213,6 +214,7 @@ class FeffResultsPanel(wx.Panel):
         ir += 1
         sizer.Add(btn_header,      (ir, 0), (1, 2),  LEFT, 2)
         sizer.Add(btn_feffinp,     (ir, 2), (1, 2),  LEFT, 2)
+        sizer.Add(btn_geom,        (ir, 4), (1, 2),  LEFT, 2)
 
         if callable(self.path_importer):
             ir += 1
@@ -225,14 +227,14 @@ class FeffResultsPanel(wx.Panel):
 
         pack(panel, sizer)
 
-        columns = [('Feff File',   90, 'text'),
-                   ('R (\u212B)',  65, 'text'),
-                   ('# legs',      60, 'text'),
-                   ('# paths',     65, 'text'),
-                   ('Importance',  90, 'text')]
+        columns = [('Feff File',   85, 'text'),
+                   ('R (\u212B)',  55, 'text'),
+                   ('# legs',      55, 'text'),
+                   ('# paths',     55, 'text'),
+                   ('Importance',  80, 'text')]
         if callable(self.path_importer):
             columns.append(('Use',     50, 'bool'))
-        columns.append(('Geometry',   200, 'text'))
+        columns.append(('Geometry',   175, 'text'))
 
         for icol, dat in enumerate(columns):
              label, width, dtype = dat
@@ -273,20 +275,44 @@ class FeffResultsPanel(wx.Panel):
             self.show_report(self.feffresult.header,
                              title=f'Header for {self.feffresult.folder:s}',
                              default_filename=f'{self.feffresult.folder:s}_header.txt')
-            
+
+    def onShowGeom(self, event=None):
+        if self.feffresult is None:
+            return
+        show = False
+        out = []
+        for data in self.model.data:
+            if data[5]:
+                show = True
+                out.append(f'### {self.feffresult.folder:s}/{data[0]:s} ###')
+                out.append('#Atom IPOT     X         Y         Z         Beta      Eta      Length')
+                fname = data[0]
+
+                for fp in self.feffresult.paths:
+                    if fname == fp.filename:
+                        for i, px in enumerate(fp.geometry):
+                            at, ipot, r, x, y, z, beta, eta = px
+                            if i == 0: r = 0
+                            t = f'{at:4s}  {ipot:3d}  {x:9.4f} {y:9.4f} {z:9.4f} {beta:9.4f} {eta:9.4f} {r:9.4f}'
+                            out.append(t)
+        if show:
+            out = '\n'.join(out)
+            self.show_report(out, title=f'Path Geometries for {self.feffresult.folder:s}',
+                             default_filename=f'{self.feffresult.folder:s}_paths.dat')
+
+
+
     def onShowFeffInp(self, event=None):
         if self.feffresult is not None:
             text = None
             fname = unixpath(os.path.join(self.feffresult.folder, 'feff.inp'))
             if os.path.exists(fname):
-                with open(fname, 'r') as fh:
-                    text = fh.read()
+                text = read_textfile(fname)
             else:
                 fname = unixpath(os.path.join(user_larchdir, 'feff',
                                               self.feffresult.folder, 'feff.inp'))
                 if os.path.exists(fname):
-                    with open(fname, 'r') as fh:
-                        text = fh.read()
+                    text = read_textfile(fname)
             if text is not None:
                 self.show_report(text, title=f'Feff.inp for {self.feffresult.folder:s}',
                                  default_filename=f'{self.feffresult.folder:s}_feff.inp',
@@ -305,21 +331,23 @@ class FeffResultsPanel(wx.Panel):
             self.report_frame = ReportFrame(parent=self,
                                             text=text, title=title,
                                             default_filename=default_filename,
-                                            wildcard=wildcard)                                                
+                                            wildcard=wildcard)
 
 
     def onImportPath(self, event=None):
-        folder  = self.feffresult.folder
+        folder = self.feffresult.folder
         _, fname = os.path.split(folder)
         for data in self.model.data:
             if data[5]:
-                self.path_importer(unixpath(os.path.join(folder, data[0])),
-                                   self.feffresult)
+                fname = data[0]
+                fullpath = unixpath(os.path.join(folder, fname))
+                for pathinfo in self.feffresult.paths:
+                    if pathinfo.filename == fname:
+                        self.path_importer(fullpath, pathinfo)
+                        break
 
         self.onSelNone()
-        # print(' on import xasmain = ', self.xasmain)
-        if self.xasmain is not None:
-            self.xasmain.nb.SetSelection(self.xasmain.get_nbpage('feffit')[0])
+
 
     def set_feffresult(self, feffresult):
         self.feffresult = feffresult
@@ -330,26 +358,25 @@ class FeffResultsPanel(wx.Panel):
         for i, text in enumerate(feffresult.header.split('\n')[:nhead]):
             self.feff_header[i].SetLabel(text)
         self.model.set_data(feffresult.paths)
+        self.dvc.EnsureVisible(self.model.GetItem(0))
+        self.dvc.SetCurrentItem(self.dvc.GetTopItem())
 
 
 class FeffResultsFrame(wx.Frame):
     """ present Feff results """
-    def __init__(self,  parent=None, feffresult=None, xasmain=None, _larch=None):
+    def __init__(self,  parent=None, feffresult=None, xasmain=None,
+                 path_importer=None, _larch=None):
         wx.Frame.__init__(self, parent, -1, size=(850, 650), style=FRAMESTYLE)
 
         title = "Manage Feff calculation results"
         self.larch = _larch
         if _larch is None:
             self.larch = larch.Interpreter()
-        self.larch.eval("# started Feff results browser\n")
-        self.larch.eval("if not hasattr('_main', '_feffruns'): _feffruns = {}")
+        # self.larch.eval("# started Feff results browser\n")
+        # self.larch.eval("if not hasattr('_sys', '_feffruns'): _sys._feffruns = {}")
+        if not hasattr(self.larch.symtable._sys, '_feffruns'):
+            self.larch.symtable._sys._feffruns = {}
         self.parent = parent
-        path_importer = None
-        if parent is not None:
-            try:
-                path_importer = parent.get_nbpage('feffit')[1].add_path
-            except:
-                pass
 
         path = unixpath(os.path.join(user_larchdir, 'feff'))
         if not os.path.exists(path):
@@ -357,7 +384,7 @@ class FeffResultsFrame(wx.Frame):
         self.feff_folder = path
 
         self.SetTitle(title)
-        self.SetSize((850, 650))
+        self.SetSize((925, 650))
         self.SetFont(Font(FONTSIZE))
         self.createMenus()
 
@@ -396,16 +423,16 @@ class FeffResultsFrame(wx.Frame):
         pack(lpanel, lsizer)
 
         # right hand side
-        panel = wx.Panel(splitter)
+        panel = wx.Panel(splitter) ## scrolled.ScrolledPanel(splitter)
         wids = self.wids = {}
         toprow = wx.Panel(panel)
 
-        wids['central_atom'] = Choice(toprow, choices=ATSYMS, size=(100, -1),
+        wids['central_atom'] = Choice(toprow, choices=ATSYMS, size=(125, -1),
                                       action=self.onCentralAtom)
-        wids['edge']         = Choice(toprow, choices=EDGES, size=(100, -1),
+        wids['edge']         = Choice(toprow, choices=EDGES, size=(125, -1),
                                       action=self.onAbsorbingEdge)
 
-        flabel = SimpleText(toprow, 'Filter Calculations by Element and Edge:', size=(275, -1))
+        flabel = SimpleText(toprow, 'Filter Calculations by Element and Edge:', size=(200, -1))
         tsizer = wx.BoxSizer(wx.HORIZONTAL)
         tsizer.Add(flabel,               1, LEFT, 2)
         tsizer.Add(wids['central_atom'], 0, LEFT|wx.GROW, 2)
@@ -414,11 +441,12 @@ class FeffResultsFrame(wx.Frame):
 
         sizer = wx.BoxSizer(wx.VERTICAL)
         self.feff_panel = FeffResultsPanel(panel, path_importer=path_importer,
-                                           xasmain=xasmain, _larch=_larch)
+                                           _larch=_larch)
         sizer.Add(toprow, 0, LEFT|wx.GROW|wx.ALL, 2)
         sizer.Add(HLine(panel, size=(550, 2)), 0,  LEFT|wx.GROW|wx.ALL, 2)
         sizer.Add(self.feff_panel, 1, LEFT|wx.GROW|wx.ALL, 2)
         pack(panel, sizer)
+        # panel.SetupScrolling()
         splitter.SplitVertically(lpanel, panel, 1)
         self.Show()
         wx.CallAfter(self.onSearch)
@@ -428,6 +456,7 @@ class FeffResultsFrame(wx.Frame):
         if fr is not None:
             self.feff_panel.set_feffresult(fr)
 
+
     def onSearch(self, event=None):
         catom = self.wids['central_atom'].GetStringSelection()
         edge = self.wids['edge'].GetStringSelection()
@@ -436,19 +465,23 @@ class FeffResultsFrame(wx.Frame):
 
         self.fefflist.Clear()
         self.feffruns = {}
-        self.larch.eval("## gathering results:\n")
         flist = os.listdir(self.feff_folder)
         flist = sorted(flist, key=lambda t: -os.stat(unixpath(os.path.join(self.feff_folder, t))).st_mtime)
-
+        _feffruns = self.larch.symtable._sys._feffruns
         for path in flist:
             fullpath = unixpath(os.path.join(self.feff_folder, path))
             if os.path.isdir(fullpath):
                 try:
-                    self.larch.eval(f"_feffruns['{path:s}'] = get_feff_pathinfo('{fullpath:s}')")
-                    self.feffruns[path] = thisrun = self.larch.symtable._feffruns[path]
-                    if ((all_catoms or (thisrun.absorber == catom)) and
-                        (all_edges  or (thisrun.edge == edge))):
-                        self.fefflist.Append(path)
+                    _feffruns[path] = thisrun = get_feff_pathinfo(fullpath)
+                    if ((len(thisrun.paths) < 1) or
+                        (len(thisrun.ipots) < 1) or thisrun.shell is None):
+
+                        self.larch.symtable._sys._feffruns.pop(path)
+                    else:
+                        self.feffruns[path] = thisrun
+                        if ((all_catoms or (thisrun.absorber == catom)) and
+                            (all_edges  or (thisrun.shell == edge))):
+                            self.fefflist.Append(path)
                 except:
                     print(f"could not read Feff calculation from '{path}'")
 
@@ -554,5 +587,5 @@ class FeffResultsBrowserApp(LarchWxApp):
         return True
 
 if __name__ == '__main__':
-    dat = None 
+    dat = None
     FeffResultsBrowserApp(dat).MainLoop()
