@@ -2,7 +2,6 @@ import sys
 import wx
 from wx.lib.agw import floatspin as fspin
 
-is_wxPhoenix = 'phoenix' in wx.PlatformInfo
 is_gtk3  = 'gtk3' in wx.PlatformInfo
 
 from wxutils.icons import get_icon
@@ -23,9 +22,6 @@ def FloatSpin(parent, value=0, action=None, tooltip=None,
         maxval = kws.pop('max_val', None)
         minval = kws.pop('min_val', None)
         fmt = "%%%df" % digits
-        fs = wx.SpinCtrlDouble(parent, -1, value=fmt % value,
-                               size=(size[0]+25, size[1]),
-                               inc=increment, **kws)
         fs = wx.SpinCtrlDouble(parent, -1, value=fmt % value,
                                size=(size[0]+25, size[1]),
                                inc=increment, **kws)
@@ -45,24 +41,18 @@ def FloatSpin(parent, value=0, action=None, tooltip=None,
         if action is not None:
             fs.Bind(fspin.EVT_FLOATSPIN, action)
     if tooltip is not None:
-        if is_wxPhoenix:
-            fs.SetToolTip(tooltip)
-        else:
-            fs.SetToolTipString(tooltip)
+        fs.SetToolTip(tooltip)
     return fs
 
 def FloatSpinWithPin(parent, value=0, pin_action=None,
-                     tooltip='use last point selected from plot', **kws):
+                     tooltip='select point from plot', **kws):
     """create a FloatSpin with Pin button with action"""
     fspin = FloatSpin(parent, value=value, **kws)
     bmbtn = wx.BitmapButton(parent, id=-1, bitmap=get_icon('pin'),
                             size=(25, 25))
     if pin_action is not None:
         parent.Bind(wx.EVT_BUTTON, pin_action, bmbtn)
-    if is_wxPhoenix:
-        bmbtn.SetToolTip(tooltip)
-    else:
-        bmbtn.SetToolTipString(tooltip)
+    bmbtn.SetToolTip(tooltip)
     return fspin, bmbtn
 
 class NumericCombo(wx.ComboBox):
