@@ -9,13 +9,54 @@ except:
 
 import time
 import os
+import locale
 
 from ..larchlib import ensuremod
 from .larchfilling import Filling
-
+from ..utils import get_cwd
 
 DEF_CHOICES = [('All Files', '*.*')]
 
+class LarchWxApp(wx.App, wx.lib.mixins.inspection.InspectionMixin):
+    """wrapper for wx apps, with the following arguments and features:
+
+    _larch (None or Interpreter):   instance of Larch Interpreter [None]
+    version_info (None or string):  larch version to check for updates [None]
+    with_inspect (bool):            use wx inspection tool for debugging [False]
+    with_c_locale (bool):           whether to force C locale [True]
+
+    """
+    def __init__(self, _larch=None, version_info=None, with_inspect=False,
+                 with_c_locale=True, **kws):
+        self._larch = _larch
+        self.version_info = version_info
+        self.with_inspect = with_inspect
+        self.with_c_locale = with_c_locale
+        wx.App.__init__(self, **kws)
+
+    def OnInit(self):
+        self.createApp()
+        if self.with_inspect:
+            self.ShowInspectionTool()
+        return True
+
+    def createApp(self):
+        return True
+
+    def InitLocale(self):
+        """over-ride wxPython default initial locale"""
+        if self.with_c_locale:
+            self._initial_locale = None
+            locale.setlocale(locale.LC_ALL, 'C')
+        else:
+            lang, enc = locale.getdefaultlocale()
+            self._initial_locale = wx.Locale(lang, lang[:2], lang)
+            locale.setlocale(locale.LC_ALL, lang)
+
+    def run(self):
+        self.MainLoop()
+
+
 def SafeWxCall(fcn):
     """decorator to wrap function in a wx.CallAfter() so that
     calls can be made in a separate thread, and asynchronously.
@@ -31,6 +72,20 @@ def SafeWxCall(fcn):
     wrapper.__dict__.update(fcn.__dict__)
     return wrapper
 
+def show_wxsizes(obj):
+    """recursively show sizes of wxPython objects --
+    useful for avoiding size<1 errors"""
+    for child in obj.GetChildren():
+        try:
+            csize = child.GetSize()
+            if csize[0] < 1 or csize[1] < 1:
+                print(child, csize)
+        except:
+            pass
+        try:
+            show_wxsizes(child)
+        except:
+            pass
 
 def wx_update(_larch=None, **kws):
     """force an update of wxPython windows"""
@@ -81,13 +136,13 @@ def gcd(wxparent=None, _larch=None, **kws):
         _larch.raise_exception(None, msg='wx not supported')
 
     dlg = wx.DirDialog(None, 'Choose Directory',
-                       defaultPath = os.getcwd(),
+                       defaultPath = get_cwd(),
                        style = wx.DD_DEFAULT_STYLE)
 
     if dlg.ShowModal() == wx.ID_OK:
         os.chdir(dlg.GetPath())
     dlg.Destroy()
-    return os.getcwd()
+    return get_cwd()
 
 
 class DataBrowserFrame(wx.Frame):
@@ -182,7 +237,7 @@ def fileprompt(mode='open', multi=True, message = None,
         parent = parent.GetTopWindow()
     timer = wxLarchTimer(parent, _larch)
     dlg = wx.FileDialog(parent=timer, message=message,
-                        defaultDir=os.getcwd(),
+                        defaultDir=get_cwd(),
                         defaultFile=fname,
                         wildcard =wildcard,
                         style=style)
@@ -195,59 +250,3 @@ def fileprompt(mode='open', multi=True, message = None,
     dlg.Destroy()
     timer.Destroy()
     return path
-
-
-class EditableListBox(wx.ListBox):
-    """
-    A ListBox with pop-up menu to arrange order of
-    items and remove items from list
-    supply select_action for EVT_LISTBOX selection action
-    """
-    def __init__(self, parent, select_action, right_click=True,
-                 remove_action=None, **kws):
-        wx.ListBox.__init__(self, parent, **kws)
-        self.SetBackgroundColour(wx.Colour(240, 240, 230))
-        self.SetForegroundColour(wx.Colour(10, 10, 20))
-
-        self.SetOwnBackgroundColour(wx.Colour(240, 240, 230))
-        self.SetOwnForegroundColour(wx.Colour(10, 10, 20))
-
-        self.Bind(wx.EVT_LISTBOX,  select_action)
-        self.remove_action = remove_action
-        if right_click:
-            self.Bind(wx.EVT_RIGHT_DOWN, self.onRightClick)
-
-    def onRightClick(self, evt=None):
-        menu = wx.Menu()
-        self.pmenu_labels = {}
-        for menulabel in ('Move up', 'Move down', 'Move to top',
-                          'Move to bottom', 'Remove from list'):
-            item = menu.Append(-1, menulabel)
-            self.pmenu_labels[item.Id] = menulabel
-            self.Bind(wx.EVT_MENU, self.onRightEvent, item)
-        self.PopupMenu(menu)
-        menu.Destroy()
-
-    def onRightEvent(self, event=None):
-        idx = self.GetSelection()
-        mlabel = self.pmenu_labels.get(event.GetId(), None)
-        if idx < 0 or mlabel is None:
-            return
-
-        names = self.GetItems()
-        this  = names.pop(idx)
-
-        if mlabel == 'Move up' and idx > 0:
-            names.insert(idx-1, this)
-        elif mlabel == 'Move down' and idx < len(names):
-            names.insert(idx+1, this)
-        elif mlabel == 'Move to top':
-            names.insert(0, this)
-        elif mlabel == 'Move to bottom':
-            names.append(this)
-        elif mlabel == 'Remove from list' and self.remove_action is not None:
-            self.remove_action(this)
-
-        self.Clear()
-        for name in names:
-            self.Append(name)
