@@ -3,8 +3,11 @@
 from __future__ import print_function
 import sys
 import os
+import time
 from functools import partial
 import wx
+import wx.lib.mixins.inspection
+
 import numpy
 import scipy
 
@@ -17,22 +20,30 @@ from .readlinetextctrl import ReadlineTextCtrl
 from .larchfilling import Filling
 from .columnframe import ColumnDataFileFrame
 from .athena_importer import AthenaImporter
-from . import inputhook
+from . import inputhook, FONTSIZE
 
 from larch.io import (read_ascii, read_xdi, read_gsexdi,
                       gsescan_group, fix_varname,
                       is_athena_project, AthenaProject)
 from larch.version import make_banner
+from larch.utils import get_cwd
 
 FILE_WILDCARDS = "Data Files(*.0*,*.dat,*.xdi)|*.0*;*.dat;*.xdi|All files (*.*)|*.*"
 
 ICON_FILE = 'larch.ico'
+BACKGROUND_COLOUR = '#FCFCFA'
+FOREGROUND_COLOUR = '#050520'
+
 
 def makeColorPanel(parent, color):
     p = wx.Panel(parent, -1)
     p.SetBackgroundColour(color)
     return p
 
+
+def wx_inspect():
+    wx.GetApp().ShowInspectionTool()
+
 class LarchWxShell(object):
     ps1 = 'Larch>'
     ps2 = ' ... >'
@@ -62,20 +73,22 @@ class LarchWxShell(object):
         self.symtable.set_symbol('_sys.wx.inputhook',   inputhook)
         self.symtable.set_symbol('_sys.wx.ping',   inputhook.ping)
         self.symtable.set_symbol('_sys.wx.force_wxupdate', False)
-        self.symtable.set_symbol('_sys.wx.wxapp', output)
+        self.symtable.set_symbol('_sys.wx.wx_inspect', wx_inspect)
+        self.symtable.set_symbol('_sys.wx.wxapp', wx.GetApp())
         self.symtable.set_symbol('_sys.wx.parent', wx.GetApp().GetTopWindow())
+        self.symtable.set_symbol('_sys.last_eval_time', 0.0)
 
         if self.output is not None:
             style = self.output.GetDefaultStyle()
             bgcol = style.GetBackgroundColour()
             sfont = style.GetFont()
+            sfont.Family = wx.MODERN
+            sfont.Weight = wx.BOLD
+            sfont.PointSize = FONTSIZE
+            style.SetFont(sfont)
+            self.output.SetDefaultStyle(style)
             self.textstyle = wx.TextAttr('black', bgcol, sfont)
-
         self.SetPrompt(True)
-        # self.flush_timer = wx.Timer(wxparent)
-        # self.needs_flush = True
-        # wxparent.Bind(wx.EVT_TIMER, self.onFlushTimer, self.flush_timer)
-        # wx.CallAfter(self.flush_timer.Start, 500)
 
     def onUpdate(self, event=None):
         symtable = self.symtable
@@ -102,12 +115,16 @@ class LarchWxShell(object):
         if self.output is None:
             return
 
-        display_colors = self._larch.symtable._sys.display.colors
+        display_colors = self.symtable._sys.display.colors
+
         textattrs = display_colors.get(mode, {'color':'black'})
         color = textattrs['color']
+
         style = self.output.GetDefaultStyle()
-        bgcol = style.GetBackgroundColour()
-        sfont = style.GetFont()
+        bgcol = BACKGROUND_COLOUR
+        sfont = self.output.GetFont()
+        style.SetFont(sfont)
+        self.output.SetDefaultStyle(style)
         self.textstyle = wx.TextAttr(color, bgcol, sfont)
 
     def write_sys(self, text):
@@ -171,6 +188,7 @@ class LarchWxShell(object):
                 self.objtree.onRefresh()
             except ValueError:
                 pass
+            self.symtable._sys.last_eval_time = time.time()
             self.SetPrompt(self._larch.input.complete)
             return ret
 
@@ -178,7 +196,7 @@ class LarchPanel(wx.Panel):
     """Larch Input/Output Panel + Data Viewer as a wx.Panel,
     suitable for embedding into apps
     """
-    def __init__(self,  parent=None, _larch=None,
+    def __init__(self,  parent=None, _larch=None, font=None,
                  historyfile='history_larchgui.lar', **kwds):
         self.parent = parent
         if not historyfile.startswith(larch.site_config.user_larchdir):
@@ -189,18 +207,25 @@ class LarchPanel(wx.Panel):
 
         self.splitter = splitter = wx.SplitterWindow(self, style=wx.SP_LIVE_UPDATE)
         splitter.SetMinimumPaneSize(150)
-        self.SetBackgroundColour('#E9EEE0')
 
-        self.objtree = Filling(splitter,  rootLabel='_main')
+        self.objtree = Filling(splitter,  rootLabel='_main',
+                               fgcol=FOREGROUND_COLOUR, bgcol=BACKGROUND_COLOUR)
 
         self.output = wx.TextCtrl(splitter, -1,  '',
                                   style=wx.TE_MULTILINE|wx.TE_RICH|wx.TE_READONLY)
 
+        self.output.SetBackgroundColour(BACKGROUND_COLOUR)
+        self.output.SetForegroundColour(FOREGROUND_COLOUR)
+        if font is None:
+            font = wx.Font(FONTSIZE, wx.MODERN, wx.NORMAL, wx.BOLD, 0, "")
+
+        self.output.SetFont(font)
+        self.objtree.tree.SetFont(font)
+        self.objtree.text.SetFont(font)
+
         self.output.CanCopy()
         self.output.SetInsertionPointEnd()
-        # self.output.SetDefaultStyle(wx.TextAttr('black', 'white', sfont))
-
-        splitter.SplitHorizontally(self.objtree, self.output, 0.5)
+        splitter.SplitHorizontally(self.objtree, self.output, 0)
 
         ipanel = wx.Panel(self)
 
@@ -240,12 +265,12 @@ class LarchPanel(wx.Panel):
                                        input  = self.input)
 
         self.objtree.SetRootObject(self.larchshell.symtable)
-        root = self.objtree.tree.GetRootItem()
-        self.objtree.tree.Expand(root)
+        # root = self.objtree.tree.GetRootItem()
+
 
     def write_banner(self):
         self.larchshell.set_textstyle('text2')
-        self.larchshell.write(make_banner([wx]))
+        self.larchshell.write(make_banner())
         self.larchshell.write("\n  \n")
         self.larchshell.set_textstyle('text')
 
@@ -267,19 +292,17 @@ class LarchPanel(wx.Panel):
         pos = self.input.GetSelection()
         ctrl = event.ControlDown()
 
-        # really, the order here is important:
-        # 1. return sends to ValidateEntry
         if key == wx.WXK_RETURN and len(entry) > 0:
             pass
-        if key == wx.WXK_UP:
-            self.hist_mark = max(0, self.hist_mark-1)
-            wx.CallAfter(self.set_input_text, self.hist_buff[self.hist_mark])
-        elif key == wx.WXK_DOWN:
-            self.hist_mark += 1
-            if self.hist_mark >= len(self.hist_buff):
-                self.input.SetValue('')
+        if key in (wx.WXK_UP, wx.WXK_DOWN):
+            if key == wx.WXK_UP:
+                self.hist_mark = max(0, self.hist_mark-1)
             else:
+                self.hist_mark += 1
+            try:
                 wx.CallAfter(self.set_input_text, self.hist_buff[self.hist_mark])
+            except IndexError:
+                wx.CallAfter(self.set_input_text, '')
         event.Skip()
 
     def set_input_text(self, text):
@@ -295,7 +318,6 @@ class LarchPanel(wx.Panel):
                 self.hist_mark = len(self.hist_buff)
 
 
-
 class LarchFrame(wx.Frame):
     def __init__(self, parent=None, _larch=None, is_standalone=True,
                  historyfile='history_larchgui.lar', with_inspection=False,
@@ -307,12 +329,11 @@ class LarchFrame(wx.Frame):
         self.historyfile = historyfile
         self.subframes = {}
         self.last_array_sel = {}
-
-        wx.Frame.__init__(self, parent, -1, size=(750, 725),
+        wx.Frame.__init__(self, parent, -1, size=(800, 725),
                           style= wx.DEFAULT_FRAME_STYLE)
         self.SetTitle('LarchGUI')
 
-        self.font = wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD, 0, "")
+        self.font = wx.Font(FONTSIZE, wx.MODERN, wx.NORMAL, wx.BOLD, 0, "")
         self.SetFont(self.font)
         sbar = self.CreateStatusBar(2, wx.CAPTION)
 
@@ -320,7 +341,8 @@ class LarchFrame(wx.Frame):
         self.SetStatusText("Larch initializing...", 0)
 
         self.mainpanel = LarchPanel(parent=self, _larch=_larch,
-                                    historyfile=historyfile)
+                                    historyfile=historyfile,
+                                    font=self.font)
 
         self.larchshell = self.mainpanel.larchshell
         self._larch = self.larchshell._larch
@@ -334,10 +356,10 @@ class LarchFrame(wx.Frame):
         if parent is None and exit_on_close:
             self.Bind(wx.EVT_CLOSE,  self.onExit)
         else:
-             self.Bind(wx.EVT_CLOSE,  self.onClose)
+            self.Bind(wx.EVT_CLOSE,  self.onClose)
         self.Bind(wx.EVT_SHOW, self.onShow)
         self.BuildMenus()
-
+        self.onSelectFont(fsize=FONTSIZE)
         # larchdir = larch.site_config.larchdir
 
         fico = os.path.join(larch.site_config.icondir, ICON_FILE)
@@ -367,8 +389,6 @@ class LarchFrame(wx.Frame):
                  'Change Directory', self.onChangeDir)
         MenuItem(self, fmenu, 'Clear Input\tCtrl+D',
                  'Clear Input', self.onClearInput)
-        MenuItem(self, fmenu, 'Select Font\tCtrl+F',
-                 'Select Font', self.onSelectFont)
 
         if self.with_inspection:
             MenuItem(self, fmenu, 'Show wxPython Inspector\tCtrl+I',
@@ -392,23 +412,43 @@ class LarchFrame(wx.Frame):
                                  name=appname, creator=creator))
             menuBar.Append(appmenu, 'Applications')
 
+        fsmenu = wx.Menu()
+        self.fontsizes = {}
+        for fsize in (7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 20, 22, 24):
+            m = MenuItem(self, fsmenu,  "%d" % fsize, "%d" % fsize,
+                         self.onSelectFont, kind=wx.ITEM_RADIO)
+            self.fontsizes[m.GetId()] = fsize
+            if fsize == FONTSIZE:
+                m.Check()
+
+        menuBar.Append(fsmenu, 'Font Size')
+
         hmenu = wx.Menu()
         MenuItem(self, hmenu, '&About',
                  'Information about this program',  self.onAbout)
         menuBar.Append(hmenu, '&Help')
         self.SetMenuBar(menuBar)
 
-    def onSelectFont(self, event=None):
-        fdata = wx.FontData()
-        fdata.SetInitialFont(self.font)
-        dlg = wx.FontDialog(self, fdata)
-        if dlg.ShowModal() == wx.ID_OK:
-            self.font = dlg.GetFontData().GetChosenFont()
-            self.SetFont(self.font)
-            self.mainpanel.output.SetFont(self.font)
-            self.mainpanel.objtree.SetFont(self.font)
-            self.mainpanel.objtree.text.SetFont(self.font)
-        dlg.Destroy()
+    def onSelectFont(self, event=None, fsize=None):
+        if fsize is None:
+            fsize = self.fontsizes.get(event.GetId(), FONTSIZE)
+
+        def set_fontsize(obj, fsize):
+            fn = obj.GetFont()
+            f1, f2 = fn.PixelSize
+            fn.SetPixelSize(wx.Size(int((f1*fsize/f2)), fsize))
+            obj.SetFont(fn)
+            # print('SetFont ', fn.Family, fn.FaceName, fn.PixelSize,
+            #       fn.PointSize, fn.Style, fn.Weight, obj)
+
+        self.PointSize = fsize
+        set_fontsize(self, fsize)
+        set_fontsize(self.mainpanel.output,  fsize)
+        set_fontsize(self.mainpanel.objtree.tree, fsize)
+        set_fontsize(self.mainpanel.objtree.text, fsize)
+        self.mainpanel.objtree.text.fontsize = fsize
+
+
 
     def onWxInspect(self, event=None):
         wx.GetApp().ShowInspectionTool()
@@ -441,7 +481,7 @@ class LarchFrame(wx.Frame):
     def onReadData(self, event=None):
         wildcard = 'Data file (*.dat)|*.dat|All files (*.*)|*.*'
         dlg = wx.FileDialog(self, message='Open Data File',
-                            defaultDir=os.getcwd(),
+                            defaultDir=get_cwd(),
                             wildcard=FILE_WILDCARDS,
                             style=wx.FD_OPEN|wx.FD_CHANGE_DIR)
         path = None
@@ -549,17 +589,17 @@ class LarchFrame(wx.Frame):
 
     def onChangeDir(self, event=None):
         dlg = wx.DirDialog(None, 'Choose a Working Directory',
-                           defaultPath = os.getcwd(),
+                           defaultPath = get_cwd(),
                            style = wx.DD_DEFAULT_STYLE)
 
         if dlg.ShowModal() == wx.ID_OK:
             os.chdir(dlg.GetPath())
         dlg.Destroy()
-        return os.getcwd()
+        return get_cwd()
 
     def onAbout(self, event=None):
         about_msg =  """LarchGui:
-        %s""" % (make_banner([wx]))
+        %s""" % (make_banner())
 
         dlg = wx.MessageDialog(self, about_msg,
                                "About LarchGui", wx.OK | wx.ICON_INFORMATION)
@@ -577,6 +617,9 @@ class LarchFrame(wx.Frame):
             pass
 
     def onExit(self, event=None, force=False):
+        if not self.exit_on_close:
+            self.Hide()
+            return
         if force:
             ret = wx.ID_YES
         else:
@@ -602,11 +645,12 @@ class LarchFrame(wx.Frame):
 
 class LarchApp(LarchWxApp):
     "simple app to wrap LarchFrame"
-    def __init__(self, **kws):
+    def __init__(self, with_inspection=False, **kws):
+        self.with_inspection = with_inspection
         LarchWxApp.__init__(self, **kws)
 
     def createApp(self):
-        frame = LarchFrame(exit_on_close=True, with_inspection=self.with_inspect)
+        frame = LarchFrame(exit_on_close=True, with_inspection=self.with_inspection)
         frame.Show()
         self.SetTopWindow(frame)
         return True
