@@ -15,17 +15,18 @@ import time
 import os
 import sys
 import wx
-
+from copy import deepcopy
 from wxmplot import PlotFrame, ImageFrame, StackedPlotFrame
+from wxmplot.interactive import get_wxapp
 
 import larch
 from ..xrf import isLarchMCAGroup
 from ..larchlib import ensuremod
-from ..site_config import usr_larchdir
+from ..site_config import user_larchdir
 
 from .xrfdisplay import XRFDisplayFrame
 
-mpl_dir = os.path.join(usr_larchdir, 'matplotlib')
+mpl_dir = os.path.join(user_larchdir, 'matplotlib')
 os.environ['MPLCONFIGDIR'] = mpl_dir
 if not os.path.exists(mpl_dir):
     try:
@@ -41,6 +42,13 @@ PLOT_DISPLAYS = {}
 FITPLOT_DISPLAYS = {}
 XRF_DISPLAYS = {}
 
+PLOTOPTS = {'theme': 'light',
+            'height': 550,
+            'width': 600,
+            'linewidth': 2.5,
+            'show_grid': True,
+            'show_fullbox': True}
+
 _larch_name = '_plotter'
 
 __DOC__ = '''
@@ -60,8 +68,8 @@ imshow           image display (false-color intensity image)
 xrf_plot         browsable display for XRF spectra
 '''
 
-MAX_WINDOWS = 20
-MAX_CURSHIST = 25
+MAX_WINDOWS = 25
+MAX_CURSHIST = 100
 
 class XRFDisplay(XRFDisplayFrame):
     def __init__(self, wxparent=None, window=1, _larch=None,
@@ -105,7 +113,7 @@ class XRFDisplay(XRFDisplayFrame):
 class PlotDisplay(PlotFrame):
     def __init__(self, wxparent=None, window=1, _larch=None, size=None, **kws):
         PlotFrame.__init__(self, parent=None, size=size,
-                           output_title='plot2d',
+                           output_title='larchplot',
                            exit_callback=self.onExit, **kws)
 
         self.Show()
@@ -249,14 +257,24 @@ class ImageDisplay(ImageFrame):
         if iy is not None:  set('%s_iy' % self.symname, iy)
         if val is not None: set('%s_val' % self.symname, val)
 
-def _getDisplay(win=1, _larch=None, wxparent=None, size=None,
-                wintitle=None, xrf=False, image=False, stacked=False):
+def get_display(win=1, _larch=None, wxparent=None, size=None,
+                wintitle=None, xrf=False, image=False, stacked=False,
+                theme=None, linewidth=None, markersize=None,
+                show_grid=None, show_fullbox=None, height=None,
+                width=None):
     """make a plotter"""
     # global PLOT_DISPLAYS, IMG_DISPlAYS
     if  hasattr(_larch, 'symtable'):
         if (getattr(_larch.symtable._sys.wx, 'wxapp', None) is None or
             getattr(_larch.symtable._plotter, 'no_plotting', False)):
             return None
+
+        global PLOTOPTS
+        try:
+            PLOTOPTS = deepcopy(_larch.symtable._sys.wx.plotopts)
+        except:
+            pass
+
     win = max(1, min(MAX_WINDOWS, int(abs(win))))
     title   = 'Plot Window %i' % win
     symname = '%s.plot%i' % (_larch_name, win)
@@ -281,8 +299,10 @@ def _getDisplay(win=1, _larch=None, wxparent=None, size=None,
     if wintitle is not None:
         title = wintitle
 
-    def _get_disp(symname, creator, win, ddict, wxparent, size, _larch):
+    def _get_disp(symname, creator, win, ddict, wxparent, size, height, width, _larch):
+        wxapp = get_wxapp()
         display = None
+        new_display = False
         if win in ddict:
             display = ddict[win]
             try:
@@ -298,26 +318,65 @@ def _getDisplay(win=1, _larch=None, wxparent=None, size=None,
                     s = display.GetSize()
                 except RuntimeError:  # window has been deleted
                     display = None
-            
+
         if display is None:
+            if size is None:
+                if height is None:
+                    height = PLOTOPTS['height']
+                if width is None:
+                    width = PLOTOPTS['width']
+                size = (int(width), int(height))
             display = creator(window=win, wxparent=wxparent,
                               size=size, _larch=_larch)
-        ddict[win] = display
-        return display
+            new_display = True
+            parent = wx.GetApp().GetTopWindow()
+            if parent is not None:
+                xpos, ypos = parent.GetPosition()
+                xsiz, ysiz = parent.GetSize()
+                display.SetPosition((xpos+(xsiz//4)*win, ypos+(ysiz//8)*win))
 
-    display = _get_disp(symname, creator, win, display_dict, wxparent,
-                        size, _larch)
+        ddict[win] = display
+        return display, new_display
+
+
+    display, isnew  = _get_disp(symname, creator, win, display_dict, wxparent,
+                                size, height, width, _larch)
+    if isnew and creator in (PlotDisplay, StackedPlotDisplay):
+        if theme is not None:
+            PLOTOPTS['theme'] = theme
+        if show_grid is not None:
+            PLOTOPTS['show_grid'] = show_grid
+        if show_fullbox is not None:
+            PLOTOPTS['show_fullbox'] = show_fullbox
+        if linewidth is not None:
+            PLOTOPTS['linewidth'] = linewidth
+        if markersize is not None:
+            PLOTOPTS['markersize'] = markersize
+        panels = [display.panel]
+        if creator == StackedPlotDisplay:
+            panels.append(display.panel_bot)
+        for panel in panels:
+            conf = panel.conf
+            conf.set_theme(theme=PLOTOPTS['theme'])
+            conf.enable_grid(PLOTOPTS['show_grid'])
+            conf.axes_style = 'box' if PLOTOPTS['show_fullbox'] else 'open'
+            for i in range(16):
+               conf.set_trace_linewidth(PLOTOPTS['linewidth'], trace=i)
     try:
         display.SetTitle(title)
+
     except:
         display_dict.pop(win)
-        display = _get_disp(symname, creator, win, display_dict, wxparent,
-                            size, _larch)
-        display.SetTitle(title) 
+        display, isnew = _get_disp(symname, creator, win, display_dict, wxparent,
+                                   size, _larch)
+        display.SetTitle(title)
     if  hasattr(_larch, 'symtable'):
         _larch.symtable.set_symbol(symname, display)
     return display
 
+
+_getDisplay = get_display # back compatibility
+
 def _xrf_plot(x=None, y=None, mca=None, win=1, new=True, as_mca2=False, _larch=None,
               wxparent=None, size=None, side='left', force_draw=True, wintitle=None,
               **kws):
@@ -342,7 +401,7 @@ def _xrf_plot(x=None, y=None, mca=None, win=1, new=True, as_mca2=False, _larch=N
 
     See Also: xrf_oplot, plot
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size,
+    plotter = get_display(wxparent=wxparent, win=win, size=size,
                           _larch=_larch, wintitle=wintitle, xrf=True)
     if plotter is None:
         return
@@ -430,7 +489,7 @@ def _plot(x,y, win=1, new=False, _larch=None, wxparent=None, size=None,
 
     See Also: oplot, newplot
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size,
+    plotter = get_display(wxparent=wxparent, win=win, size=size,
                           xrf=xrf, stacked=stacked,
                           wintitle=wintitle,  _larch=_larch)
     if plotter is None:
@@ -451,7 +510,7 @@ def _redraw_plot(win=1, xrf=False, stacked=False, size=None, wintitle=None,
     multiple plot()s with delay_draw=True
     """
 
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size,
+    plotter = get_display(wxparent=wxparent, win=win, size=size,
                           xrf=xrf, stacked=stacked,
                           wintitle=wintitle,  _larch=_larch)
     plotter.panel.unzoom_all()
@@ -460,7 +519,7 @@ def _redraw_plot(win=1, xrf=False, stacked=False, size=None, wintitle=None,
 def _update_trace(x, y, trace=1, win=1, _larch=None, wxparent=None,
                  side='left', redraw=False, **kws):
     """update a plot trace with new data, avoiding complete redraw"""
-    plotter = _getDisplay(wxparent=wxparent, win=win, _larch=_larch)
+    plotter = get_display(wxparent=wxparent, win=win, _larch=_larch)
     if plotter is None:
         return
     plotter.Raise()
@@ -480,7 +539,7 @@ def wx_update(_larch=None, **kws):
 def _plot_setlimits(xmin=None, xmax=None, ymin=None, ymax=None, win=1, wxparent=None,
                     _larch=None):
     """set plot view limits for plot in window `win`"""
-    plotter = _getDisplay(wxparent=wxparent, win=win, _larch=_larch)
+    plotter = get_display(wxparent=wxparent, win=win, _larch=_larch)
     if plotter is None:
         return
     plotter.panel.set_xylims((xmin, xmax, ymin, ymax))
@@ -536,7 +595,7 @@ def _plot_text(text, x, y, win=1, side='left', size=None,
 
     See Also: plot, oplot, plot_arrow
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, xrf=xrf,
+    plotter = get_display(wxparent=wxparent, win=win, size=size, xrf=xrf,
                           stacked=stacked, _larch=_larch)
     if plotter is None:
         return
@@ -572,7 +631,7 @@ def _plot_arrow(x1, y1, x2, y2, win=1, side='left',
 
     See Also: plot, oplot, plot_text
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, xrf=xrf,
+    plotter = get_display(wxparent=wxparent, win=win, size=size, xrf=xrf,
                           stacked=stacked, _larch=_larch)
     if plotter is None:
         return
@@ -598,7 +657,7 @@ def _plot_marker(x, y, marker='o', size=4, color='black', label='_nolegend_',
 
     See Also: plot, oplot, plot_text
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=None, xrf=xrf,
+    plotter = get_display(wxparent=wxparent, win=win, size=None, xrf=xrf,
                           stacked=stacked, _larch=_larch)
     if plotter is None:
         return
@@ -618,7 +677,7 @@ def _plot_axhline(y, xmin=0, xmax=1, win=1, wxparent=None, xrf=False,
         xmax:   ending x fraction (window units -- not user units!)
     See Also: plot, oplot, plot_arrow
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, xrf=xrf,
+    plotter = get_display(wxparent=wxparent, win=win, size=size, xrf=xrf,
                           stacked=stacked, _larch=_larch)
     if plotter is None:
         return
@@ -641,7 +700,7 @@ def _plot_axvline(x, ymin=0, ymax=1, win=1, wxparent=None, xrf=False,
         ymax:   ending y fraction (window units -- not user units!)
     See Also: plot, oplot, plot_arrow
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, xrf=xrf,
+    plotter = get_display(wxparent=wxparent, win=win, size=size, xrf=xrf,
                           stacked=stacked, _larch=_larch)
     if plotter is None:
         return
@@ -652,7 +711,7 @@ def _plot_axvline(x, ymin=0, ymax=1, win=1, wxparent=None, xrf=False,
     if not delay_draw:
         plotter.panel.canvas.draw()
 
-def _getcursor(win=1, timeout=30, _larch=None, wxparent=None, size=None,
+def _getcursor(win=1, timeout=15, _larch=None, wxparent=None, size=None,
                xrf=False, stacked=False, **kws):
     """get_cursor(win=1, timeout=30)
 
@@ -667,31 +726,29 @@ def _getcursor(win=1, timeout=30, _larch=None, wxparent=None, size=None,
     For a more consistent programmatic approach, this routine can be called
     with timeout <= 0 to read the most recently clicked cursor position.
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, xrf=xrf,
+    plotter = get_display(wxparent=wxparent, win=win, size=size, xrf=xrf,
                           stacked=stacked, _larch=_larch)
     if plotter is None:
         return
     symtable = ensuremod(_larch, _larch_name)
-    sentinal = '%s.plot%i_cursorflag' % (_larch_name, win)
     xsym = '%s.plot%i_x' % (_larch_name, win)
     ysym = '%s.plot%i_y' % (_larch_name, win)
 
     xval = symtable.get_symbol(xsym, create=True)
     yval = symtable.get_symbol(ysym, create=True)
-    symtable.set_symbol(sentinal, False)
-
-    def onChange(symbolname=None, **kws):
-        symtable.set_symbol(kws['sentinal'], True)
-
-    symtable.add_callback(xsym, onChange, kws={'sentinal': sentinal})
+    symtable.set_symbol(xsym, None)
 
     t0 = time.time()
     while time.time() - t0 < timeout:
         wx_update(_larch=_larch)
-        if symtable.get_symbol(sentinal):
+        time.sleep(0.05)
+        if symtable.get_symbol(xsym) is not None:
             break
-    symtable.del_symbol(sentinal)
-    symtable.clear_callbacks(xsym)
+
+    # restore value on timeout
+    if symtable.get_symbol(xsym, create=False) is None:
+        symtable.set_symbol(xsym, xval)
+
     return (symtable.get_symbol(xsym), symtable.get_symbol(ysym))
 
 def last_cursor_pos(win=None, _larch=None):
@@ -740,7 +797,7 @@ def _scatterplot(x,y, win=1, _larch=None, wxparent=None, size=None,
 
     See Also: plot, newplot
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, _larch=_larch)
+    plotter = get_display(wxparent=wxparent, win=win, size=size, _larch=_larch)
     if plotter is None:
         return
     plotter.Raise()
@@ -763,7 +820,7 @@ def _fitplot(x, y, y2=None, panel='top', label=None, label2=None, win=1,
 
     See Also: plot, newplot
     """
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size,
+    plotter = get_display(wxparent=wxparent, win=win, size=size,
                           stacked=True, _larch=_larch)
     if plotter is None:
         return
@@ -775,14 +832,16 @@ def _fitplot(x, y, y2=None, panel='top', label=None, label2=None, win=1,
         plotter.plot(x, y2-y, panel='bot')
         plotter.panel.conf.set_margins(top=0.15, bottom=0.01,
                                        left=0.15, right=0.05)
+        plotter.panel.unzoom_all()
         plotter.panel_bot.conf.set_margins(top=0.01, bottom=0.35,
                                            left=0.15, right=0.05)
+        plotter.panel_bot.unzoom_all()
 
 
 def _hist(x, bins=10, win=1, new=False,
            _larch=None, wxparent=None, size=None, force_draw=True,  *args, **kws):
 
-    plotter = _getDisplay(wxparent=wxparent, win=win, size=size, _larch=_larch)
+    plotter = get_display(wxparent=wxparent, win=win, size=size, _larch=_larch)
     if plotter is None:
         return
     plotter.Raise()
@@ -811,7 +870,7 @@ def _imshow(map, x=None, y=None, colormap=None, win=1, _larch=None,
 
     map: 2-dimensional array for map
     """
-    img = _getDisplay(wxparent=wxparent, win=win, size=size, _larch=_larch, image=True)
+    img = get_display(wxparent=wxparent, win=win, size=size, _larch=_larch, image=True)
     if img is not None:
         img.display(map, x=x, y=y, colormap=colormap, **kws)
 
@@ -838,7 +897,7 @@ def _saveplot(fname, dpi=300, format=None, win=1, _larch=None, wxparent=None,
             format = suffix
     if format is None: format = 'png'
     format = format.lower()
-    canvas = _getDisplay(wxparent=wxparent, win=win, size=size,
+    canvas = get_display(wxparent=wxparent, win=win, size=size,
                          _larch=_larch, image=image).panel.canvas
     if canvas is None:
         return
@@ -863,3 +922,42 @@ def _closeDisplays(_larch=None, **kws):
                     FITPLOT_DISPLAYS, XRF_DISPLAYS):
         for win in display.values():
             win.Destroy()
+
+def get_zoomlimits(plotpanel, dgroup):
+    """save current zoom limits, to be reapplied with set_zoomlimits()"""
+    view_lims = plotpanel.get_viewlimits()
+    zoom_lims = plotpanel.conf.zoom_lims
+    out = None
+    inrange = 3
+    if len(zoom_lims) > 0:
+        if zoom_lims[-1] is not None:
+            _ax =  list(zoom_lims[0].keys())[-1]
+            if all([_ax.get_xlabel() == dgroup.plot_xlabel,
+                    _ax.get_ylabel() == dgroup.plot_ylabel,
+                    min(dgroup.xdat) <= view_lims[1],
+                    max(dgroup.xdat) >= view_lims[0],
+                    min(dgroup.ydat) <= view_lims[3],
+                    max(dgroup.ydat) >= view_lims[2]]):
+                out = (_ax, view_lims, zoom_lims)
+    return out
+
+def set_zoomlimits(plotpanel, limits, verbose=False):
+    """set zoom limits returned from get_zoomlimits()"""
+    if limits is None:
+        if verbose:
+            print("set zoom, no limits")
+        return False
+    ax, vlims, zoom_lims = limits
+    if ax == plotpanel.axes:
+        try:
+            ax.set_xlim((vlims[0], vlims[1]), emit=True)
+            ax.set_ylim((vlims[2], vlims[3]), emit=True)
+            if len(plotpanel.conf.zoom_lims) == 0 and len(zoom_lims) > 0:
+                plotpanel.conf.zoom_lims = zoom_lims
+                if verbose:
+                    print("set zoom, ", zoom_lims)
+        except:
+            if verbose:
+                print("set zoom, exception")
+            return False
+    return True
