@@ -246,11 +246,12 @@ class EditColumnFrame(wx.Frame) :
             cind = SimpleText(panel, label='  %i ' % (i+1))
             cold = SimpleText(panel, label=' %s ' % name)
             cret = SimpleText(panel, label=fix_varname(name), size=(150, -1))
-            cnew = wx.TextCtrl(panel,  value=name, size=(150, -1))
 
-            cnew.Bind(wx.EVT_KILL_FOCUS, partial(self.update, index=i))
-            cnew.Bind(wx.EVT_CHAR, partial(self.update_char, index=i))
+            cnew = wx.TextCtrl(panel, value=name, size=(150, -1),
+                               style=wx.TE_PROCESS_ENTER)
+
             cnew.Bind(wx.EVT_TEXT_ENTER, partial(self.update, index=i))
+            cnew.Bind(wx.EVT_KILL_FOCUS, partial(self.update, index=i))
 
             arr = group.data[i,:]
             info_str = " [ %8g : %8g ] " % (arr.min(), arr.max())
@@ -301,11 +302,6 @@ class EditColumnFrame(wx.Frame) :
         newval = fix_varname(self.wids["%d" % index].GetValue())
         self.wids["ret_%i" % index].SetLabel(newval)
 
-    def update_char(self, evt=None, index=-1):
-        if evt.GetKeyCode() == wx.WXK_RETURN:
-            self.update(evt=evt, index=index)
-        evt.Skip()
-
     def onOK(self, evt=None):
         group = self.group
         array_labels = []
@@ -338,7 +334,7 @@ class SpecfileImporter(wx.Frame) :
         curscan = None
         for scandata in self.specfile.get_scans():
             name, cmd, dtime = scandata
-            self.scans.append("%s: %s" % (name, cmd))
+            self.scans.append("%s | %s" % (name, cmd))
             if curscan is None:
                 curscan = name
 
@@ -443,10 +439,10 @@ class SpecfileImporter(wx.Frame) :
         self.ypop = Choice(panel, choices=YPRE_OPS, action=self.onUpdate, size=(150, -1))
         self.yop =  Choice(panel, choices=ARR_OPS, action=self.onUpdate, size=(50, -1))
         self.yerr_op = Choice(panel, choices=YERR_OPS, action=self.onYerrChoice, size=(150, -1))
-        
+
         self.yerr_val = FloatCtrl(panel, value=1, precision=4, size=(90, -1))
         self.monod_val  = FloatCtrl(panel, value=3.1355316, precision=7, size=(90, -1))
-        
+
         xlab = SimpleText(panel, ' X array: ')
         ylab = SimpleText(panel, ' Y array: ')
         units_lab = SimpleText(panel, '  Units:  ')
@@ -463,7 +459,7 @@ class SpecfileImporter(wx.Frame) :
         self.yop.SetStringSelection(self.array_sel['yop'])
         self.monod_val.SetValue(self.array_sel['monod'])
         self.monod_val.SetAction(self.onUpdate)
-        
+
         self.monod_val.Enable(self.array_sel['en_units'].startswith('deg'))
         self.en_units.SetStringSelection(self.array_sel['en_units'])
         self.yerr_op.SetStringSelection(self.array_sel['yerror'])
@@ -512,7 +508,7 @@ class SpecfileImporter(wx.Frame) :
         sizer.Add(self.yarr1, (ir, 2), (1, 1), LEFT, 0)
         sizer.Add(self.yop,   (ir, 3), (1, 1), RIGHT, 0)
         sizer.Add(self.yarr2, (ir, 4), (1, 1), LEFT, 0)
-        sizer.Add(self.ysuf,  (ir, 5), (1, 1), LEFT, 0)        
+        sizer.Add(self.ysuf,  (ir, 5), (1, 1), LEFT, 0)
 
         ir += 1
         sizer.Add(yerr_lab,      (ir, 0), (1, 1), LEFT, 0)
@@ -532,23 +528,30 @@ class SpecfileImporter(wx.Frame) :
         self.nb.SetActiveTabTextColour(wx.Colour(80,0,0))
 
         self.plotpanel = PlotPanel(rightpanel, messenger=self.plot_messages)
+        try:
+            plotopts = self._larch.symtable._sys.wx.plotopts
+            self.plotpanel.conf.set_theme(plotopts['theme'])
+            self.plotpanel.conf.enable_grid(plotopts['show_grid'])
+        except:
+            pass
+
         self.plotpanel.SetMinSize((300, 250))
 
         shead = wx.Panel(rightpanel)
         self.scanheader = wx.TextCtrl(shead, style=wx.TE_MULTILINE|wx.TE_READONLY,
                                       size=(400, 250))
         self.scanheader.SetValue('\n'.join(self.curscan.scan_header))
-        self.scanheader.SetFont(Font(10))        
+        self.scanheader.SetFont(Font(10))
         textsizer = wx.BoxSizer(wx.VERTICAL)
         textsizer.Add(self.scanheader, 1, LEFT|wx.GROW, 1)
         pack(shead, textsizer)
 
-        
+
         fhead = wx.Panel(rightpanel)
         self.fileheader = wx.TextCtrl(fhead, style=wx.TE_MULTILINE|wx.TE_READONLY,
                                       size=(400, 250))
         self.fileheader.SetValue('\n'.join(self.curscan.file_header))
-        self.fileheader.SetFont(Font(10))        
+        self.fileheader.SetFont(Font(10))
         textsizer = wx.BoxSizer(wx.VERTICAL)
         textsizer.Add(self.fileheader, 1, LEFT|wx.GROW, 1)
         pack(fhead, textsizer)
@@ -589,7 +592,7 @@ class SpecfileImporter(wx.Frame) :
             ncol, npts = rdata.shape
         except:
             self.statusbar.SetStatusText(f"Warning: Could not read data for scan '{self.curscan.title:s}'")
-            
+
         workgroup = self.workgroup
         if xname.startswith('_index') or ix >= ncol:
             workgroup.xdat = 1.0*np.arange(npts)
@@ -604,7 +607,7 @@ class SpecfileImporter(wx.Frame) :
     def onScanSelect(self, event=None):
         try:
             scan_desc = event.GetString()
-            name = [s.strip() for s in scan_desc.split(':')][0]
+            name = [s.strip() for s in scan_desc.split(' | ')][0]
             self.curscan = self.specfile.get_scan(name)
         except:
             return
@@ -712,7 +715,7 @@ class SpecfileImporter(wx.Frame) :
         """ build arrays according to selection """
         scanlist = []
         for s in self.scanlist.GetCheckedStrings():
-            words = [s.strip() for s in s.split(':')]
+            words = [s.strip() for s in s.split('|')]
             scanlist.append(words[0])
         if len(scanlist) == 0:
             cancel = Popup(self, """No scans selected.
@@ -731,8 +734,8 @@ class SpecfileImporter(wx.Frame) :
         ypop     = self.ypop.GetStringSelection()
         yop      = self.yop.GetStringSelection()
         yerr_op = self.yerr_op.GetStringSelection()
-        yerr_arr = self.yerr_arr.GetStringSelection()        
-        yerr_idx = self.yerr_arr.GetSelection()        
+        yerr_arr = self.yerr_arr.GetStringSelection()
+        yerr_idx = self.yerr_arr.GetSelection()
         yerr_val = self.yerr_val.GetValue()
         yerr_expr = '1'
         if yerr_op.startswith('const'):
@@ -744,8 +747,8 @@ class SpecfileImporter(wx.Frame) :
         self.expressions['yerr'] = yerr_expr
 
         # generate script to pass back to calling program:
-        read_cmd = "_specfile.get_scan(scan='{scan}')"
-        buff = ["{group} = %s" % read_cmd,
+        # read_cmd = "_specfile.get_scan(scan='{scan}')"
+        buff = ["{group} = {specfile}.get_scan(scan='{scan}')",
                 "{group}.path = '{path}'",
                 "{group}.is_frozen = False"]
 
@@ -847,7 +850,7 @@ class SpecfileImporter(wx.Frame) :
                 self.monod_val.Enable()
             else:
                 self.en_units.SetSelection(0)
-                
+
         self.onUpdate()
 
     def onEnUnitsSelect(self, evt=None):
@@ -892,7 +895,7 @@ class SpecfileImporter(wx.Frame) :
         elif en_units.startswith('keV'):
             workgroup.xdat *= 1000.0
             xlabel = xname + ' (eV)'
-        
+
         workgroup.datatype = self.datatype.GetStringSelection().strip().lower()
 
         def pre_op(opwid, arr):
@@ -1003,7 +1006,10 @@ class SpecfileImporter(wx.Frame) :
         popts = dict(marker='o', markersize=4, linewidth=1.5,
                      title=fname, ylabel=ylabel, xlabel=xlabel,
                      label="%s: %s" % (fname, workgroup.plot_ylabel))
-        self.plotpanel.plot(workgroup.xdat, workgroup.ydat, **popts)
+        try:
+            self.plotpanel.plot(workgroup.xdat, workgroup.ydat, **popts)
+        except:
+            pass
 
 
     def plot_messages(self, msg, panel=1):
