@@ -33,13 +33,13 @@ except ImportError:
     HAS_WXPYTHON = False
 
 if HAS_WXPYTHON:
-    from .plotter import (_getDisplay, _plot, _oplot, _newplot, _fitplot,
+    from .plotter import (get_display, _plot, _oplot, _newplot, _fitplot,
                           _plot_text, _plot_marker, _plot_arrow,
                           _plot_axvline, _plot_axhline, _imshow)
 else:
     def nullfunc(*args, **kws): pass
 
-    _getDisplay = _plot = _oplot = _newplot = nullfunc
+    get_display = _plot = _oplot = _newplot = nullfunc
     _fitplot = _plot_text = _plot_marker = nullfunc
     _plot_arrow = _plot_axvline = _plot_axhline = nullfunc
 
@@ -75,13 +75,17 @@ plotlabels = Group(k       = r'$k \rm\,(\AA^{-1})$',
                    norm    = r'normalized $\mu(E)$',
                    flat    = r'flattened $\mu(E)$',
                    deconv  = r'deconvolved $\mu(E)$',
-                   dmude   = r'$d\mu(E)/dE$',
-                   dnormde = r'$d\mu_{\rm norm}(E)/dE$',
-                   d2mude  = r'$d^2\mu(E)/dE^2$',
-                   d2normde= r'$d^2\mu_{\rm norm}(E)/dE^2$',
+                   dmude   = r'$d\mu_{\rm norm}(E)/dE$',
+                   d2mude  = r'$d^2\mu_{\rm norm}(E)/dE^2$',
                    chie    = r'$\chi(E)$',
-                   chiew   = r'$E^{{{0:g}}}\chi(E) \rm\,(eV^{{-{0:g}}})$',
+                   chie0   = r'$\chi(E)$',
+                   chie1   = r'$E\chi(E) \rm\, (eV)$',
+                   chiew   = r'$E^{{{0:g}}}\chi(E) \rm\,(eV^{{{0:g}}})$',
                    chikw   = r'$k^{{{0:g}}}\chi(k) \rm\,(\AA^{{-{0:g}}})$',
+                   chi0    = r'$\chi(k)$',
+                   chi1    = r'$k\chi(k) \rm\,(\AA^{-1})$',
+                   chi2    = r'$k^2\chi(k) \rm\,(\AA^{-2})$',
+                   chi3    = r'$k^3\chi(k) \rm\,(\AA^{-3})$',
                    chir    = r'$\chi(R) \rm\,(\AA^{{-{0:g}}})$',
                    chirmag = r'$|\chi(R)| \rm\,(\AA^{{-{0:g}}})$',
                    chirre  = r'${{\rm Re}}[\chi(R)] \rm\,(\AA^{{-{0:g}}})$',
@@ -90,11 +94,15 @@ plotlabels = Group(k       = r'$k \rm\,(\AA^{-1})$',
                    e0color = '#B2B282',
                    chirlab = chirlab)
 
+def safetitle(t):
+    if "'" in t:
+        t = t.replace("'", "\\'")
+    return t
+
 def _get_title(dgroup, title=None):
     """get best title for group"""
     if title is not None:
-        return title
-    #endif
+        return safetitle(title)
     data_group = getattr(dgroup, 'data', None)
 
     for attr in ('title', 'plot_title', 'filename', 'name', '__name__'):
@@ -107,18 +115,21 @@ def _get_title(dgroup, title=None):
                 else:
                     top, folder = os.path.split(folder)
                     t = '/'.join((folder, file))
-            return t
-        #endif
+            return safetitle(t)
         if data_group is not None:
             t = getattr(data_group, attr, None)
             if t is not None:
                 return t
-            #endif
-        #endif
-    #endfor
+    return safetitle(repr(dgroup))
+
+
+def _get_kweight(dgroup, kweight=None):
+    if kweight is not None:
+        return kweight
+    callargs = getattr(dgroup, 'callargs', None)
+    ftargs = getattr(callargs, 'xftf', {'kweight':0})
+    return ftargs['kweight']
 
-    return repr(dgroup)
-#enddef
 
 def _get_erange(dgroup, emin=None, emax=None):
     """get absolute emin/emax for data range, allowing using
@@ -138,8 +149,9 @@ def _get_erange(dgroup, emin=None, emax=None):
 #enddef
 
 def redraw(win=1, xmin=None, xmax=None, ymin=None, ymax=None,
+           dymin=None, dymax=None,
            show_legend=True, stacked=False, _larch=None):
-    disp = _getDisplay(win=win, stacked=stacked, _larch=_larch)
+    disp = get_display(win=win, stacked=stacked, _larch=_larch)
     if disp is None:
         return
     panel = disp.panel
@@ -147,17 +159,26 @@ def redraw(win=1, xmin=None, xmax=None, ymin=None, ymax=None,
     if (xmin is not None or xmax is not None or
         ymin is not None or ymax is not None):
         panel.set_xylims((xmin, xmax, ymin, ymax))
-    else:
-        panel.unzoom_all()
+        if stacked:
+            disp.panel_bot.set_xylims((xmin, xmax, dymin, dymax))
+
+    panel.unzoom_all()
+    panel.reset_formats()
+    if stacked:
+        disp.panel_bot.unzoom_all()
+        disp.panel_bot.reset_formats()
     if show_legend:  # note: draw_legend *will* redraw the canvas
         panel.conf.draw_legend()
     else:
         panel.canvas.draw()
+        if stacked:
+            disp.panel_bot.canvas.draw()
+
     #endif
 #enddef
 
 
-def plot_mu(dgroup, show_norm=False, show_deriv=False,
+def plot_mu(dgroup, show_norm=False, show_flat=False, show_deriv=False,
             show_pre=False, show_post=False, show_e0=False, with_deriv=False,
             emin=None, emax=None, label='mu', new=True, delay_draw=False,
             offset=0, title=None, win=1, _larch=None):
@@ -172,11 +193,12 @@ def plot_mu(dgroup, show_norm=False, show_deriv=False,
     ----------
      dgroup     group of XAFS data after pre_edge() results (see Note 1)
      show_norm  bool whether to show normalized data [False]
-     show_deriv bool whether to show derivative of XAFS data [False]
+     show_flat  bool whether to show flattened, normalized data [False]
+     show_deriv bool whether to show derivative of normalized data [False]
      show_pre   bool whether to show pre-edge curve [False]
      show_post  bool whether to show post-edge curve [False]
      show_e0    bool whether to show E0 [False]
-     with_deriv bool whether to show deriv together with mu [False]
+     with_deriv bool whether to show deriv (dmu/de) together with mu [False]
      emin       min energy to show, absolute or relative to E0 [None, start of data]
      emax       max energy to show, absolute or relative to E0 [None, end of data]
      label      string for label [None:  'mu', `dmu/dE', or 'mu norm']
@@ -202,19 +224,23 @@ def plot_mu(dgroup, show_norm=False, show_deriv=False,
     #endif
     ylabel = plotlabels.mu
     if label is None:
-        label = 'mu'
+        label = getattr(dgroup, 'filename', 'mu')
     #endif
     if show_deriv:
-        mu = gradient(mu)/gradient(dgroup.energy)
-        ylabel = plotlabels.dmude
-        dlabel = '%s (deriv)' % label
+        mu = dgroup.dmude
+        ylabel = "%s (deriv)" % ylabel
+        dlabel = plotlabels.dmude
     elif show_norm:
         mu = dgroup.norm
         ylabel = "%s (norm)" % ylabel
-        dlabel = "%s (norm)" % label
+        dlabel = plotlabels.norm
+    #endif
+    elif show_flat:
+        mu = dgroup.flat
+        ylabel = "%s (flat)" % ylabel
+        dlabel = plotlabels.flat
     #endif
     emin, emax = _get_erange(dgroup, emin, emax)
-
     title = _get_title(dgroup, title=title)
 
     opts = dict(win=win, show_legend=True, linewidth=3,
@@ -225,7 +251,7 @@ def plot_mu(dgroup, show_norm=False, show_deriv=False,
           label=label, zorder=20, new=new, **opts)
 
     if with_deriv:
-        dmu = gradient(mu)/gradient(dgroup.energy)
+        dmu = dgroup.dmude
         _plot(dgroup.energy, dmu+offset, ylabel=plotlabels.dmude,
               label='%s (deriv)' % label, zorder=18, side='right', **opts)
     #endif
@@ -251,7 +277,7 @@ def plot_mu(dgroup, show_norm=False, show_deriv=False,
         _plot_axvline(dgroup.e0, zorder=2, size=3,
                       label='E0', color=plotlabels.e0color, win=win,
                       _larch=_larch)
-        disp = _getDisplay(win=win, _larch=_larch)
+        disp = get_display(win=win, _larch=_larch)
         if disp is not None:
             disp.panel.conf.draw_legend()
     redraw(win=win, xmin=emin, xmax=emax, _larch=_larch)
@@ -273,7 +299,7 @@ def plot_bkg(dgroup, norm=True, emin=None, emax=None, show_e0=False,
      emax       max energy to show, absolute or relative to E0 [None, end of data]
      show_e0     bool whether to show E0 [False]
      label       string for label [``None``: 'mu']
-     title       string for plot title [None, may use filename if available]
+     title       string for plot titlte [None, may use filename if available]
      new         bool whether to start a new plot [True]
      delay_draw  bool whether to delay draw until more traces are added [False]
      offset      vertical offset to use for y-array [0]
@@ -311,7 +337,7 @@ def plot_bkg(dgroup, norm=True, emin=None, emax=None, show_e0=False,
          title=title, label=label, zorder=20, new=new, xmin=emin, xmax=emax,
          **opts)
     ymin, ymax = None, None
-    disp = _getDisplay(win=win, _larch=_larch)
+    disp = get_display(win=win, _larch=_larch)
     if disp is not  None:
         xylims = disp.panel.get_viewlimits()
         ymin, ymax = xylims[2], xylims[3]
@@ -342,7 +368,7 @@ def plot_chie(dgroup, emin=-5, emax=None, label=None, title=None,
      label       string for label [``None``: 'mu']
      title       string for plot title [None, may use filename if available]
      new         bool whether to start a new plot [True]
-     eweight     energy weightingn for energies>e0  [0]
+     eweight     energy weightingn for energisdef es>e0  [0]
      show_k      bool whether to show k values   [True]
      delay_draw  bool whether to delay draw until more traces are added [False]
      offset      vertical offset to use for y-array [0]
@@ -383,7 +409,7 @@ def plot_chie(dgroup, emin=-5, emax=None, label=None, title=None,
         if ex < 0:
             s = ''
         else:
-            s = '\n[%.1f]' % (etok(ex))
+            s = '\n[%.2f]' % (etok(ex))
         return r"%1.4g%s" % (x, s)
 
     _plot(dgroup.energy-e0, chie+offset, xlabel=xlabel, ylabel=ylabel,
@@ -392,7 +418,7 @@ def plot_chie(dgroup, emin=-5, emax=None, label=None, title=None,
           linewidth=3, _larch=_larch)
 
     if show_k:
-        disp = _getDisplay(win=win, _larch=_larch)
+        disp = get_display(win=win, _larch=_larch)
         axes = disp.panel.axes
         axes.xaxis.set_major_formatter(FuncFormatter(ek_formatter))
 
@@ -429,13 +455,7 @@ def plot_chik(dgroup, kweight=None, kmax=None, show_window=True,
      1. The input data group must have the following attributes:
          k, chi, kwin, filename
     """
-    if kweight is None:
-        kweight = 0
-        xft = getattr(dgroup, 'xftf_details', None)
-        if xft is not None:
-            kweight = xft.call_args.get('kweight', 0)
-        #endif
-    #endif
+    kweight = _get_kweight(dgroup, kweight)
 
     chi = dgroup.chi * dgroup.k ** kweight
     opts = dict(win=win, show_legend=True, delay_draw=True, linewidth=3,
@@ -487,11 +507,7 @@ def plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
      1. The input data group must have the following attributes:
          r, chir_mag, chir_im, chir_re, kweight, filename
     """
-
-    try:
-        kweight = dgroup.xftf_details.call_args['kweight']
-    except:
-        kweight = 0
+    kweight = _get_kweight(dgroup, None)
 
     if new:
         title = _get_title(dgroup, title=title)
@@ -503,7 +519,10 @@ def plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
     ylabel = plotlabels.chirlab(kweight, show_mag=show_mag,
                                 show_real=show_real, show_imag=show_imag)
     opts['ylabel'] = ylabel
-
+    if not hasattr(dgroup, 'r'):
+        print("group does not have chi(R) data")
+        return
+    #endif
     if label is None:
         label = 'chir'
     #endif
@@ -531,7 +550,7 @@ def plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
 
 def plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
               title=None, new=True, delay_draw=False, offset=0, win=1,
-              _larch=None):
+              show_window=False, scale_window=True, _larch=None):
     """
     plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
               new=True, win=1)
@@ -544,6 +563,8 @@ def plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
      kweight      k-weighting for plot [read from last xftf(), or 0]
      kmax         max k to show [None, end of data]
      show_chik    bool whether to also plot k-weighted chi(k) [False]
+     show_window  bool whether to also plot FT k-window [False]
+     scale_window bool whether to scale FT k-window to max |chi(q)| [True]
      label        string for label [``None`` to use 'chi']
      title        string for plot title [None, may use filename if available]
      new          bool whether to start a new plot [True]
@@ -556,13 +577,7 @@ def plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
      1. The input data group must have the following attributes:
          k, chi, kwin, filename
     """
-    if kweight is None:
-        kweight = 0
-        xft = getattr(dgroup, 'xftf_details', None)
-        if xft is not None:
-            kweight = xft.call_args.get('kweight', 0)
-        #endif
-    #endif
+    kweight = _get_kweight(dgroup, kweight)
     nk = len(dgroup.k)
     chiq = dgroup.chiq_re[:nk]
     opts = dict(win=win, show_legend=True, delay_draw=True, linewidth=3, _larch=_larch)
@@ -580,6 +595,13 @@ def plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
         chik = dgroup.chi * dgroup.k ** kweight
         _plot(dgroup.k, chik+offset, zorder=16, label='chi(k)',  **opts)
     #endif
+    if show_window and hasattr(dgroup, 'kwin'):
+        kwin = dgroup.kwin
+        if scale_window:
+            kwin = kwin*max(abs(chiq))
+        _plot(dgroup.k, kwin+offset, zorder=12, label='window',  **opts)
+    #endif
+
     redraw(win=win, xmax=kmax, _larch=_larch)
 #enddef
 
@@ -608,16 +630,13 @@ def plot_wavelet(dgroup, show_mag=True, show_real=False, show_imag=False,
     -----
      The wavelet will be performed
     """
-    if kweight is None:
-        kweight = dgroup.xftf_details.call_args['kweight']
-
+    kweight = _get_kweight(dgroup, kweight)
+    cauchy_wavelet(dgroup, kweight=kweight, rmax_out=rmax)
     title = _get_title(dgroup, title=title)
 
-    opts = dict(win=win, title=title, x=dgroup.k, y=dgroup.r, xmax=kmax,
+    opts = dict(win=win, title=title, x=dgroup.k, y=dgroup.wcauchy_r, xmax=kmax,
                 ymax=rmax, xlabel=plotlabels.k, ylabel=plotlabels.r,
                 show_axis=True, _larch=_larch)
-
-    cauchy_wavelet(dgroup, kweight=kweight)
     if show_mag:
         _imshow(dgroup.wcauchy_mag, **opts)
     elif show_real:
@@ -975,33 +994,34 @@ def plot_prepeaks_fit(dgroup, nfit=0, show_init=False, subtract_baseline=False,
         raise ValueError('Group needs prepeaks')
     #endif
     if show_init:
-        result = dgroup.prepeaks
+        result = pkfit = dgroup.prepeaks
     else:
-        result = getattr(dgroup.prepeaks, 'fit_history', None)
-        if nfit > len(result):
+        hist = getattr(dgroup.prepeaks, 'fit_history', None)
+        if nfit > len(hist):
             nfit = 0
-        result = result[nfit]
+        pkfit = hist[nfit]
+        result = pkfit.result
     #endif
 
-    if result is None:
+    if pkfit is None:
         raise ValueError('Group needs prepeaks.fit_history or init_fit')
     #endif
 
-    opts = result.user_options
+    opts = pkfit.user_options
     xeps = min(diff(dgroup.xdat)) / 5.
-    xdat = 1.0*result.energy
-    ydat = 1.0*result.norm
+    xdat = 1.0*pkfit.energy
+    ydat = 1.0*pkfit.norm
 
     xdat_full = 1.0*dgroup.xdat
     ydat_full = 1.0*dgroup.ydat
 
     if show_init:
-        yfit   = 1.0*result.init_fit
-        ycomps = None
+        yfit   = pkfit.init_fit
+        ycomps = None #  pkfit.init_ycomps
         ylabel = 'model'
     else:
         yfit   = 1.0*result.best_fit
-        ycomps = result.ycomps
+        ycomps = pkfit.ycomps
         ylabel = 'best fit'
 
     baseline = 0.*ydat
@@ -1030,9 +1050,14 @@ def plot_prepeaks_fit(dgroup, nfit=0, show_init=False, subtract_baseline=False,
     ncolor = 0
     popts = {'win': win, '_larch': _larch}
     plotopts.update(popts)
+    dymin = dymax = None
     if show_residual:
         popts['stacked'] = True
         _fitplot(xdat, ydat, yfit, label='data', label2=ylabel, **plotopts)
+        dy = yfit - ydat
+        dymax, dymin = dy.max(), dy.min()
+        dymax += 0.05 * (dymax - dymin)
+        dymin -= 0.05 * (dymax - dymin)
     else:
         _plot(xdat_full, ydat_full, new=True, label='data',
               color=LineColors[0], **plotopts)
@@ -1072,8 +1097,8 @@ def plot_prepeaks_fit(dgroup, nfit=0, show_init=False, subtract_baseline=False,
             _plot_axvline(pcen, delay_draw=False, ymin=0, ymax=1,
                           color='#EECCCC', label='_nolegend_', **popts)
 
-    redraw(win=win, xmin=dx0, xmax=dx1, ymin=min(dy0, fy0),
-           ymax=max(dy1, fy1), show_legend=True, _larch=_larch)
+    redraw(xmin=dx0, xmax=dx1, ymin=min(dy0, fy0),
+           ymax=max(dy1, fy1), dymin=dymin, dymax=dymax, show_legend=True, **popts)
 
 def _pca_ncomps(result, min_weight=0, ncomps=None):
     if ncomps is None:
@@ -1081,10 +1106,10 @@ def _pca_ncomps(result, min_weight=0, ncomps=None):
             ncomps = where(result.variances < min_weight)[0][0]
         else:
             ncomps = argmin(result.ind)
-    return ncomps - 1
+    return ncomps
 
 
-def plot_pca_components(result, min_weight=0, ncomps=None, win=1, _larch=None, **kws):
+def plot_pca_components(result, min_weight=0, ncomps=None, min_variance=1.e-5, win=1, _larch=None, **kws):
     """Plot components from PCA result
 
     result must be output of `pca_train`
@@ -1097,12 +1122,13 @@ def plot_pca_components(result, min_weight=0, ncomps=None, win=1, _larch=None, *
                  win=win, _larch=_larch)
 
     popts.update(kws)
-    ncomps = _pca_ncomps(result, min_weight=min_weight, ncomps=ncomps)
+    ncomps = int(result.nsig)
 
     _plot(result.x, result.mean, label='Mean', **popts)
-    for i, comp in enumerate(result.components[:ncomps+1]):
-        label = 'Comp# %d (%.4f)' % (i+1, result.variances[i])
-        _oplot(result.x, comp, label=label, **popts)
+    for i, comp in enumerate(result.components):
+        if result.variances[i] > min_variance:
+            label = 'Comp# %d (%.4f)' % (i+1, result.variances[i])
+            _oplot(result.x, comp, label=label, **popts)
 
     redraw(win=win, show_legend=True, _larch=_larch)
 
@@ -1122,8 +1148,7 @@ def plot_pca_weights(result, min_weight=0, ncomps=None, win=1, _larch=None, **kw
 
     popts.update(kws)
 
-    ncomps = _pca_ncomps(result, min_weight=min_weight, ncomps=ncomps)
-
+    ncomps = max(1, int(result.nsig))
     x = 1 + arange(ncomps)
     y = result.variances[:ncomps]
     _plot(x, y, label='significant', **popts)
@@ -1132,12 +1157,12 @@ def plot_pca_weights(result, min_weight=0, ncomps=None, win=1, _larch=None, **kw
     ye = result.variances[ncomps-1:ncomps+max_comps]
 
     popts.update(dict(new=False, zorder=5, style='short dashed',
-                      color='#B34050', ymin=2.e-3*result.variances[ncomps]))
+                      color='#B34050', ymin=2e-3*result.variances[ncomps-1]))
     _plot(xe, ye, label='not significant', **popts)
 
-    xi = 1 + arange(len(result.ind)-2)
+    xi = 1 + arange(len(result.ind)-1)
 
-    _plot(xi, result.ind[1:len(xi)+1], zorder=15, y2label='Indicator Value',
+    _plot(xi, result.ind[1:], zorder=15, y2label='Indicator Value',
           label='IND', style='solid', win=win, show_legend=True,
           linewidth=1, marker='o', side='right', _larch=_larch)
 
@@ -1162,7 +1187,7 @@ def plot_pca_fit(dgroup, win=1, with_components=False, _larch=None, **kws):
     _fitplot(result.x, result.ydat, result.yfit,
              label='data', label2='PCA fit', **popts)
 
-    disp = _getDisplay(win=win, stacked=True, _larch=_larch)
+    disp = get_display(win=win, stacked=True, _larch=_larch)
     if with_components and disp is not None:
         disp.panel.oplot(result.x, model.mean, label='mean')
         for n in range(len(result.weights)):
