@@ -15,29 +15,24 @@ import wx.lib.scrolledpanel as scrolled
 import wx.dataview as dv
 import wx.lib.colourselect  as csel
 
-try:
-    from wx._core import PyDeadObjectError
-except:
-    PyDeadObjectError = Exception
-
-
 import numpy as np
 import matplotlib
 from matplotlib.ticker import LogFormatter, FuncFormatter
 
 from wxmplot import PlotPanel
-
 from wxutils import (SimpleText, EditableListBox, Font, pack, Popup,
                      get_icon, SetTip, Button, Check, MenuItem, Choice,
                      FileOpen, FileSave, fix_filename, HLine, GridPanel,
                      CEN, LEFT, RIGHT)
-
+from pyshortcuts import platform
+from . import FONTSIZE, FONTSIZE_FW
 from ..math import index_of
-from ..utils import bytes2str, debugtime
+from ..utils import bytes2str, debugtime, get_cwd
 from ..io import GSEMCA_File
 from ..site_config import icondir
 from ..interpreter import Interpreter
 
+from .gui_utils import LarchWxApp
 from .larchframe import LarchFrame
 from .periodictable import PeriodicTablePanel
 
@@ -79,7 +74,7 @@ class XRFDisplayFrame(wx.Frame):
     def __init__(self, _larch=None, parent=None, filename=None,
                  size=(725, 450), axissize=None, axisbg=None,
                  title='XRF Display', exit_callback=None,
-                 output_title='XRF', **kws):
+                 output_title='XRF', roi_callback=None, **kws):
 
         if size is None: size = (725, 450)
         wx.Frame.__init__(self, parent=parent,
@@ -88,6 +83,7 @@ class XRFDisplayFrame(wx.Frame):
         self.subframes = {}
         self.data = None
         self.title = title
+        self.roi_callback = roi_callback
         self.plotframe = None
         self.wids = {}
         self.larch = _larch
@@ -175,7 +171,7 @@ class XRFDisplayFrame(wx.Frame):
             self.xmarker_left  = min(ix1, ix2)
             self.xmarker_right = max(ix1, ix2)
 
-        if side == 'left':
+        if side == 'left' and ix is not None:
             self.energy_for_zoom = self.mca.energy[ix]
         self.update_status()
         self.draw()
@@ -278,7 +274,7 @@ class XRFDisplayFrame(wx.Frame):
                         output_title='test.xrf',
                         messenger=self.write_message)
         pan.SetSize((650, 350))
-        
+
         pan.conf.grid_color='#E5E5C0'
         pan.conf.show_grid = False
         pan.conf.canvas.figure.set_facecolor('#FCFCFE')
@@ -293,11 +289,12 @@ class XRFDisplayFrame(wx.Frame):
 
     def createControlPanel(self):
         ctrlpanel = wx.Panel(self, name='Ctrl Panel')
-
+        ptable_fontsize = 11 if platform=='darwin' else 9
         ptable = PeriodicTablePanel(ctrlpanel, onselect=self.onShowLines,
                                     tooltip_msg='Select Element for KLM Lines',
-                                    fontsize=10)
+                                    fontsize=ptable_fontsize, size=(360, 180))
         self.wids['ptable'] = ptable
+        self.font_fixedwidth = wx.Font(FONTSIZE_FW, wx.MODERN, wx.NORMAL, wx.NORMAL)
 
         labstyle = wx.ALIGN_LEFT|wx.EXPAND
         ctrlstyle = wx.ALIGN_LEFT
@@ -411,14 +408,19 @@ class XRFDisplayFrame(wx.Frame):
 
         dvstyle = dv.DV_SINGLE|dv.DV_VERT_RULES|dv.DV_ROW_LINES
         xlines = dv.DataViewListCtrl(ctrlpanel, style=dvstyle)
+        xlines.SetFont(self.font_fixedwidth)
         self.wids['xray_lines'] = xlines
-        xlines.AppendTextColumn(' Line ',         width=60)
-        xlines.AppendTextColumn(' Energy(keV) ',  width=110)
-        xlines.AppendTextColumn(' Strength ',     width=85)
-        xlines.AppendTextColumn(' Levels ',       width=75)
+
+        xw = (60, 100, 80, 100)
+        if platform=='win':
+            xw = (65, 105, 85, 100)
+        xlines.AppendTextColumn(' Line ',         width=xw[0])
+        xlines.AppendTextColumn(' Energy(keV) ',  width=xw[1])
+        xlines.AppendTextColumn(' Strength ',     width=xw[2])
+        xlines.AppendTextColumn(' Levels ',       width=xw[3])
         for col in (0, 1, 2, 3):
             this = xlines.Columns[col]
-            this.Sortable = True
+            this.Sortable = False
             align = RIGHT
             if col in (0, 3):
                 align = wx.ALIGN_LEFT
@@ -489,7 +491,7 @@ class XRFDisplayFrame(wx.Frame):
         else:
             self.mca2 = self.mca
             self.mca = mca
-
+        # print("Add MCA ", mca, as_mca2)
         xrfgroup = self.larch.symtable.get_group(XRFGROUP)
         mcaname = next_mcaname(self.larch)
         if filename is not None:
@@ -498,7 +500,7 @@ class XRFDisplayFrame(wx.Frame):
                                                 filename=filename))
             if label is None:
                 label = filename
-        if label is None and mca.filename is not None:
+        if label is None and hasattr(mca, 'filename'):
             label = mca.filename
         if label is None:
             label = mcaname
@@ -507,6 +509,7 @@ class XRFDisplayFrame(wx.Frame):
         setattr(xrfgroup, '_mca2', getattr(xrfgroup, '_mca', ''))
         setattr(xrfgroup, '_mca', mcaname)
         setattr(xrfgroup, mcaname, mca)
+        # print("Add MCA ", xrfgroup, mcaname)
         if plot:
             self.plotmca(self.mca)
             if as_mca2:
@@ -618,6 +621,9 @@ class XRFDisplayFrame(wx.Frame):
         self.onROI(label=roiname)
         if self.selected_elem is not None:
             self.onShowLines(elem=self.selected_elem)
+        if self.roi_callback is not None:
+            xrange = [self.mca.energy[left], self.mca.energy[right]]
+            self.roi_callback(roiname, xrange=xrange, action='add', units='keV', roitype='XRF')
         return True
 
     def onConfirmDelROI(self, event=None):
@@ -625,6 +631,8 @@ class XRFDisplayFrame(wx.Frame):
         msg = "Delete ROI {:s}?".format(roiname)
         if (wx.ID_YES == Popup(self, msg,   'Delete ROI?', style=wx.YES_NO)):
             self.onDelROI()
+            if self.roi_callback is not None:
+                self.roi_callback(roiname, action='delete', roitype='XRF')
 
     def onRenameROI(self, event=None):
         roiname = self.get_roiname()
@@ -751,6 +759,8 @@ class XRFDisplayFrame(wx.Frame):
         MenuItem(self, fmenu, "&Save ASCII Column File\tCtrl+A",
                  "Save Column File",  self.onSaveColumnFile)
 
+        MenuItem(self, fmenu, "&Inspect \tCtrl+J",
+                 " wx inspection tool ",  self.showInspectionTool)
         fmenu.AppendSeparator()
         # MenuItem(self, fmenu, "Save ROIs to File",
         #         "Save ROIs to File",  self.onSaveROIs)
@@ -1273,7 +1283,7 @@ class XRFDisplayFrame(wx.Frame):
 
     def onReadMCAFile(self, event=None):
         dlg = wx.FileDialog(self, message="Open MCA File for reading",
-                            defaultDir=os.getcwd(),
+                            defaultDir=get_cwd(),
                             wildcard=FILE_WILDCARDS,
                             style = wx.FD_OPEN|wx.FD_CHANGE_DIR)
 
@@ -1310,6 +1320,7 @@ class XRFDisplayFrame(wx.Frame):
         if outfile is not None:
             self.mca.save_mcafile(outfile)
 
+
     def onSaveColumnFile(self, event=None, **kws):
         deffile = ''
         if getattr(self.mca, 'sourcefile', None) is not None:
@@ -1354,6 +1365,10 @@ class XRFDisplayFrame(wx.Frame):
         """write a message to the Status Bar"""
         self.SetStatusText(s, panel)
 
+    def showInspectionTool(self, event=None):
+        app = wx.GetApp()
+        app.ShowInspectionTool()
+
     def onAbout(self, event=None):
         dlg = wx.MessageDialog(self,
                                """XRF Spectral Viewer
@@ -1366,7 +1381,7 @@ class XRFDisplayFrame(wx.Frame):
 
     def onReadFile(self, event=None):
         dlg = wx.FileDialog(self, message="Read MCA File",
-                            defaultDir=os.getcwd(),
+                            defaultDir=get_cwd(),
                             wildcard=FILE_WILDCARDS,
                             style=wx.FD_OPEN)
         path, re1ad = None, False
@@ -1384,13 +1399,12 @@ class XRFDisplayFrame(wx.Frame):
             except:
                 return
 
-class XRFApp(wx.App, wx.lib.mixins.inspection.InspectionMixin):
+class XRFApp(LarchWxApp):
     def __init__(self, filename=None, **kws):
         self.filename = filename
-        wx.App.__init__(self)
+        LarchWxApp.__init__(self, **kws)
 
-    def OnInit(self):
-        self.Init()
+    def createApp(self):
         frame = XRFDisplayFrame(filename=self.filename)
         frame.Show()
         self.SetTopWindow(frame)
