@@ -2,6 +2,7 @@
 """
 fitting GUI for XRF display
 """
+import os
 import time
 import copy
 from functools import partial
@@ -28,7 +29,7 @@ from wxutils import (SimpleText, FloatCtrl, FloatSpin, Choice, Font, pack,
                      SetTip, GridPanel, Popup, FloatSpinWithPin, get_icon,
                      fix_filename)
 
-from . import FONTSIZE
+from . import FONTSIZE, FONTSIZE_FW
 from xraydb import (material_mu, xray_edge, materials, add_material,
                     atomic_number, atomic_symbol, xray_line)
 from .notebooks import flatnotebook
@@ -38,9 +39,10 @@ from .periodictable import PeriodicTablePanel
 from larch import Group
 
 from ..xrf import xrf_background, MCA, FanoFactors
+from ..utils import json_dump, json_load
 from ..utils.jsonutils import encode4js, decode4js
 from ..utils.physical_constants import E_MASS
-
+from ..site_config import user_larchdir
 from .xrfdisplay_utils import (XRFGROUP, mcaname, XRFRESULTS_GROUP,
                                MAKE_XRFRESULTS_GROUP)
 
@@ -55,6 +57,10 @@ def read_filterdata(flist, _larch):
     return out
 
 def VarChoice(p, default=0, size=(75, -1)):
+    if default in (False, 'False', 'Fix', 'No'):
+        default = 0
+    else:
+        default = 1
     return Choice(p, choices=['Fix', 'Vary'],
                   size=size, default=default)
 
@@ -62,11 +68,12 @@ NFILTERS = 4
 MIN_CORREL = 0.10
 
 tooltips = {'ptable': 'Select Elements to include in model',
+            'cen': 'centroid of peak',
             'step': 'size of step extending to low energy side of peak, fraction of peak height',
             'gamma': 'gamma (lorentzian-like weight) of Voigt function',
             'tail': 'intensity of tail function at low energy side of peak',
             'beta': 'width of tail function at low energy side of peak',
-            'sigmax': 'scale sigma from Energy/Noise by this amount',
+            'sigma': 'scale sigma from Energy/Noise by this amount',
         }
 
 CompositionUnits = ('ng/mm^2', 'wt %', 'ppm')
@@ -122,6 +129,41 @@ Filter_Lengths = ['microns', 'mm', 'cm']
 Filter_Materials = ['None', 'air', 'nitrogen', 'helium', 'kapton',
                     'beryllium', 'aluminum', 'mylar', 'pmma']
 
+
+DEF_CONFIG = { "mca_name": "", "escape_use": True, "escape_amp": 0.25,
+               "pileup_use": True, "pileup_amp": 0.1, "escape_amp_vary":
+               True, "pileup_amp_vary": True, "cal_slope": 0.01,
+               "cal_offset": 0, "cal_vary": True, "det_mat": "Si", "det_thk":
+               0.4, "det_noise_vary": True,
+               "en_xray": 30, "en_min": 2.5, "en_max": 30,
+               "flux_in": 1e9, "det_noise": 0.035, "angle_in": 45.0,
+               "angle_out": 45.0, "det_dist": 50.0, "det_area": 50.0,
+               "elements": [16, 17, 18, 20, 22, 24, 25, 26, 28, 29, 30],
+               "filter1_mat": "beryllium", "filter1_thk": 0.025, "filter1_var": False,
+               "filter2_mat": "air", "filter2_thk": 50.0, "filter2_var": False,
+               "filter3_mat": "kapton", "filter3_thk": 0.0, "filter3_var": False,
+               "filter4_mat": "aluminum", "filter4_thk": 0.0, "filter4_var": False,
+               "matrix_mat": "", "matrix_thk": 0.0, "matrix_den": 1.0,
+               "peak_step": 0.025, "peak_gamma": 0.05, "peak_tail": 0.1,
+               "peak_beta": 0.5,  "peak_step_vary": False, "peak_tail_vary": False,
+               "peak_gamma_vary": False, "peak_beta_vary": False,
+               "elastic_use": True, "elastic_cen_vary": False, "elastic_step_vary": False,
+               "elastic_beta_vary": False, "elastic_tail_vary": False,
+               "elastic_sigma_vary": False, "elastic_cen": 30,
+               "elastic_step": 0.025, "elastic_tail": 0.1,
+               "elastic_beta": 0.5, "elastic_sigma": 1.0,
+               "compton1_use": True, "compton1_cen_vary": True,
+               "compton1_step_vary": True, "compton1_beta_vary": False,
+               "compton1_tail_vary": False, "compton1_sigma_vary": False,
+               "compton1_cen": 12.1875, "compton1_step": 0.025, "compton1_tail": 0.25,
+               "compton1_beta": 2.0, "compton1_sigma": 1.5,
+               "compton2_use": True, "compton2_cen_vary": True, "compton2_step_vary": False,
+               "compton2_beta_vary": False, "compton2_tail_vary": False,
+               "compton2_sigma_vary": False, "compton2_cen": 11.875,
+               "compton2_step": 0.025, "compton2_tail": 0.25, "compton2_beta": 2.5,
+               "compton2_sigma": 2.0, "count_time": 120.0,
+              }
+
 class FitSpectraFrame(wx.Frame):
     """Frame for Spectral Analysis"""
 
@@ -138,6 +180,17 @@ class FitSpectraFrame(wx.Frame):
         mcagroup = getattr(xrfgroup, '_mca')
         self.mca = getattr(xrfgroup, mcagroup)
         self.mcagroup = '%s.%s' % (XRFGROUP, mcagroup)
+        print("Get MCA Groups: ",XRFGROUP, xrfgroup, mcagroup)
+
+        self.config = DEF_CONFIG
+
+        opts = getattr(xrfgroup, 'fitconfig', None)
+        if opts is None:
+            xrf_conffile = os.path.join(user_larchdir, 'xrf_fitconfig.json')
+            if os.path.exists(xrf_conffile):
+                opts = json_load(xrf_conffile)
+        if opts is not None:
+            self.config.update(opts)
 
         efactor = 1.0 if max(self.mca.energy) < 250. else 1000.0
 
@@ -151,15 +204,15 @@ class FitSpectraFrame(wx.Frame):
         wx.Frame.__init__(self, parent, -1, 'Fit XRF Spectra',
                           size=size, style=wx.DEFAULT_FRAME_STYLE)
 
+        self.font_fixedwidth = wx.Font(FONTSIZE_FW, wx.MODERN, wx.NORMAL, wx.NORMAL)   # fixed width
+
+
         self.wids = {}
         self.owids = {}
 
         pan = GridPanel(self)
-        mca_label = getattr(self.mca, 'label', None)
-        if mca_label is None:
-            mca_label = getattr(self.mca, 'filename', 'mca')
-        self.mca_label = mca_label
-        self.wids['mca_name'] = SimpleText(pan, mca_label, size=(300, -1), style=LEFT)
+        self.mca_label = self.mca.label
+        self.wids['mca_name'] = SimpleText(pan, self.mca_label, size=(300, -1), style=LEFT)
         self.wids['btn_calc'] = Button(pan, 'Calculate Model', size=(150, -1),
                                        action=self.onShowModel)
         self.wids['btn_fit'] = Button(pan, 'Fit Model', size=(150, -1),
@@ -177,8 +230,7 @@ class FitSpectraFrame(wx.Frame):
         self.panels['Fit Results']        = self.fitresult_page
         self.panels['Composition'] = self.composition_page
 
-        self.nb = flatnotebook(pan, self.panels,
-                               on_change=self.onNBChanged)
+        self.nb = flatnotebook(pan, self.panels, on_change=self.onNBChanged)
         pan.Add((5, 5), newrow=True)
         pan.Add(self.nb, dcol=5, drow=10, newrow=True)
         pan.pack()
@@ -197,29 +249,21 @@ class FitSpectraFrame(wx.Frame):
         wids = self.wids
         p = GridPanel(self)
         self.selected_elems = []
-        self.ptable = PeriodicTablePanel(p, multi_select=True, fontsize=10,
+        self.ptable = PeriodicTablePanel(p, multi_select=True, fontsize=11,
+                                         size=(360, 180),
                                          tooltip_msg=tooltips['ptable'],
                                          onselect=self.onElemSelect)
-
-        dstep, dtail, dbeta, dgamma = 0.05, 0.10, 0.5, 0.05
-        wids['peak_step'] = FloatSpin(p, value=dstep, digits=3, min_val=0,
-                                      max_val=1.0, increment=0.01,
-                                      tooltip=tooltips['step'])
-        wids['peak_gamma'] = FloatSpin(p, value=dgamma, digits=3, min_val=0,
-                                       max_val=10.0, increment=0.01,
-                                      tooltip=tooltips['gamma'])
-        wids['peak_tail'] = FloatSpin(p, value=dtail, digits=3, min_val=0,
-                                      max_val=1.0, increment=0.05,
-                                      tooltip=tooltips['tail'])
-
-        wids['peak_beta'] = FloatSpin(p, value=dbeta, digits=3, min_val=0,
-                                      max_val=10.0, increment=0.01,
-                                      tooltip=tooltips['beta'])
-        wids['peak_step_vary'] = VarChoice(p, default=0)
-        wids['peak_tail_vary'] = VarChoice(p, default=0)
-        wids['peak_gamma_vary'] = VarChoice(p, default=0)
-        wids['peak_beta_vary'] = VarChoice(p, default=0)
-
+        cnf = self.config
+        for name, xmax, xinc in (('step',   1, 0.005),
+                                 ('gamma', 10, 0.01),
+                                 ('beta',  10, 0.01),
+                                 ('tail', 1.0, 0.05)):
+            fname = 'peak_%s' % name
+            vname = 'peak_%s_vary' % name
+            wids[fname] = FloatSpin(p, value=cnf[fname], digits=3,
+                                    min_val=0, max_val=xmax,
+                                    increment=xinc, tooltip=tooltips[name])
+            wids[vname] = VarChoice(p, default=cnf[vname])
 
         btn_from_peaks = Button(p, 'Guess Peaks', size=(150, -1),
                                 action=self.onElems_GuessPeaks)
@@ -257,73 +301,40 @@ class FitSpectraFrame(wx.Frame):
         p.AddText('  Tail: ')
         p.Add(wids['peak_tail'])
         p.Add(wids['peak_tail_vary'])
-
         p.Add((2, 2), newrow=True)
         p.Add(HLine(p, size=(650, 3)), dcol=8)
         p.Add((2, 2), newrow=True)
 
-        #                name, escale, step, sigmax, beta, tail
-        scatter_peaks = (('Elastic',  0, 0.05, 1.0, 0.5, 0.10),
-                         ('Compton1', 1, 0.05, 1.5, 2.0, 0.25),
-                         ('Compton2', 2, 0.05, 2.0, 2.5, 0.25))
         opts = dict(size=(100, -1), min_val=0, digits=4, increment=0.010)
-        for name, escale, dstep, dsigma, dbeta, dtail in scatter_peaks:
+        for name, escale in (('elastic',  0), ('compton1', 1), ('compton2', 2)):
             en = self.mca.incident_energy
             for i in range(escale):
                 en = en * (1 - 1/(1 + (E_MASS*0.001)/en))    # Compton shift at 90 deg
-            
-            t = name.lower()
-            vary_en = 1 if t.startswith('compton') else 0
-
-            wids['%s_use'%t] = Check(p, label='Include', default=True)
-            wids['%s_cen_vary'%t]   = VarChoice(p, default=vary_en)
-            wids['%s_step_vary'%t]  = VarChoice(p, default=0)
-            wids['%s_beta_vary'%t]  = VarChoice(p, default=0)
-            wids['%s_tail_vary'%t]  = VarChoice(p, default=0)
-            wids['%s_sigma_vary'%t] = VarChoice(p, default=0)
-
-            wids['%s_cen'%t]  = FloatSpin(p, value=en, digits=3, min_val=0,
-                                           increment=0.01)
-            wids['%s_step'%t] = FloatSpin(p, value=dstep, digits=3, min_val=0,
-                                          max_val=1.0, increment=0.01,
-                                          tooltip=tooltips['step'])
-            wids['%s_tail'%t] = FloatSpin(p, value=dtail, digits=3, min_val=0,
-                                          max_val=1.0, increment=0.05,
-                                          tooltip=tooltips['tail'])
-            wids['%s_beta'%t] = FloatSpin(p, value=dbeta, digits=3, min_val=0,
-                                          max_val=10.0, increment=0.10,
-                                          tooltip=tooltips['beta'])
-            wids['%s_sigma'%t] = FloatSpin(p, value=dsigma, digits=3, min_val=0,
-                                           max_val=10.0, increment=0.05,
-                                           tooltip=tooltips['sigmax'])
 
+            wids[f'{name:s}_use'] = Check(p, label='Include', default=cnf[f'{name:s}_use'])
             p.Add((2, 2), newrow=True)
-            p.AddText("  %s Peak:" % name,  colour='#880000')
-            p.Add(wids['%s_use' % t], dcol=2)
+            p.AddText("  %s Peak:" % name.title(),  colour='#880000')
+            p.Add(wids[f'{name:s}_use'], dcol=2)
 
-            p.AddText('  Energy (keV): ')
-            p.Add(wids['%s_cen'%t])
-            p.Add(wids['%s_cen_vary'%t])
+            for att, title, xmax, xinc, newrow in (('cen',   '  Energy (kev): ', 9e12, 0.01, True),
+                                                   ('step',  '  Step: ', 1, 0.005, False),
+                                                   ('sigma', '  Sigma Scale:',  10, 0.05, True),
+                                                   ('beta',  '  Beta: ', 10, 0.01, False),
+                                                   ('tail',  '  Tail: ', 10, 0.01, True)):
+                label = f'{name:s}_{att:s}'
+                val = en if att == 'cen' else cnf[label]
+                wids[f'{label:s}_vary'] = VarChoice(p, default=cnf[f'{label:s}_vary'])
 
-            p.Add((2, 2), newrow=True)
-            p.AddText('  Step: ')
-            p.Add(wids['%s_step'%t])
-            p.Add(wids['%s_step_vary'%t])
+                wids[label] = FloatSpin(p, value=val, digits=3, min_val=0,
+                                        max_val=xmax, increment=xinc,
+                                        tooltip=tooltips[att])
 
-            p.AddText('  Sigma Scale : ')
-            p.Add(wids['%s_sigma'%t])
-            p.Add(wids['%s_sigma_vary'%t])
+                p.AddText(title)
+                p.Add(wids[label])
+                p.Add(wids[f'{label:s}_vary'])
+                if newrow:
+                    p.Add((2, 2), newrow=True)
 
-            p.Add((2, 2), newrow=True)
-            p.AddText('  Beta : ')
-            p.Add(wids['%s_beta'%t])
-            p.Add(wids['%s_beta_vary'%t])
-
-            p.AddText('  Tail: ')
-            p.Add(wids['%s_tail'%t])
-            p.Add(wids['%s_tail_vary'%t])
-
-            p.Add((2, 2), newrow=True)
             p.Add(HLine(p, size=(650, 3)), dcol=7)
 
         p.pack()
@@ -338,11 +349,13 @@ class FitSpectraFrame(wx.Frame):
         cal_offset = getattr(mca, 'offset',  0)
         cal_slope = getattr(mca, 'slope',  0.010)
         det_noise = getattr(mca, 'det_noise',  0.035)
-        escape_amp = getattr(mca, 'escape_amp', 1.0)
-        pileup_amp = getattr(mca, 'pileup_amp', 0.1)
+        escape_amp = getattr(mca, 'escape_amp', 0.25)
+
+        if not hasattr(self.mca, 'pileup_scale'):
+            self.mca.predict_pileup()
+        pileup_amp = max(0.001, getattr(mca, 'pileup_scale', 0.10))
 
         wids = self.wids
-        # main = wx.Panel(self)
         pdet = GridPanel(self, itemstyle=LEFT)
 
         def addLine(pan):
@@ -352,17 +365,18 @@ class FitSpectraFrame(wx.Frame):
         wids['escape_use'] = Check(pdet, label='Include Escape in Fit',
                                    default=True, action=self.onUsePileupEscape)
         wids['escape_amp'] = FloatSpin(pdet, value=escape_amp,
-                                         min_val=0, max_val=100, digits=2,
-                                         increment=0.02, size=(100, -1))
+                                         min_val=0, max_val=100, digits=3,
+                                         increment=0.01, size=(100, -1))
 
         wids['pileup_use'] = Check(pdet, label='Include Pileup in Fit',
-                                   default=True, action=self.onUsePileupEscape)
+                                   default=True,
+                                   action=self.onUsePileupEscape)
         wids['pileup_amp'] = FloatSpin(pdet, value=pileup_amp,
-                                         min_val=0, max_val=100, digits=2,
-                                         increment=0.02, size=(100, -1))
+                                         min_val=0, max_val=100, digits=3,
+                                         increment=0.01, size=(100, -1))
 
         wids['escape_amp_vary'] = VarChoice(pdet, default=True)
-        wids['pileup_amp_vary'] = VarChoice(pdet, default=True)
+        wids['pileup_amp_vary'] = VarChoice(pdet, default=(pileup_amp>0.002))
 
 
         wids['cal_slope'] = FloatSpin(pdet, value=cal_slope,
@@ -636,10 +650,11 @@ class FitSpectraFrame(wx.Frame):
         sizer.Add(title, (irow, 0), (1, 4), LEFT)
 
         sview = wids['stats'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        sview.SetFont(self.font_fixedwidth)
         sview.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED, self.onSelectFit)
         sview.AppendTextColumn('  Fit Label', width=90)
-        sview.AppendTextColumn(' N_vary', width=65)
-        sview.AppendTextColumn(' N_eval', width=65)
+        sview.AppendTextColumn(' N_vary', width=70)
+        sview.AppendTextColumn(' N_eval', width=70)
         sview.AppendTextColumn(' \u03c7\u00B2', width=125)
         sview.AppendTextColumn(' \u03c7\u00B2_reduced', width=125)
         sview.AppendTextColumn(' Akaike Info', width=125)
@@ -648,7 +663,7 @@ class FitSpectraFrame(wx.Frame):
             this = sview.Columns[col]
             isort, align = True, wx.ALIGN_RIGHT
             if col == 0:
-                align = wx.ALIGN_CENTER
+                align = wx.ALIGN_LEFT
             this.Sortable = isort
             this.Alignment = this.Renderer.Alignment = align
         sview.SetMinSize((675, 150))
@@ -665,12 +680,13 @@ class FitSpectraFrame(wx.Frame):
         sizer.Add(title, (irow, 0), (1, 1), LEFT)
 
         pview = wids['params'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        pview.SetFont(self.font_fixedwidth)
         wids['paramsdata'] = []
         pview.AppendTextColumn('Parameter',      width=150)
-        pview.AppendTextColumn('Refined Value',  width=100)
-        pview.AppendTextColumn('Standard Error', width=100)
-        pview.AppendTextColumn('% Uncertainty', width=100)
-        pview.AppendTextColumn('Initial Value',  width=150)
+        pview.AppendTextColumn('Refined Value',  width=110)
+        pview.AppendTextColumn('Standard Error', width=110)
+        pview.AppendTextColumn('% Uncertainty',  width=110)
+        pview.AppendTextColumn('Initial Value',  width=125)
 
         for col in range(4):
             this = pview.Columns[col]
@@ -707,7 +723,7 @@ class FitSpectraFrame(wx.Frame):
         sizer.Add(wids['all_correl'], (irow, 3), (1, 1), LEFT)
 
         cview = wids['correl'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
-
+        cview.SetFont(self.font_fixedwidth)
         cview.AppendTextColumn('Parameter 1',    width=150)
         cview.AppendTextColumn('Parameter 2',    width=150)
         cview.AppendTextColumn('Correlation',    width=150)
@@ -738,11 +754,12 @@ class FitSpectraFrame(wx.Frame):
                                              colour=self.colors.title, style=LEFT)
 
         cview = wids['composition'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        cview.SetFont(self.font_fixedwidth)
         cview.AppendTextColumn(' Z ', width=50)
         cview.AppendTextColumn(' Element ', width=100)
-        cview.AppendTextColumn(' Amplitude', width=150)
-        cview.AppendTextColumn(' Concentration',  width=150)
-        cview.AppendTextColumn(' Uncertainty',  width=150)
+        cview.AppendTextColumn(' Amplitude', width=125)
+        cview.AppendTextColumn(' Concentration',  width=125)
+        cview.AppendTextColumn(' Uncertainty',  width=170)
 
         for col in range(5):
             this = cview.Columns[col]
@@ -887,14 +904,14 @@ class FitSpectraFrame(wx.Frame):
             for l in result.fit_report.split('\n'):
                 buff.append("#    %s" % l)
             buff.append("###########")
-            buff.append("#Element  Concentration  Uncertainty  RawAmplitude")
+            buff.append("#Element  Concentration  Uncertainty  Raw_Amplitude")
             for elem, dat in result.concentration_results.items():
                 eout = (elem  + ' '*4)[:4]
                 val, serr = dat
-                rval = "%16.5f" % val
-                sval = "%16.5f" % (val/scale)
-                uval = "%16.5f" % (serr/scale)
-                buff.append("  ".join([elem, sval, uval, rval]))
+                rval = "%15.4f" % val
+                sval = "%15.4f" % (val*scale)
+                uval = "%15.4f" % (serr*scale)
+                buff.append("  ".join([eout, sval, uval, rval]))
             buff.append('')
             with open(sfile, 'w') as fh:
                 fh.write('\n'.join(buff))
@@ -1165,7 +1182,14 @@ class FitSpectraFrame(wx.Frame):
         for elem in self.ptable.selected:
             elemz.append( 1 + self.ptable.syms.index(elem))
         elemz.sort()
-        syms = ["'%s'" % self.ptable.syms[iz-1] for iz in sorted(elemz)]
+        opts['elements'] = elemz
+
+        xrfgroup = self._larch.symtable.get_group(XRFGROUP)
+        setattr(xrfgroup, 'fitconfig', opts)
+        json_dump(opts, os.path.join(user_larchdir, 'xrf_fitconfig.json'))
+
+
+        syms = ["'%s'" % self.ptable.syms[iz-1] for iz in elemz]
         syms = '[%s]' % (', '.join(syms))
         script.append(xrfmod_elems.format(elemlist=syms))
         script.append("{group:s}.xrf_init = _xrfmodel.calc_spectrum({group:s}.energy)")
@@ -1175,6 +1199,7 @@ class FitSpectraFrame(wx.Frame):
         self._larch.eval(self.model_script)
 
         cmds = []
+        self._larch.symtable.get_symbol('_xrfmodel')
         self.xrfmod = self._larch.symtable.get_symbol('_xrfmodel')
         floor = 1.e-12*max(self.mca.counts)
         if match_amplitudes:
@@ -1194,7 +1219,7 @@ class FitSpectraFrame(wx.Frame):
                         scale = 1.0
                     if nam in ('compton2',):
                         scale /= 5.0
-                        
+
                 paramval = self.xrfmod.params[ampname].value
                 s = "_xrfmodel.params['%s'].value = %.5f" % (ampname, paramval*scale)
                 cmds.append(s)
@@ -1225,21 +1250,37 @@ class FitSpectraFrame(wx.Frame):
         if model_spectrum is None:
             model_spectrum = self.xrfmod.current_model if init else self.xrfmod.best_fit
         if label is None:
-            label = 'predicted model' if init else 'best fit'
+            label = 'model' if init else 'best fit'
 
         self.parent.oplot(self.mca.energy, model_spectrum,
                           label=label, color=conf.fit_color, **plotkws)
 
-        if with_comps:
-            for label, arr in self.xrfmod.comps.items():
-                ppanel.oplot(self.mca.energy, arr, label=label, **plotkws)
+        comp_traces = []
+        plotkws.update({'fill': True, 'alpha':0.35, 'show_legend': True})
+        for label, arr in self.xrfmod.comps.items():
+            ppanel.oplot(self.mca.energy, arr, label=label, **plotkws)
+            comp_traces.append(ppanel.conf.ntrace - 1)
+
 
         yscale = {False:'linear', True:'log'}[self.parent.ylog_scale]
         ppanel.set_logscale(yscale=yscale)
         ppanel.set_viewlimits()
-        ppanel.conf.set_legend_location('upper right', True)
+        ppanel.conf.auto_margins = False
+        ppanel.conf.set_margins(0.1, 0.02, 0.20, 0.1)
+        ppanel.conf.set_legend_location('upper right', False)
+        ppanel.conf.show_legend_frame = True
         ppanel.conf.draw_legend(show=True, delay_draw=False)
 
+        if not with_comps:
+            for obj, data in ppanel.conf.legend_map.items():
+                line, trace, legline, legtext = data
+                if trace in comp_traces:
+                    legline.set_alpha(0.50)
+                    legtext.set_alpha(0.50)
+                    line.set_visible(False)
+                    ppanel.conf.fills[trace].set_visible(False)
+        ppanel.draw()
+
     def onShowModel(self, event=None):
         self.build_model()
         self.plot_model(init=True, with_comps=True)
@@ -1438,12 +1479,11 @@ class FitSpectraFrame(wx.Frame):
                 val = gformat(param.value, 10)
             except (TypeError, ValueError):
                 val = ' ??? '
-
             serr, perr = ' N/A ', ' N/A '
             if param.stderr is not None:
                 serr = gformat(param.stderr, 10)
                 try:
-                    perr = ' {:.2%}'.format(abs(param.stderr/param.value))
+                    perr = '{:.3f}'.format(100.0*abs(param.stderr/param.value))
                 except ZeroDivisionError:
                     perr = '?'
             extra = ' '
