@@ -15,11 +15,11 @@ from larch.wxlib import (BitmapButton, FloatCtrl, FloatSpin, ToggleButton,
                          get_icon, SimpleText, pack, Button, HLine, Choice,
                          plotlabels, Check, CEN, RIGHT, LEFT)
 
-from larch.xafs.xafsutils import etok, ktoe
-
+from larch.xafs.xafsutils import etok, ktoe, FT_WINDOWS
 
 from .xas_dialogs import EnergyUnitsDialog
 from .taskpanel import TaskPanel
+from .config import ATHENA_CLAMPNAMES, PlotWindowChoices
 
 np.seterr(all='ignore')
 
@@ -47,16 +47,15 @@ PlotCmds = {mu_bkg:  "plot_bkg({group:s}",
             chie:    "plot_chie({group:s}",
             chik:    "plot_chik({group:s}, show_window=False, kweight={plot_kweight:.0f}",
             chikwin: "plot_chik({group:s}, show_window=True, kweight={plot_kweight:.0f}",
-            chirmag: "plot_chir({group:s}, show_mag=True, show_real=False",
-            chirre:  "plot_chir({group:s}, show_mag=False, show_real=True",
-            chirmr:  "plot_chir({group:s}, show_mag=True, show_real=True",
-            wavelet: "plot_wavelet({group:s}",
-            chir_w:  "plot_chir({group:s}, show_mag=True, show_real=True, show_window=True",
+            chirmag: "plot_chir({group:s}, show_mag=True, show_real=False, rmax={plot_rmax:.1f}",
+            chirre:  "plot_chir({group:s}, show_mag=False, show_real=True, rmax={plot_rmax:.1f}",
+            chirmr:  "plot_chir({group:s}, show_mag=True, show_real=True, rmax={plot_rmax:.1f}",
+            chir_w:  "plot_chir({group:s}, show_mag=True, show_real=True, show_window=True, rmax={plot_rmax:.1f}",
             chiq:    "plot_chiq({group:s}, show_chik=False",
             chikq:   "plot_chiq({group:s}, show_chik=True",
+            wavelet: "plot_wavelet({group:s}, rmax={plot_rmax:.1f}",
             noplot: None}
 
-FTWINDOWS = ('Kaiser-Bessel', 'Hanning', 'Gaussian', 'Sine', 'Parzen', 'Welch')
 
 CLAMPLIST = ('0', '1', '2', '5', '10', '20', '50', '100', '200', '500', '1000',
              '2000', '5000', '10000')
@@ -65,27 +64,18 @@ autobk_cmd = """autobk({group:s}, rbkg={rbkg: .3f}, e0={e0: .4f},
       kmin={bkg_kmin: .3f}, kmax={bkg_kmax: .3f}, kweight={bkg_kweight: .1f},
       clamp_lo={bkg_clamplo: .1f}, clamp_hi={bkg_clamphi: .1f})"""
 
-xftf_cmd = """xftf({group:s}, kmin={fft_kmin: .3f}, kmax={fft_kmax: .3f},
-      kweight={fft_kweight: .3f}, dk={fft_dk: .3f}, window='{fft_kwindow:s}')"""
+xftf_cmd = """xftf({group:s}, kmin={fft_kmin: .3f}, kmax={fft_kmax: .3f}, dk={fft_dk: .3f},
+      kweight={fft_kweight: .3f}, window='{fft_kwindow:s}', rmax_out={fft_rmaxout:.3f})"""
 
 xftr_cmd = """xftr({group:s}, rmin={fft_rmin: .3f}, rmax={fft_rmax: .3f},
       dr={fft_dr: .3f}, window='{fft_rwindow:s}')"""
 
 
-defaults = dict(e0=-1.0, rbkg=1, bkg_kmin=0, bkg_kmax=None, bkg_clamplo=0,
-                bkg_clamphi=1, bkg_kweight=2, fft_kmin=2.5, fft_kmax=None,
-                fft_dk=4, fft_kweight=2, fft_kwindow='Kaiser-Bessel',
-                fft_rmin=1, fft_rmax=6, fft_dr=0.25,
-                fft_rwindow='Hanning')
-
-
 class EXAFSPanel(TaskPanel):
     """EXAFS Panel"""
     def __init__(self, parent, controller, **kws):
-        TaskPanel.__init__(self, parent, controller,
-                           configname='exafs_config',
-                           title='EXAFS Background Subtraction and Fourier Transforms',
-                           config=defaults, **kws)
+        TaskPanel.__init__(self, parent, controller, panel='exafs', **kws)
+
         self.skip_process = False
         self.last_plot = 'one'
         self.last_process_bkg = {}
@@ -114,19 +104,8 @@ class EXAFSPanel(TaskPanel):
         plot_sel = Button(panel, 'Plot Selected Groups', size=(175, -1),
                           action=self.onPlotSel)
 
-
-        saveconf = Button(panel, 'Save as Default Settings', size=(200, -1),
-                          action=self.onSaveConfigBtn)
-
-        def xxxFSWithPinPanel(name, value, **kws):
-            s = wx.BoxSizer(wx.HORIZONTAL)
-            self.wids[name] = FloatSpin(panel, value=value, **kws)
-            bb = BitmapButton(panel, get_icon('pin'), size=(25, 25),
-                              action=partial(self.onSelPoint, opt=name),
-                              tooltip='use last point selected from plot')
-            s.Add(self.wids[name])
-            s.Add(bb)
-            return s
+        ## saveconf = Button(panel, 'Save as Default Settings', size=(200, -1),
+        #                  action=self.onSaveConfigBtn)
 
         wids['plot_voffset'] = FloatSpin(panel, value=0, digits=2, increment=0.25,
                                          action=self.onProcess)
@@ -137,6 +116,13 @@ class EXAFSPanel(TaskPanel):
                                              action=self.onProcess,
                                              min_val=0, max_val=5)
 
+        wids['plot_rmax'] = FloatSpin(panel, value=8, digits=1, increment=0.5,
+                                             action=self.onProcess,
+                                             min_val=2, max_val=20)
+        wids['plot_win']  = Choice(panel, size=(60, -1), choices=PlotWindowChoices,
+                                   action=self.onProcess)
+        wids['plot_win'].SetStringSelection('2')
+
         opts = dict(digits=2, increment=0.1, min_val=0, action=self.onProcess)
         wids['e0'] = FloatSpin(panel, **opts)
 
@@ -158,6 +144,8 @@ class EXAFSPanel(TaskPanel):
         fft_rmax = self.add_floatspin('fft_rmax', value=6, with_pin=True, **opts)
 
         wids['fft_dr'] = FloatSpin(panel, value=0.5,  **opts)
+        wids['fft_rmaxout'] = FloatSpin(panel, value=12, min_val=2,
+                                        increment=0.5, digits=1, max_val=20)
 
         opts.update({'increment': 1, 'digits': 1, 'max_val': 5})
         wids['bkg_kweight'] = FloatSpin(panel, value=2, **opts)
@@ -167,11 +155,11 @@ class EXAFSPanel(TaskPanel):
         wids['bkg_clamplo'] = Choice(panel, **opts)
         wids['bkg_clamphi'] = Choice(panel, **opts)
 
-        wids['fft_kwindow'] = Choice(panel, choices=list(FTWINDOWS),
+        wids['fft_kwindow'] = Choice(panel, choices=list(FT_WINDOWS),
                                      action=self.onProcess, size=(125, -1))
 
 
-        wids['fft_rwindow'] = Choice(panel, choices=list(FTWINDOWS),
+        wids['fft_rwindow'] = Choice(panel, choices=list(FT_WINDOWS),
                                      action=self.onProcess, size=(125, -1))
         wids['fft_rwindow'].SetStringSelection('Hanning')
 
@@ -185,6 +173,8 @@ class EXAFSPanel(TaskPanel):
         def CopyBtn(name):
             return Button(panel, 'Copy', size=(60, -1),
                           action=partial(self.onCopyParam, name))
+        copy_all = Button(panel, 'Copy All Parameters', size=(150, -1),
+                          action=partial(self.onCopyParam, 'all'))
 
 
         panel.Add(SimpleText(panel, 'EXAFS Data Reduction and Fourier Transforms',
@@ -206,7 +196,10 @@ class EXAFSPanel(TaskPanel):
         panel.Add(self.wids['plotalt_op'], dcol=2)
         add_text('Plot2 k weight: ', newrow=False)
         panel.Add(wids['plot_kweight_alt'])
-
+        add_text('Window for Second Plot: ', newrow=True)
+        panel.Add(self.wids['plot_win'], dcol=2)
+        add_text('Plot R max: ', newrow=False)
+        panel.Add(wids['plot_rmax'])
 
         panel.Add(HLine(panel, size=(500, 3)), dcol=6, newrow=True)
 
@@ -267,8 +260,11 @@ class EXAFSPanel(TaskPanel):
         panel.Add(wids['fft_dk'])
         panel.Add(CopyBtn('fft_kwindow'), style=RIGHT)
 
+        panel.Add(SimpleText(panel, 'R max output: '), newrow=True)
+        panel.Add(wids['fft_rmaxout'])
+        panel.Add((10, 10), dcol=2)
+        panel.Add(CopyBtn('fft_rmaxout'), style=RIGHT)
 
-        panel.Add((10, 10), newrow=True)
         panel.Add(HLine(panel, size=(500, 3)), dcol=6, newrow=True)
 
         panel.Add(SimpleText(panel, ' Back Fourier transform (R->q) ', size=(275, -1),
@@ -287,14 +283,11 @@ class EXAFSPanel(TaskPanel):
         panel.Add(wids['fft_rwindow'])
         panel.Add(SimpleText(panel, 'dR : '))
         panel.Add(wids['fft_dr'])
+
         panel.Add(CopyBtn('fft_rwindow'), style=RIGHT)
-
-
         panel.Add((10, 10), newrow=True)
         panel.Add(self.wids['is_frozen'], dcol=1, newrow=True)
-        panel.Add(saveconf, dcol=4)
-        panel.Add((10, 10), newrow=True)
-        panel.Add(HLine(self, size=(500, 3)), dcol=8, newrow=True)
+        panel.Add(copy_all, dcol=5, style=RIGHT)
 
         panel.pack()
 
@@ -311,47 +304,33 @@ class EXAFSPanel(TaskPanel):
         if dgroup is None:
             return self.get_defaultconfig()
 
-        conf = getattr(dgroup, self.configname, None)
+        conf = getattr(dgroup.config, self.configname, None)
         if conf is None:
-            # initial reading: start with default then take Athena Project values
             conf = self.get_defaultconfig()
 
-            bkg_kmax = conf.get('bkg_kmax', None)
-            fft_kmax = conf.get('fft_kmax', None)
-            if None in (bkg_kmax, fft_kmax):
-                e0 = conf.get('e0', -1)
-                emin = min(dgroup.energy)
-                if e0 is None or e0 < emin:
-                    e0 = getattr(dgroup, 'e0',  emin)
-                kmax = etok(max(dgroup.energy) - e0)
+        e0 = getattr(dgroup, 'e0', None)
+        if e0 is None:
+            e0 = conf.get('e0', None)
+        if e0 is None:
+            nconf = getattr(dgroup.config, 'xasnorm', {'e0': None})
+            e0 = nconf.get('e0', None)
+        if e0 is None:
+            e0 = min(dgroup.energy)
+        kmax = etok(max(dgroup.energy) - e0)
 
-                if bkg_kmax is None or bkg_kmax < 0:
-                    conf['bkg_kmax'] = kmax + 0.1
-                if fft_kmax is None or fft_kmax < 0:
-                    conf['fft_kmax'] = kmax - 1
+        bkg_kmax = conf.get('bkg_kmax', -1)
+        if bkg_kmax < 0 or bkg_kmax > kmax:
+            conf['bkg_kmax'] = 0.25*int(kmax*4.0 + 1.0)
 
-            if hasattr(dgroup, 'bkg_params'): # from Athena
-                conf['e0'] =  dgroup.bkg_params.e0
-                conf['rbkg'] =  dgroup.bkg_params.rbkg
-                conf['bkg_kmin'] =  dgroup.bkg_params.spl1
-                conf['bkg_kmax'] =  dgroup.bkg_params.spl2
-                conf['bkg_kweight'] =  dgroup.bkg_params.kw
-                conf['bkg_clamplo'] =  dgroup.bkg_params.clamp1
-                conf['bkg_clamphi'] =  dgroup.bkg_params.clamp2
+        fft_kmax = conf.get('fft_kmax', -1)
+        if fft_kmax < 0 or fft_kmax > kmax:
+            conf['fft_kmax'] = 0.25*int(kmax*4.0 - 1.0)
 
-            if hasattr(dgroup, 'fft_params'): # from Athena
-                conf['fft_kmin'] =  2.0
-                conf['fft_kmax'] =  None
-                conf['fft_dk'] =  4.0
-                conf['fft_kwindow'] =  'kaiser-bessel'
-                conf['fft_kweight'] =  2
-                if hasattr(dgroup.fft_params, 'kw'):
-                    conf[f'fft_kweight'] = getattr(dgroup.fft_params, 'kw')
-                for attr in ('kmin', 'kmax', 'dk', 'kwindow'):
-                    if hasattr(dgroup.fft_params, attr):
-                        conf[f'fft_{attr:s}'] = getattr(dgroup.fft_params, attr)
+        fft_rmin = conf.get('fft_rmin', -1)
+        if fft_rmin:
+            conf['fft_rmin'] = conf['rbkg']
 
-            setattr(dgroup, self.configname, conf)
+        setattr(dgroup.config, self.configname, conf)
         return conf
 
     def fill_form(self, dgroup):
@@ -363,26 +342,32 @@ class EXAFSPanel(TaskPanel):
 
         self.skip_process = True
         wids = self.wids
-        for attr in ('e0', 'rbkg', 'bkg_kmin', 'bkg_kmax',
-                     'bkg_kweight', 'fft_kmin', 'fft_kmax',
-                     'fft_kweight', 'fft_dk'):
+        for attr in ('e0', 'rbkg'):
             val = getattr(dgroup, attr, None)
             if val is None:
                 val = opts.get(attr, -1)
             wids[attr].SetValue(val)
 
-        for attr in ('bkg_clamplo', 'bkg_clamphi'):
-            val = getattr(dgroup, attr, None)
-            if val is None:
-                val = opts.get(attr, -1)
-                if val is None:
-                    val = '0'
+        for attr in ('bkg_kmin', 'bkg_kmax', 'bkg_kweight', 'fft_kmin',
+                     'fft_kmax', 'fft_kweight', 'fft_dk', 'fft_rmaxout',
+                     'plot_rmax'):
             try:
-                wids[attr].SetStringSelection("%d" % float(opts.get(attr, 0)))
+                wids[attr].SetValue(float(opts.get(attr)))
+            except:
+                pass
+
+        for attr in ('bkg_clamplo', 'bkg_clamphi'):
+            val = opts.get(attr, 0)
+            try:
+                val = float(val)
+            except:
+                if isinstance(val, str):
+                    val = ATHENA_CLAMPNAMES.get(val.lower(), 0)
+            try:
+                wids[attr].SetStringSelection("%d" % int(val))
             except:
                 print(f"could not set '{attr:s}' to {val}")
 
-
         for attr in ('fft_kwindow', 'plotone_op', 'plotsel_op', 'plotalt_op'):
             if attr in opts:
                 wids[attr].SetStringSelection(opts[attr])
@@ -412,13 +397,13 @@ class EXAFSPanel(TaskPanel):
         wids = self.wids
         for attr in ('e0', 'rbkg', 'bkg_kmin', 'bkg_kmax',
                      'bkg_kweight', 'fft_kmin', 'fft_kmax',
-                     'fft_kweight', 'fft_dk',
+                     'fft_kweight', 'fft_dk',  'fft_rmaxout',
                      'fft_rmin', 'fft_rmax', 'fft_dr',
-                     'plot_kweight',
+                     'plot_kweight', 'plot_rmax',
                      'plot_kweight_alt', 'plot_voffset'):
             conf[attr] = wids[attr].GetValue()
 
-        for attr in ('bkg_clamplo', 'bkg_clamphi'):
+        for attr in ('bkg_clamplo', 'bkg_clamphi', 'plot_win'):
             conf[attr] = int(wids[attr].GetStringSelection())
 
         for attr in ('fft_kwindow', 'fft_rwindow', 'plotone_op',
@@ -429,10 +414,9 @@ class EXAFSPanel(TaskPanel):
         if as_copy:
             conf = copy.deepcopy(conf)
         if dgroup is not None:
-            setattr(dgroup, self.configname, conf)
+            setattr(dgroup.config, self.configname, conf)
         return conf
 
-
     def onSaveConfigBtn(self, evt=None):
         self.set_defaultconfig(self.read_form())
 
@@ -442,8 +426,23 @@ class EXAFSPanel(TaskPanel):
         def copy_attrs(*args):
             return {a: conf[a] for a in args}
         name = str(name)
-        if name in ('e0', 'rbkg', 'bkg_kweight', 'fft_kweight'):
+        set_e0 = set_rbkg = False
+        if name == 'all':
+            opts = copy_attrs( 'e0', 'rbkg', 'bkg_kweight', 'fft_kweight',
+                               'bkg_kmin', 'bkg_kmax', 'bkg_clamplo',
+                               'bkg_clamphi', 'fft_kmin', 'fft_kmax',
+                               'fft_kwindow', 'fft_dk', 'fft_rmin',
+                               'fft_rmax', 'fft_rmaxout', 'fft_rwindow',
+                               'fft_dr')
+            set_e0 = True
+            set_rbkg = True
+
+        elif name in ('e0', 'rbkg', 'bkg_kweight', 'fft_kweight'):
             opts = copy_attrs(name)
+            if name == 'e0':
+                set_e0 = True
+            elif name == 'rbkg':
+                set_rbkg = True
         elif name == 'bkg_krange':
             opts = copy_attrs('bkg_kmin', 'bkg_kmax')
         elif name == 'bkg_clamp':
@@ -454,6 +453,8 @@ class EXAFSPanel(TaskPanel):
             opts = copy_attrs('fft_kwindow', 'fft_dk')
         elif name == 'fft_rrange':
             opts = copy_attrs('fft_rmin', 'fft_rmax')
+        elif name == 'fft_rmaxout':
+            opts = copy_attrs('fft_rmaxout',)
         elif name == 'fft_rwindow':
             opts = copy_attrs('fft_rwindow', 'fft_dr')
 
@@ -462,6 +463,11 @@ class EXAFSPanel(TaskPanel):
             grp = self.controller.get_group(groupname)
             if grp != self.controller.group and not grp.is_frozen:
                 self.update_config(opts, dgroup=grp)
+                if set_e0:
+                    grp.e0 = opts['e0']
+                if set_rbkg:
+                    grp.rbkg = opts['rbkg']
+                self.process(dgroup=grp, read_form=False)
 
 
     def _set_frozen(self, frozen):
@@ -496,11 +502,11 @@ class EXAFSPanel(TaskPanel):
         plotter = self.onPlotSel if self.last_plot=='selected' else self.onPlotOne
         plotter()
 
-    def process(self, dgroup=None, read_form=True, **kws):
+    def process(self, dgroup=None, read_form=True, force=False, **kws):
         conf = {}
         if dgroup is not None:
             self.dgroup = dgroup
-            conf = getattr(dgroup, self.configname, None)
+            conf = getattr(dgroup.config, self.configname, None)
             if conf is None:
                 conf = self.get_config(dgroup=dgroup)
 
@@ -511,8 +517,8 @@ class EXAFSPanel(TaskPanel):
         if dgroup is None or 'fft_kwindow' not in conf:
             return
 
-        if 'group' not in conf:
-            conf['group'] = dgroup.groupname
+        # if 'group' not in conf:
+        conf['group'] = dgroup.groupname
 
         try:
             txt = autobk_cmd.format(**conf)
@@ -520,7 +526,6 @@ class EXAFSPanel(TaskPanel):
             conf.update(self.read_form())
 
         bkgpars = []
-        # print(" EXAFS Process #1 e0= ", conf.get('e0', -10.))
         for attr in ('e0', 'rbkg', 'bkg_kmin', 'bkg_kmax',
                      'bkg_kweight', 'bkg_clamplo', 'bkg_clamphi'):
             val = conf.get(attr, 0.0)
@@ -528,15 +533,15 @@ class EXAFSPanel(TaskPanel):
                 val = -1.0
             bkgpars.append("%.3f" % val)
         bkgpars = ':'.join(bkgpars)
-
-        if bkgpars != self.last_process_bkg.get(self.dgroup.groupname, ''):
+        lastpars = self.last_process_bkg.get(self.dgroup.groupname, '')
+        if force or (bkgpars != lastpars):
             self.larch_eval(autobk_cmd.format(**conf))
             self.last_process_bkg[self.dgroup.groupname] = bkgpars
             self.last_process_fft[self.dgroup.groupname] = ''
 
         fftpars = [conf['fft_kwindow'], conf['fft_rwindow']]
         for attr in ('fft_kmin', 'fft_kmax', 'fft_kweight', 'fft_dk',
-                     'fft_rmin', 'fft_rmax', 'fft_dr'):
+                     'fft_rmin', 'fft_rmax', 'fft_dr', 'fft_rmaxout'):
             fftpars.append("%.3f" % conf.get(attr, 0.0))
         fftpars = ':'.join(fftpars)
         if fftpars != self.last_process_fft.get(self.dgroup.groupname, ''):
@@ -544,7 +549,7 @@ class EXAFSPanel(TaskPanel):
             self.larch_eval(xftr_cmd.format(**conf))
             self.last_process_fft[self.dgroup.groupname] = fftpars
 
-        setattr(dgroup, self.configname, conf)
+        setattr(dgroup.config, self.configname, conf)
 
     def plot(self, dgroup=None):
         if self.skip_plotting:
@@ -566,15 +571,15 @@ class EXAFSPanel(TaskPanel):
         cmd2 =  PlotCmds[conf['plotalt_op']]
         if cmd2 is not None:
             cmd2 = cmd2.replace('plot_kweight', 'plot_kweight_alt')
-            cmd2 = cmd2 + ", win=2, title={title:s})"
+            cmd2 = cmd2 + ", win={plot_win:d}, title={title:s})"
             cmd = "%s\n%s" % (cmd, cmd2)
             self.controller.get_display(win=2)
 
         self.larch_eval(cmd.format(**conf))
         self.last_plot = 'one'
-        self.parent.SetFocus()
-        # if evt is not None:
-        #     evt.Skip()
+
+        self.controller.set_focus()
+
 
     def onPlotSel(self, evt=None):
         if self.skip_plotting:
@@ -605,6 +610,5 @@ class EXAFSPanel(TaskPanel):
 
         self.larch_eval("redraw(win=1, show_legend=True)")
         self.last_plot = 'selected'
-        self.parent.SetFocus()
-        # if evt is not None:
-        #    evt.Skip()
+
+        self.controller.set_focus()
