@@ -17,91 +17,108 @@ from lmfit.printfuncs import gformat, fit_report
 
 from larch import Group
 from larch.math import index_of
+from larch.xafs import etok, ktoe
 
 from larch.wxlib import (BitmapButton, FloatCtrl, FloatSpin, ToggleButton,
-                         GridPanel, get_icon, SimpleText, pack, Button, HLine,
-                         Choice, Check, CEN, LEFT, Font, FONTSIZE,
-                         MenuItem, FRAMESTYLE, GUIColors, FileSave,
-                         EditableListBox)
+                         GridPanel, get_icon, SimpleText, pack, Button,
+                         HLine, Choice, Check, CEN, LEFT, Font, FONTSIZE,
+                         FONTSIZE_FW, MenuItem, FRAMESTYLE, COLORS,
+                         set_color, FileSave, EditableListBox,
+                         DataTableGrid)
 
-from .taskpanel import TaskPanel, DataTableGrid
+from .taskpanel import TaskPanel
+from .config import ARRAYS, Linear_ArrayChoices, PlotWindowChoices
 from larch.io.columnfile import write_ascii
 
 np.seterr(all='ignore')
 
 # plot options:
-norm   = 'Normalized \u03bC(E)'
-rawmu  = 'Raw \u03bC(E)'
-dmude  = 'd\u03bC(E)/dE'
-chik   = '\u03c7(k)'
-noplot = '<no plot>'
-noname = '<none>'
-
-FitSpace_Choices = [norm, rawmu, dmude, chik]
 Plot_Choices = ['Data + Sum', 'Data + Sum + Components']
 
 DVSTYLE = dv.DV_SINGLE|dv.DV_VERT_RULES|dv.DV_ROW_LINES
 
-defaults = dict(elo=-5.e5, ehi=5.e5, elo_rel=-40, ehi_rel=110,
-                fitspace=norm, all_combos=True, sum_to_one=True,
-                show_e0=False, show_fitrange=True)
+MAX_COMPONENTS = 12
 
-MAX_COMPONENTS = 20
-
-def make_lcfplot(dgroup, form, nfit=0):
+def make_lcfplot(dgroup, form, with_fit=True, nfit=0):
     """make larch plot commands to plot LCF fit from form"""
     form['group'] = dgroup.groupname
     form['filename'] = dgroup.filename
-
     form['nfit'] = nfit
     form['label'] = label = 'Fit #%2.2d' % (nfit+1)
-    form['plotopt'] = 'show_norm=True'
-    if form['arrayname'] == 'dmude':
-        form['plotopt'] = 'show_deriv=True'
 
-    erange = form['ehi'] - form['elo']
-    form['pemin'] = 10*int( (form['elo'] - 5 - erange/4.0) / 10.0)
-    form['pemax'] = 10*int( (form['ehi'] + 5 + erange/4.0) / 10.0)
+    if 'win' not in form: form['win'] = 1
+    kspace = form['arrayname'].startswith('chi')
+    if kspace:
+        kw = 0
+        if len(form['arrayname']) > 3:
+            kw = int(form['arrayname'][3:])
+        form['plotopt'] = 'kweight=%d' % kw
 
+        cmds = ["""plot_chik({group:s}, {plotopt:s}, delay_draw=False, label='data',
+         show_window=False, title='{filename:s}, {label:s}', win={win:d})"""]
 
-    cmds = ["""plot_mu({group:s}, {plotopt:s}, delay_draw=True, label='data',
-    emin={pemin:.1f}, emax={pemax:.1f}, title='{filename:s}, {label:s}')"""]
+    else:
+        form['plotopt'] = 'show_norm=False'
+        if form['arrayname'] == 'norm':
+            form['plotopt'] = 'show_norm=True'
+        elif form['arrayname'] == 'flat':
+            form['plotopt'] = 'show_flat=True'
+        elif form['arrayname'] == 'dmude':
+            form['plotopt'] = 'show_deriv=True'
 
-    if hasattr(dgroup, 'lcf_result'):
+        erange = form['ehi'] - form['elo']
+        form['pemin'] = 10*int( (form['elo'] - 5 - erange/4.0) / 10.0)
+        form['pemax'] = 10*int( (form['ehi'] + 5 + erange/4.0) / 10.0)
+
+        cmds = ["""plot_mu({group:s}, {plotopt:s}, delay_draw=True, label='data',
+        emin={pemin:.1f}, emax={pemax:.1f}, title='{filename:s}, {label:s}', win={win:d})"""]
+
+    if with_fit and hasattr(dgroup, 'lcf_result'):
         with_comps = True # "Components" in form['plotchoice']
         delay = 'delay_draw=True' if with_comps else 'delay_draw=False'
         xarr = "{group:s}.lcf_result[{nfit:d}].xdata"
         yfit = "{group:s}.lcf_result[{nfit:d}].yfit"
         ycmp = "{group:s}.lcf_result[{nfit:d}].ycomps"
-        cmds.append("plot(%s, %s, label='%s', zorder=30, %s)" % (xarr, yfit, label, delay))
+        cmds.append("plot(%s, %s, label='%s', zorder=30, %s, win={win:d})" % (xarr, yfit, label, delay))
         ncomps = len(dgroup.lcf_result[nfit].ycomps)
         if with_comps:
             for i, key in enumerate(dgroup.lcf_result[nfit].ycomps):
                 delay = 'delay_draw=False' if i==(ncomps-1) else 'delay_draw=True'
-                cmds.append("plot(%s, %s['%s'], label='%s', %s)" % (xarr, ycmp, key, key, delay))
+                cmds.append("plot(%s, %s['%s'], label='%s', %s, win={win:d})" % (xarr, ycmp, key, key, delay))
 
     # if form['show_e0']:
     #     cmds.append("plot_axvline({e0:1f}, color='#DDDDCC', zorder=-10)")
     if form['show_fitrange']:
-        cmds.append("plot_axvline({elo:1f}, color='#EECCCC', zorder=-10)")
-        cmds.append("plot_axvline({ehi:1f}, color='#EECCCC', zorder=-10)")
+        cmds.append("plot_axvline({elo:1f}, color='#EECCCC', zorder=-10, win={win:d})")
+        cmds.append("plot_axvline({ehi:1f}, color='#EECCCC', zorder=-10, win={win:d})")
 
     script = "\n".join(cmds)
     return script.format(**form)
 
 class LinComboResultFrame(wx.Frame):
-    def __init__(self, parent=None,  **kws):
+    def __init__(self, parent=None,  datagroup=None, mainpanel=None, **kws):
         wx.Frame.__init__(self, None, -1, title='Linear Combination Results',
-                          style=FRAMESTYLE, size=(825, 675), **kws)
+                          style=FRAMESTYLE, size=(925, 675), **kws)
         self.parent = parent
+        self.mainpanel = mainpanel
+        self.datagroup = datagroup
         self.datasets = {}
-        self.datagroup = None
-        self.form = None
-        self.larch_eval = None
+        self.form = self.mainpanel.read_form()
+        self.larch_eval = self.mainpanel.larch_eval
         self.current_fit = 0
         self.createMenus()
         self.build()
 
+        if self.mainpanel is not None:
+            symtab = self.mainpanel.larch.symtable
+            xasgroups = getattr(symtab, '_xasgroups', None)
+            if xasgroups is not None:
+                for dname, dgroup in xasgroups.items():
+                    dgroup = getattr(symtab, dgroup, None)
+                    hist  = getattr(dgroup, 'lcf_result', None)
+                    if hist is not None:
+                        self.add_results(dgroup, show=False)
+
     def createMenus(self):
         self.menubar = wx.MenuBar()
         fmenu = wx.Menu()
@@ -131,25 +148,28 @@ class LinComboResultFrame(wx.Frame):
         splitter = wx.SplitterWindow(self, style=wx.SP_LIVE_UPDATE)
         splitter.SetMinimumPaneSize(200)
 
-        dl = self.datalistbox = EditableListBox(splitter, self.ShowDataSet,
+        dl = self.filelist = EditableListBox(splitter, self.ShowDataSet,
                                            size=(250, -1))
+        set_color(self.filelist, 'list_fg', bg='list_bg')
+
 
-        dl.SetOwnBackgroundColour(wx.Colour(235, 235, 200))
-        dl.SetBackgroundColour(wx.Colour(235, 235, 200))
-        dl.SetForegroundColour(wx.Colour(5, 5, 50))
-        dl.SetOwnForegroundColour(wx.Colour(5, 5, 50))
         panel = scrolled.ScrolledPanel(splitter)
 
         self.SetMinSize((650, 600))
-        self.colors = GUIColors()
 
-        #
+        self.font_fixedwidth = wx.Font(FONTSIZE_FW, wx.MODERN, wx.NORMAL, wx.BOLD)
+
         self.wids = wids = {}
         wids['plot_one'] = Button(panel, 'Plot This Fit', size=(125, -1),
                                   action=self.onPlotOne)
         wids['plot_sel'] = Button(panel, 'Plot N Best Fits', size=(125, -1),
                                   action=self.onPlotSel)
 
+        wids['plot_win'] = Choice(panel, choices=PlotWindowChoices,
+                                  action=self.onPlotOne, size=(60, -1))
+        wids['plot_win'].SetStringSelection('1')
+
+        wids['plot_wtitle'] = SimpleText(panel, 'Plot Window: ')
         wids['plot_ntitle'] = SimpleText(panel, 'N fits to plot: ')
 
         wids['plot_nchoice'] = Choice(panel, size=(60, -1),
@@ -159,8 +179,9 @@ class LinComboResultFrame(wx.Frame):
         wids['data_title'] = SimpleText(panel, 'Linear Combination Result: <> ',
                                         font=Font(FONTSIZE+2),
                                         size=(400, -1),
-                                        colour=self.colors.title, style=LEFT)
+                                        colour=COLORS['title'], style=LEFT)
         wids['nfits_title'] = SimpleText(panel, 'showing 5 best fits')
+        wids['fitspace_title'] = SimpleText(panel, 'Array Fit: ')
 
         copts = dict(size=(125, 30), default=True, action=self.onPlotOne)
         # wids['show_e0'] = Check(panel, label='show E0?', **copts)
@@ -171,18 +192,20 @@ class LinComboResultFrame(wx.Frame):
 
         irow += 1
         sizer.Add(wids['nfits_title'],     (irow, 0), (1, 1), LEFT)
+        sizer.Add(wids['fitspace_title'],  (irow, 1), (1, 2), LEFT)
 
 
         irow += 1
         self.wids['paramstitle'] = SimpleText(panel, '[[Parameters]]',
                                               font=Font(FONTSIZE+2),
-                                              colour=self.colors.title, style=LEFT)
-        sizer.Add(self.wids['paramstitle'], (irow, 0), (1, 1), LEFT)
+                                              colour=COLORS['title'], style=LEFT)
+        sizer.Add(self.wids['paramstitle'], (irow, 0), (1, 3), LEFT)
 
 
         pview = self.wids['params'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        pview.SetFont(self.font_fixedwidth)
         pview.SetMinSize((475, 200))
-        pview.AppendTextColumn(' Parameter ', width=230)
+        pview.AppendTextColumn(' Parameter ', width=200)
         pview.AppendTextColumn(' Best-Fit Value', width=120)
         pview.AppendTextColumn(' Standard Error ', width=120)
         for col in range(3):
@@ -193,28 +216,35 @@ class LinComboResultFrame(wx.Frame):
             this.Sortable = isort
             this.Alignment = this.Renderer.Alignment = align
 
-
         irow += 1
-        sizer.Add(self.wids['params'],       (irow,   0), (4, 1), LEFT)
-        sizer.Add(self.wids['plot_one'],     (irow,   1), (1, 2), LEFT)
-        sizer.Add(self.wids['plot_sel'],     (irow+1, 1), (1, 2), LEFT)
-        sizer.Add(self.wids['plot_ntitle'],  (irow+2, 1), (1, 1), LEFT)
-        sizer.Add(self.wids['plot_nchoice'], (irow+2, 2), (1, 1), LEFT)
-        # sizer.Add(self.wids['show_e0'],      (irow+3, 1), (1, 2), LEFT)
-        sizer.Add(self.wids['show_fitrange'],(irow+3, 1), (1, 2), LEFT)
+        sizer.Add(self.wids['params'],       (irow,   0), (7, 2), LEFT)
+        sizer.Add(self.wids['plot_one'],     (irow,   2), (1, 2), LEFT)
 
-        irow += 4
+        sizer.Add(self.wids['plot_wtitle'],  (irow+1, 2), (1, 1), LEFT)
+        sizer.Add(self.wids['plot_win'],     (irow+1, 3), (1, 1), LEFT)
+
+
+        sizer.Add(self.wids['show_fitrange'],(irow+2, 2), (1, 2), LEFT)
+        sizer.Add((5, 5),                    (irow+3, 2), (1, 2), LEFT)
+        sizer.Add(self.wids['plot_sel'],     (irow+4, 2), (1, 2), LEFT)
+        sizer.Add(self.wids['plot_ntitle'],  (irow+5, 2), (1, 1), LEFT)
+        sizer.Add(self.wids['plot_nchoice'], (irow+5, 3), (1, 1), LEFT)
+        # sizer.Add(self.wids['show_e0'],      (irow+3, 1), (1, 2), LEFT)
+
+
+        irow += 7
         sizer.Add(HLine(panel, size=(675, 3)), (irow, 0), (1, 4), LEFT)
 
         sview = self.wids['stats'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        sview.SetFont(self.font_fixedwidth)
         sview.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED, self.onSelectFitStat)
         sview.AppendTextColumn(' Fit #', width=50)
-        sview.AppendTextColumn(' N_vary', width=50)
+        sview.AppendTextColumn(' N_vary', width=60)
         sview.AppendTextColumn(' N_eval', width=60)
-        sview.AppendTextColumn(' \u03c7\u00B2', width=110)
-        sview.AppendTextColumn(' \u03c7\u00B2_reduced', width=110)
-        sview.AppendTextColumn(' Akaike Info', width=110)
-        sview.AppendTextColumn(' Bayesian Info', width=110)
+        sview.AppendTextColumn(' \u03c7\u00B2', width=90)
+        sview.AppendTextColumn(' \u03c7\u00B2_reduced', width=90)
+        sview.AppendTextColumn(' R Factor', width=90)
+        sview.AppendTextColumn(' Akaike Info', width=90)
 
         for col in range(sview.ColumnCount):
             this = sview.Columns[col]
@@ -224,11 +254,11 @@ class LinComboResultFrame(wx.Frame):
             this.Sortable = isort
             this.Alignment = this.Renderer.Alignment = align
 
-        sview.SetMinSize((675, 175))
+        sview.SetMinSize((700, 175))
 
         irow += 1
         title = SimpleText(panel, '[[Fit Statistics]]', font=Font(FONTSIZE+2),
-                           colour=self.colors.title, style=LEFT)
+                           colour=COLORS['title'], style=LEFT)
         sizer.Add(title, (irow, 0), (1, 4), LEFT)
 
         irow += 1
@@ -239,7 +269,7 @@ class LinComboResultFrame(wx.Frame):
 
         irow += 1
         title = SimpleText(panel, '[[Weights]]', font=Font(FONTSIZE+2),
-                           colour=self.colors.title, style=LEFT)
+                           colour=COLORS['title'], style=LEFT)
         sizer.Add(title, (irow, 0), (1, 4), LEFT)
         self.wids['weightspanel'] = ppan = wx.Panel(panel)
 
@@ -247,7 +277,7 @@ class LinComboResultFrame(wx.Frame):
         os = wx.BoxSizer(wx.VERTICAL)
         os.Add(p1, 1, 3)
         pack(ppan, os)
-        ppan.SetMinSize((675, 175))
+        ppan.SetMinSize((700, 175))
 
         irow += 1
         sizer.Add(ppan, (irow, 0), (1, 4), LEFT)
@@ -258,7 +288,7 @@ class LinComboResultFrame(wx.Frame):
         pack(panel, sizer)
         panel.SetupScrolling()
 
-        splitter.SplitVertically(self.datalistbox, panel, 1)
+        splitter.SplitVertically(self.filelist, panel, 1)
 
         mainsizer = wx.BoxSizer(wx.VERTICAL)
         mainsizer.Add(splitter, 1, wx.GROW|wx.ALL, 5)
@@ -276,8 +306,8 @@ class LinComboResultFrame(wx.Frame):
 
     def add_results(self, dgroup, form=None, larch_eval=None, show=True):
         name = dgroup.filename
-        if name not in self.datalistbox.GetItems():
-            self.datalistbox.Append(name)
+        if name not in self.filelist.GetItems():
+            self.filelist.Append(name)
         self.datasets[name] = dgroup
         if show:
             self.show_results(datagroup=dgroup, form=form, larch_eval=larch_eval)
@@ -291,26 +321,33 @@ class LinComboResultFrame(wx.Frame):
             self.larch_eval = larch_eval
 
         form = self.form
+        if form is None:
+            form = self.mainpanel.read_form()
         datagroup = self.datagroup
-        lcf_history = getattr(self.datagroup, 'lcf_history', [])
 
         wids = self.wids
         wids['data_title'].SetLabel('Linear Combination Result: %s ' %  self.datagroup.filename)
         wids['show_fitrange'].SetValue(form['show_fitrange'])
 
         wids['stats'].DeleteAllItems()
+        if not hasattr(self.datagroup, 'lcf_result'):
+            return
         results = self.datagroup.lcf_result[:20]
         self.nresults = len(results)
-        wids['nfits_title'].SetLabel('showing %i best results' % self.nresults)
+        wids['nfits_title'].SetLabel('showing %i best results' % (self.nresults,))
+        wids['fitspace_title'].SetLabel('Array Fit: %s' % ARRAYS.get(results[0].arrayname, 'unknown'))
 
         for i, res in enumerate(results):
+            res.result.rfactor = getattr(res, 'rfactor', 0)
             args = ['%2.2d' % (i+1)]
-            for attr in ('nvarys', 'nfev', 'chisqr', 'redchi', 'aic', 'bic'):
+            for attr in ('nvarys', 'nfev', 'chisqr', 'redchi', 'rfactor', 'aic'):
                 val = getattr(res.result, attr)
                 if isinstance(val, int):
                     val = '%d' % val
+                elif attr in ('aic',):
+                    val = "%.2f" % val
                 else:
-                    val = gformat(val, 12)
+                    val = gformat(val, 10)
                 args.append(val)
             wids['stats'].AppendItem(tuple(args))
 
@@ -318,8 +355,10 @@ class LinComboResultFrame(wx.Frame):
         wpan.DestroyChildren()
 
         wview = self.wids['weights'] = dv.DataViewListCtrl(wpan, style=DVSTYLE)
+        wview.SetFont(self.font_fixedwidth)
         wview.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED, self.onSelectFitParam)
         wview.AppendTextColumn(' Fit #', width=50)
+        wview.AppendTextColumn(' E shift', width=75)
 
         for i, cname in enumerate(form['comp_names']):
             wview.AppendTextColumn(cname, width=100)
@@ -334,7 +373,7 @@ class LinComboResultFrame(wx.Frame):
             this.Alignment = this.Renderer.Alignment = align
 
         for i, res in enumerate(results):
-            args = ['%2.2d' % (i+1)]
+            args = ['%2.2d' % (i+1), "%.4f" % res.params['e0_shift'].value]
             for cname in form['comp_names'] + ['total']:
                 val = '--'
                 if cname in res.params:
@@ -346,7 +385,7 @@ class LinComboResultFrame(wx.Frame):
         os.Add(wview, 1, wx.GROW|wx.ALL)
         pack(wpan, os)
 
-        wview.SetMinSize((675, 200))
+        wview.SetMinSize((700, 500))
         s1, s2 = self.GetSize()
         if s2 % 2 == 0:
             s2 = s2 + 1
@@ -372,42 +411,46 @@ class LinComboResultFrame(wx.Frame):
         fit_result = self.datagroup.lcf_result[n]
         self.current_fit = n
         wids = self.wids
-        wids['nfits_title'].SetLabel('Showing Fit # %2.2d' % (n+1))
+        wids['nfits_title'].SetLabel('Showing Fit # %2.2d' % (n+1,))
+        wids['fitspace_title'].SetLabel('Array Fit: %s' % ARRAYS.get(fit_result.arrayname, 'unknown'))
         wids['paramstitle'].SetLabel('[[Parameters for Fit # %2.2d]]' % (n+1))
 
         wids['params'].DeleteAllItems()
 
         for pname, par in fit_result.params.items():
-            args = [pname, gformat(par.value, 12), '--']
+            args = [pname, gformat(par.value, 10), '--']
             if par.stderr is not None:
-                args[2] = gformat(par.stderr, 12)
+                args[2] = gformat(par.stderr, 10)
             self.wids['params'].AppendItem(tuple(args))
 
     def onPlotOne(self, evt=None):
-        if self.form is None or self.larch_eval is None:
-            return
-
+        self.form = self.mainpanel.read_form()
         self.form['show_fitrange'] = self.wids['show_fitrange'].GetValue()
+        self.form['win'] = int(self.wids['plot_win'].GetStringSelection())
         self.larch_eval(make_lcfplot(self.datagroup,
                                      self.form, nfit=self.current_fit))
+        self.parent.controller.set_focus(topwin=self)
 
     def onPlotSel(self, evt=None):
         if self.form is None or self.larch_eval is None:
             return
         self.form['show_fitrange'] = self.wids['show_fitrange'].GetValue()
+        self.form['win'] = int(self.wids['plot_win'].GetStringSelection())
         form = self.form
         dgroup = self.datagroup
 
         form['plotopt'] = 'show_norm=True'
         if form['arrayname'] == 'dmude':
             form['plotopt'] = 'show_deriv=True'
+        if form['arrayname'] == 'flat':
+            form['plotopt'] = 'show_flat=True'
 
         erange = form['ehi'] - form['elo']
         form['pemin'] = 10*int( (form['elo'] - 5 - erange/4.0) / 10.0)
         form['pemax'] = 10*int( (form['ehi'] + 5 + erange/4.0) / 10.0)
 
         cmds = ["""plot_mu({group:s}, {plotopt:s}, delay_draw=True, label='data',
-        emin={pemin:.1f}, emax={pemax:.1f}, title='{filename:s}')"""]
+        emin={pemin:.1f}, emax={pemax:.1f}, title='{filename:s}', win={win:d})"""]
 
         nfits = int(self.wids['plot_nchoice'].GetStringSelection())
         for i in range(nfits):
@@ -417,14 +460,13 @@ class LinComboResultFrame(wx.Frame):
             lab = 'Fit #%2.2d' % (i+1)
             cmds.append("plot(%s, %s, label='%s', zorder=30, %s)" % (xarr, yfit, lab, delay))
 
-        # if form['show_e0']:
-        #     cmds.append("plot_axvline({e0:1f}, color='#DDDDCC', zorder=-10)")
         if form['show_fitrange']:
             cmds.append("plot_axvline({elo:1f}, color='#EECCCC', zorder=-10)")
             cmds.append("plot_axvline({ehi:1f}, color='#EECCCC', zorder=-10)")
 
         script = "\n".join(cmds)
         self.larch_eval(script.format(**form))
+        self.parent.controller.set_focus(topwin=self)
 
     def onSaveGroupFit(self, evt=None):
         "Save Fit and Compoents for current fit to Data File"
@@ -578,12 +620,12 @@ class LinComboResultFrame(wx.Frame):
                 label = (label + ' '*25)[:25]
             dat = [label]
             for attr in ('nvarys', 'chisqr', 'redchi', 'aic', 'bic'):
-                dat.append(gformat(getattr(res.result, attr), 12))
+                dat.append(gformat(getattr(res.result, attr), 10))
             for cname in form['comp_names'] + ['total']:
                 val = 0
                 if cname in res.params:
                     val = res.params[cname].value
-                dat.append(gformat(val, 12))
+                dat.append(gformat(val, 10))
             out.append(', '.join(dat))
         out.append('')
 
@@ -593,53 +635,60 @@ class LinComboResultFrame(wx.Frame):
 class LinearComboPanel(TaskPanel):
     """Liear Combination Panel"""
     def __init__(self, parent, controller, **kws):
-        TaskPanel.__init__(self, parent, controller,
-                           configname='lincombo_config',
-                           title='Linear Combination Analysis',
-                           config=defaults, **kws)
+        TaskPanel.__init__(self, parent, controller, panel='lincombo', **kws)
 
     def process(self, dgroup, **kws):
         """ handle linear combo processing"""
         if self.skip_process:
             return
-        return self.read_form()
+        form = self.read_form()
+        conf = self.get_config(dgroup)
+        for key in ('elo', 'ehi', 'max_ncomps', 'fitspace', 'all_combos',
+                     'vary_e0', 'sum_to_one', 'show_fitrange'):
+            conf[key]  = form[key]
+        self.update_config(conf, dgroup=dgroup)
+
 
     def build_display(self):
         panel = self.panel
         wids = self.wids
         self.skip_process = True
 
-        wids['fitspace'] = Choice(panel, choices=FitSpace_Choices,
-                                  size=(175, -1))
-        wids['fitspace'].SetStringSelection(norm)
+        wids['fitspace'] = Choice(panel, choices=list(Linear_ArrayChoices.keys()),
+                                 action=self.onFitSpace, size=(175, -1))
+        wids['fitspace'].SetSelection(0)
 
         add_text = self.add_text
 
-        opts = dict(digits=2, increment=1.0, relative_e0=False) # True)
+        opts = dict(digits=2, increment=1.0, relative_e0=False)
+        defaults = self.get_defaultconfig()
 
-        elo_wids = self.add_floatspin('elo', value=defaults['elo'], **opts)
-        ehi_wids = self.add_floatspin('ehi', value=defaults['ehi'], **opts)
+        self.make_fit_xspace_widgets(elo=defaults['elo_rel'], ehi=defaults['ehi_rel'])
 
-        wids['fit_group'] = Button(panel, 'Fit this Group', size=(175, -1),
+        wids['fit_group'] = Button(panel, 'Fit this Group', size=(150, -1),
                                    action=self.onFitOne)
         wids['fit_selected'] = Button(panel, 'Fit Selected Groups', size=(175, -1),
                                       action=self.onFitAll)
 
-        wids['add_selected'] = Button(panel, 'Use Selected Groups as Components',
-                                      size=(300, -1),
-                                      action=self.onUseSelected)
+        wids['fit_group'].Disable()
+        wids['fit_selected'].Disable()
 
-        wids['saveconf'] = Button(panel, 'Save as Default Settings', size=(225, -1),
-                                  action=self.onSaveConfigBtn)
+        wids['show_results'] = Button(panel, 'Show Fit Results',
+                                      action=self.onShowResults, size=(150, -1))
+        wids['show_results'].Disable()
+
+        wids['add_selected'] = Button(panel, 'Use Selected Groups as Components',
+                                      size=(300, -1), action=self.onUseSelected)
 
         opts = dict(default=True, size=(75, -1), action=self.onPlotOne)
 
         wids['show_fitrange'] = Check(panel, label='show?', **opts)
 
-        wids['sum_to_one'] = Check(panel, label='Weights Must Sum to 1?', default=True)
+        wids['vary_e0'] = Check(panel, label='Allow energy shift in fit?', default=False)
+        wids['sum_to_one'] = Check(panel, label='Weights Must Sum to 1?', default=False)
         wids['all_combos'] = Check(panel, label='Fit All Combinations?', default=True)
-        max_ncomps = self.add_floatspin('max_ncomps', value=10, digits=0, increment=1,
-                                        min_val=0, max_val=20, size=(60, -1),
+        max_ncomps = self.add_floatspin('max_ncomps', value=5, digits=0, increment=1,
+                                        min_val=0, max_val=MAX_COMPONENTS, size=(60, -1),
                                         with_pin=False)
 
         panel.Add(SimpleText(panel, 'Linear Combination Analysis',
@@ -647,18 +696,40 @@ class LinearComboPanel(TaskPanel):
 
         add_text('Array to Fit: ', newrow=True)
         panel.Add(wids['fitspace'], dcol=3)
+        panel.Add(wids['show_results'])
 
-        add_text('Fit Energy Range: ')
-        panel.Add(elo_wids)
+        panel.Add(wids['fitspace_label'], newrow=True)
+        panel.Add(self.elo_wids)
         add_text(' : ', newrow=False)
-        panel.Add(ehi_wids)
+        panel.Add(self.ehi_wids)
         panel.Add(wids['show_fitrange'])
 
         panel.Add(HLine(panel, size=(625, 3)), dcol=5, newrow=True)
-        add_text('Run Fit: ')
+
+        add_text('Build Model : ')
+        panel.Add(wids['add_selected'], dcol=4)
+
+        collabels = [' File /Group Name   ', 'weight', 'min', 'max']
+        colsizes = [325, 100, 100, 100]
+        coltypes = ['str', 'float:12,4', 'float:12,4', 'float:12,4']
+        coldefs  = ['', 1.0/MAX_COMPONENTS, 0.0, 1.0]
+
+        self.font_fixedwidth = wx.Font(FONTSIZE_FW, wx.MODERN, wx.NORMAL, wx.BOLD)
+        wids['table'] = DataTableGrid(panel, nrows=MAX_COMPONENTS,
+                                      collabels=collabels,
+                                      datatypes=coltypes, defaults=coldefs,
+                                      colsizes=colsizes)
+
+        wids['table'].SetMinSize((700, 250))
+        wids['table'].SetFont(self.font_fixedwidth)
+        panel.Add(wids['table'], newrow=True, dcol=6)
+
+        panel.Add(HLine(panel, size=(625, 3)), dcol=5, newrow=True)
+        add_text('Fit with this Model: ')
         panel.Add(wids['fit_group'], dcol=2)
         panel.Add(wids['fit_selected'], dcol=3)
         add_text('Fit Options: ')
+        panel.Add(wids['vary_e0'], dcol=2)
         panel.Add(wids['sum_to_one'], dcol=2)
         panel.Add((10, 10), dcol=1, newrow=True)
         panel.Add(wids['all_combos'], dcol=2)
@@ -666,25 +737,7 @@ class LinearComboPanel(TaskPanel):
         panel.Add(max_ncomps, dcol=2)
 
         panel.Add(HLine(panel, size=(625, 3)), dcol=5, newrow=True)
-
-        add_text('Components: ')
-        panel.Add(wids['add_selected'], dcol=4)
-
-        collabels = [' File /Group Name   ', 'weight', 'min', 'max']
-        colsizes = [300, 80, 80, 80]
-        coltypes = ['str', 'float:12,4', 'float:12,4', 'float:12,4']
-        coldefs  = ['', 1.0/MAX_COMPONENTS, 0.0, 1.0]
-
-        wids['table'] = DataTableGrid(panel, nrows=MAX_COMPONENTS,
-                                      collabels=collabels,
-                                      datatypes=coltypes, defaults=coldefs,
-                                      colsizes=colsizes)
-
-        wids['table'].SetMinSize((625, 250))
-        panel.Add(wids['table'], newrow=True, dcol=6)
-
-        panel.Add(HLine(panel, size=(625, 3)), dcol=5, newrow=True)
-        panel.Add(wids['saveconf'], dcol=4, newrow=True)
+        # panel.Add(wids['saveconf'], dcol=4, newrow=True)
         panel.pack()
 
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -693,6 +746,52 @@ class LinearComboPanel(TaskPanel):
         pack(self, sizer)
         self.skip_process = False
 
+    def onPanelExposed(self, **kws):
+        # called when notebook is selected
+        try:
+            fname = self.controller.filelist.GetStringSelection()
+            gname = self.controller.file_groups[fname]
+            dgroup = self.controller.get_group(gname)
+            if not hasattr(dgroup, 'norm'):
+                self.xasmain.process_normalization(dgroup)
+            self.fill_form(dgroup)
+        except:
+            pass # print(" Cannot Fill prepeak panel from group ")
+
+        lcf_result = getattr(self.larch.symtable, 'lcf_result', None)
+        if lcf_result is  None:
+            return
+        self.wids['show_results'].Enable()
+        self.skip_process = True
+        selected_groups = []
+        for r in lcf_result[:100]:
+            for gname in r.weights:
+                if gname not in selected_groups:
+                    selected_groups.append(gname)
+
+        if len(selected_groups) > 0:
+            if len(selected_groups) >= MAX_COMPONENTS:
+                selected_groups = selected_groups[:MAX_COMPONENTS]
+            weight = 1.0/len(selected_groups)
+            grid_data = []
+            for grp in selected_groups:
+                grid_data.append([grp, weight, 0, 1])
+
+            self.wids['fit_group'].Enable()
+            self.wids['fit_selected'].Enable()
+            self.wids['table'].table.data = grid_data
+            self.wids['table'].table.View.Refresh()
+        self.skip_process = False
+
+
+    def onFitSpace(self, evt=None):
+        fitspace = self.wids['fitspace'].GetStringSelection()
+        self.update_config(dict(fitspace=fitspace))
+
+        arrname = Linear_ArrayChoices.get(fitspace, 'norm')
+        self.update_fit_xspace(arrname)
+        self.plot()
+
     def onComponent(self, evt=None, comp=None):
         if comp is None or evt is None:
             return
@@ -702,7 +801,7 @@ class LinearComboPanel(TaskPanel):
             if wname.startswith('compchoice'):
                 pref, n = wname.split('_')
                 if wid.GetSelection() > 0:
-                    comps.append((int(n), wid.GetStringSelection()))
+                    caomps.append((int(n), wid.GetStringSelection()))
                 else:
                     self.wids["compval_%s" % n].SetValue(0)
 
@@ -719,60 +818,52 @@ class LinearComboPanel(TaskPanel):
 
     def fill_form(self, dgroup):
         """fill in form from a data group"""
-        opts = self.get_config(dgroup)
+        opts = self.get_config(dgroup, with_erange=True)
         if not hasattr(dgroup, 'norm'):
             self.xasmain.process_normalization(dgroup)
         self.dgroup = dgroup
-        if isinstance(dgroup, Group):
-            d_emin = min(dgroup.energy)
-            d_emax = max(dgroup.energy)
-            if opts['elo'] < d_emin:
-                opts['elo'] = defaults['elo_rel'] + int(dgroup.e0/10.0)*10
-            if opts['ehi'] > d_emax:
-                opts['ehi'] =  defaults['ehi_rel'] + int(dgroup.e0/10.0)*10
+        defaults = self.get_defaultconfig()
 
         self.skip_process = True
         wids = self.wids
-        for attr in ('elo', 'ehi'):
-            val = opts.get(attr, None)
-            if val is not None:
-                wids[attr].SetValue(val)
 
         for attr in ('all_combos', 'sum_to_one', 'show_fitrange'):
             wids[attr].SetValue(opts.get(attr, True))
 
-        for attr in ('fitspace',):
+        for attr in ('elo', 'ehi', ):
+            val = opts.get(attr, None)
+            if val is not None:
+                wids[attr].SetValue(val)
+
+        for attr in ('fitspace', ):
             if attr in opts:
                 wids[attr].SetStringSelection(opts[attr])
 
-#         groupnames = [noname] + list(self.controller.file_groups.keys())
-#         for wname, wid in wids.items():
-#             if wname.startswith('compchoice'):
-#                 cur = wid.GetStringSelection()
-#                 wid.Clear()
-#                 wid.AppendItems(groupnames)
-#                 if cur in groupnames:
-#                     wid.SetStringSelection(cur)
-#                 else:
-#                     wid.SetSelection(0)
-#             elif wname.startswith('comp'):
-#                 wid.SetValue(opts.get(wname, wid.GetValue()))
+        fitspace = self.wids['fitspace'].GetStringSelection()
+        self.update_config(dict(fitspace=fitspace))
+        arrname = Linear_ArrayChoices.get(fitspace, 'norm')
+        self.update_fit_xspace(arrname)
+
         self.skip_process = False
 
     def read_form(self, dgroup=None):
         "read form, return dict of values"
-        self.skip_process = True
+        self.skiap_process = True
         if dgroup is None:
             dgroup = self.controller.get_group()
         self.dgroup = dgroup
-        opts = {'group': dgroup.groupname, 'filename':dgroup.filename}
+        if dgroup is None:
+            opts = {'group': '', 'filename': ''}
+        else:
+            opts = {'group': dgroup.groupname, 'filename':dgroup.filename}
+
         wids = self.wids
         for attr in ('elo', 'ehi', 'max_ncomps'):
             opts[attr] = wids[attr].GetValue()
 
-        for attr in ('fitspace', ): # 'plotchoice'):
-            opts[attr] = wids[attr].GetStringSelection()
-        for attr in ('all_combos', 'sum_to_one', 'show_fitrange'):
+        opts['fitspace'] = wids['fitspace'].GetStringSelection()
+
+        for attr in ('all_combos', 'vary_e0', 'sum_to_one', 'show_fitrange'):
             opts[attr] = wids[attr].GetValue()
 
         for attr, wid in wids.items():
@@ -802,14 +893,7 @@ class LinearComboPanel(TaskPanel):
         if opts['all_combos']:
             opts['func'] = 'lincombo_fitall'
 
-        opts['arrayname'] = 'norm'
-        if opts['fitspace'] == dmude:
-            opts['arrayname'] = 'dmude'
-        elif opts['fitspace'] == rawmu:
-            opts['arrayname'] = 'mu'
-        elif opts['fitspace'] == chik:
-            opts['arrayname'] = 'chi'
-
+        opts['arrayname'] = Linear_ArrayChoices.get(opts['fitspace'], 'norm')
         self.skip_process = False
         return opts
 
@@ -832,37 +916,47 @@ class LinearComboPanel(TaskPanel):
         for grp in selected_groups:
             grid_data.append([grp, weight, 0, 1])
 
+        self.wids['fit_group'].Enable()
+        self.wids['fit_selected'].Enable()
         self.wids['table'].table.data = grid_data
         self.wids['table'].table.View.Refresh()
         self.skip_process = False
 
-
-    def do_fit(self, groupname, form):
+    def do_fit(self, groupname, form, plot=True):
         """run lincombo fit for a group"""
         form['gname'] = groupname
+        if len(groupname) == 0:
+            print("no group to fit?")
+            return
+
         script = """# do LCF for {gname:s}
-result = {func:s}({gname:s}, [{comps:s}],
+lcf_result = {func:s}({gname:s}, [{comps:s}],
             xmin={elo:.4f}, xmax={ehi:.4f},
             arrayname='{arrayname:s}',
-            sum_to_one={sum_to_one},
+            sum_to_one={sum_to_one}, vary_e0={vary_e0},
             weights=[{weights:s}],
             minvals=[{minvals:s}],
             maxvals=[{maxvals:s}],
             max_ncomps={max_ncomps:.0f})
 """
         if form['all_combos']:
-            script = "%s\n{gname:s}.lcf_result = result\n" % script
+            script = "%s\n{gname:s}.lcf_result = lcf_result\n" % script
         else:
-            script = "%s\n{gname:s}.lcf_result = [result]\n" % script
+            script = "%s\n{gname:s}.lcf_result = [lcf_result]\n" % script
 
         self.larch_eval(script.format(**form))
 
         dgroup = self.controller.get_group(groupname)
-        self.show_subframe('lcf_result',  LinComboResultFrame)
+        self.show_subframe('lcf_result',  LinComboResultFrame,
+                           datagroup=dgroup, mainpanel=self)
 
         self.subframes['lcf_result'].add_results(dgroup, form=form,
-                                                 larch_eval=self.larch_eval)
-        self.plot(dgroup=dgroup)
+                                                 larch_eval=self.larch_eval, show=plot)
+        if plot:
+            self.plot(dgroup=dgroup, with_fit=True)
+
+    def onShowResults(self, event=None):
+        self.show_subframe('lcf_result',  LinComboResultFrame, mainpanel=self)
 
     def onFitOne(self, event=None):
         """ handle process events"""
@@ -880,21 +974,20 @@ result = {func:s}({gname:s}, [{comps:s}],
             return
         self.skip_process = True
         form = self.read_form()
-        for sel in self.controller.filelist.GetCheckedStrings():
+        groups = self.controller.filelist.GetCheckedStrings()
+        for i, sel in enumerate(groups):
             gname = self.controller.file_groups[sel]
-            self.do_fit(gname, form)
+            self.do_fit(gname, form, plot=(i==len(groups)-1))
         self.skip_process = False
 
-    def plot(self, dgroup=None):
+    def plot(self, dgroup=None, with_fit=False):
         if self.skip_plotting:
             return
-        self.onPlot(dgroup=dgroup)
 
-    def onPlot(self, evt=None, dgroup=None):
-        if self.skip_plotting:
-            return
         if dgroup is None:
             dgroup = self.controller.get_group()
+
         form = self.read_form(dgroup=dgroup)
-        script = make_lcfplot(dgroup, form, nfit=0)
+        script = make_lcfplot(dgroup, form, with_fit=with_fit, nfit=0)
         self.larch_eval(script)
+        self.controller.set_focus()
