@@ -12,27 +12,27 @@ import wx.lib.scrolledpanel as scrolled
 import wx.dataview as dv
 
 from lmfit import Parameter
-from lmfit.model import (save_modelresult, load_modelresult,
-                             save_model, load_model)
 
 import lmfit.models as lm_models
 from lmfit.printfuncs import gformat
 
 from larch import Group, site_config
+from larch.utils import uname
 from larch.math import index_of
-from larch.utils.jsonutils import encode4js, decode4js
 from larch.io.export_modelresult import export_modelresult
+from larch.io import save_groups, read_groups
 
 from larch.wxlib import (ReportFrame, BitmapButton, FloatCtrl, FloatSpin,
                          SetTip, GridPanel, get_icon, SimpleText, pack,
-                         Button, HLine, Choice, Check, MenuItem, GUIColors,
-                         CEN, RIGHT, LEFT, FRAMESTYLE, Font, FONTSIZE,
-                         FileSave, FileOpen, flatnotebook, Popup,
-                         EditableListBox)
+                         Button, HLine, Choice, Check, MenuItem, COLORS,
+                         set_color, CEN, RIGHT, LEFT, FRAMESTYLE, Font,
+                         FONTSIZE, FONTSIZE_FW, FileSave, FileOpen,
+                         flatnotebook, Popup, EditableListBox)
 
 from larch.wxlib.parameter import ParameterWidgets
 from larch.wxlib.plotter import last_cursor_pos
 from .taskpanel import TaskPanel
+from .config import PrePeak_ArrayChoices, PlotWindowChoices
 
 DVSTYLE = dv.DV_SINGLE|dv.DV_VERT_RULES|dv.DV_ROW_LINES
 
@@ -43,7 +43,7 @@ ModelChoices = {'other': ('<General Models>', 'Constant', 'Linear',
                 'peaks': ('<Peak Models>', 'Gaussian', 'Lorentzian',
                           'Voigt', 'PseudoVoigt', 'DampedHarmonicOscillator',
                           'Pearson7', 'StudentsT', 'SkewedGaussian',
-                          'Moffat', 'BreitWigner', 'Donaich', 'Lognormal'),
+                          'Moffat', 'BreitWigner', 'Doniach', 'Lognormal'),
                 }
 
 # map of lmfit function name to Model Class
@@ -64,7 +64,7 @@ ModelFuncs = {'constant': 'ConstantModel',
               'dho': 'DampedHarmonicOscillatorModel',
               'expgaussian': 'ExponentialGaussianModel',
               'skewed_gaussian': 'SkewedGaussianModel',
-              'donaich': 'DonaichModel',
+              'doniach': 'DoniachModel',
               'powerlaw': 'PowerLawModel',
               'exponential': 'ExponentialModel',
               'step': 'StepModel',
@@ -80,12 +80,6 @@ BaselineFuncs = ['No Baseline',
                  'Quadratic', 'Linear']
 
 
-Array_Choices = {'Raw \u03BC(E)': 'mu',
-                 'Normalized \u03BC(E)': 'norm',
-                 'Flattened \u03BC(E)': 'flat',
-                 'Deconvolved \u03BC(E)': 'deconv',
-                 'Derivative \u03BC(E)': 'dmude'}
-
 PLOT_BASELINE = 'Data+Baseline'
 PLOT_FIT      = 'Data+Fit'
 PLOT_INIT     = 'Data+Init Fit'
@@ -96,11 +90,6 @@ FitMethods = ("Levenberg-Marquardt", "Nelder-Mead", "Powell")
 ModelWcards = "Fit Models(*.modl)|*.modl|All files (*.*)|*.*"
 DataWcards = "Data Files(*.dat)|*.dat|All files (*.*)|*.*"
 
-PLOTOPTS_1 = dict(style='solid', linewidth=3, marker='None', markersize=4)
-PLOTOPTS_2 = dict(style='short dashed', linewidth=2, marker='None', markersize=4)
-
-PLOTOPTS_D = dict(style='solid', linewidth=2, zorder=2,
-                  side='right', marker='None', markersize=4)
 
 MIN_CORREL = 0.10
 
@@ -115,8 +104,8 @@ if not hasattr({group}.prepeaks, 'fit_history'): {group}.prepeaks.fit_history =
 COMMANDS['prepeaks_setup'] = """# setup prepeaks
 {group:s}.xdat = 1.0*{group:s}.energy
 {group:s}.ydat = 1.0*{group:s}.{array_name:s}
-prepeaks_setup(energy={group:s}.energy, norm={group:s}.ydat, group={group:s},
-               elo={elo:.3f}, ehi={ehi:.3f}, emin={emin:.3f}, emax={emax:.3f})
+prepeaks_setup(energy={group:s}, arrayname='{array_name:s}', elo={elo:.3f}, ehi={ehi:.3f},
+               emin={emin:.3f}, emax={emax:.3f})
 """
 
 COMMANDS['set_yerr_const'] = "{group}.prepeaks.norm_std = {group}.yerr*ones(len({group}.prepeaks.norm))"
@@ -127,18 +116,9 @@ yerr_min = 1.e-9*{group}.prepeaks.ydat.mean()
 """
 
 COMMANDS['dofit'] = """# do fit
-peakresult = peakmodel.fit({group}.prepeaks.norm, params=peakpars, x={group}.prepeaks.energy, weights=1.0/{group}.prepeaks.norm_std)
-peakresult.energy   = {group}.prepeaks.energy[:]
-peakresult.norm     = {group}.prepeaks.norm[:]
-peakresult.norm_std = {group}.prepeaks.norm_std[:]
-peakresult.ycomps   = peakmodel.eval_components(params=peakresult.params, x={group}.prepeaks.energy)
-peakresult.init_fit = {group}.prepeaks.init_fit[:]
-peakresult.init_ycomps = peakmodel.eval_components(params=peakpars, x={group}.prepeaks.energy)
+peakresult = prepeaks_fit({group}, peakmodel, peakpars)
 peakresult.user_options = {user_opts:s}
-peakresult.label = 'Fit %i' % (1+len({group}.prepeaks.fit_history))
-{group}.prepeaks.fit_history.insert(0, peakresult)"""
-
-defaults = dict(e=None, elo=-10, ehi=-5, emin=-40, emax=0, yarray='norm')
+"""
 
 def get_xlims(x, xmin, xmax):
     xeps = min(np.diff(x))/ 5.
@@ -150,19 +130,33 @@ class PrePeakFitResultFrame(wx.Frame):
     config_sect = 'prepeak'
     def __init__(self, parent=None, peakframe=None, datagroup=None, **kws):
         wx.Frame.__init__(self, None, -1, title='Pre-edge Peak Fit Results',
-                          style=FRAMESTYLE, size=(900, 700), **kws)
+                          style=FRAMESTYLE, size=(950, 700), **kws)
         self.peakframe = peakframe
-        self.datagroup = datagroup
-        prepeaks = getattr(datagroup, 'prepeaks', None)
-        self.peakfit_history = getattr(prepeaks, 'fit_history', [])
+
+        if datagroup is not None:
+            self.datagroup = datagroup
+            # prepeaks = getattr(datagroup, 'prepeaks', None)
+            # self.peakfit_history = getattr(prepeaks, 'fit_history', [])
         self.parent = parent
         self.datasets = {}
         self.form = {}
-        self.larch_eval = None
+        self.larch_eval = self.peakframe.larch_eval
         self.nfit = 0
         self.createMenus()
         self.build()
 
+        if datagroup is None:
+            symtab = self.peakframe.larch.symtable
+            xasgroups = getattr(symtab, '_xasgroups', None)
+            if xasgroups is not None:
+                for dname, dgroup in xasgroups.items():
+                    dgroup = getattr(symtab, dgroup, None)
+                    ppeak = getattr(dgroup, 'prepeaks', None)
+                    hist =  getattr(ppeak, 'fit_history', None)
+                    if hist is not None:
+                        self.add_results(dgroup, show=True)
+
+
     def createMenus(self):
         self.menubar = wx.MenuBar()
         fmenu = wx.Menu()
@@ -190,31 +184,54 @@ class PrePeakFitResultFrame(wx.Frame):
         splitter = wx.SplitterWindow(self, style=wx.SP_LIVE_UPDATE)
         splitter.SetMinimumPaneSize(200)
 
-        self.datalistbox = EditableListBox(splitter, self.ShowDataSet,
+        self.filelist = EditableListBox(splitter, self.ShowDataSet,
                                            size=(250, -1))
+        set_color(self.filelist, 'list_fg', bg='list_bg')
+
         panel = scrolled.ScrolledPanel(splitter)
 
         panel.SetMinSize((775, 575))
-        self.colors = GUIColors()
+
+        self.font_fixedwidth = wx.Font(FONTSIZE_FW, wx.MODERN, wx.NORMAL, wx.BOLD)
 
         # title row
         self.wids = wids = {}
         title = SimpleText(panel, 'Fit Results', font=Font(FONTSIZE+2),
-                           colour=self.colors.title, style=LEFT)
+                           colour=COLORS['title'], style=LEFT)
 
         wids['data_title'] = SimpleText(panel, '< > ', font=Font(FONTSIZE+2),
                                         minsize=(350, -1),
-                                        colour=self.colors.title, style=LEFT)
+                                        colour=COLORS['title'], style=LEFT)
 
         opts = dict(default=False, size=(200, -1), action=self.onPlot)
-        wids['plot_bline'] = Check(panel, label='Plot baseline-subtracted?', **opts)
-        wids['plot_resid'] = Check(panel, label='Plot with residual?', **opts)
-        self.plot_choice = Button(panel, 'Plot This Fit',
-                                  size=(125, -1), action=self.onPlot)
+        ppanel = wx.Panel(panel)
+        wids['plot_bline'] = Check(ppanel, label='Plot baseline-subtracted?', **opts)
+        wids['plot_resid'] = Check(ppanel, label='Plot with residual?', **opts)
+        wids['plot_win']   = Choice(ppanel, size=(60, -1), choices=PlotWindowChoices,
+                                    action=self.onPlot)
+        wids['plot_win'].SetStringSelection('1')
 
-        wids['fit_label'] = wx.TextCtrl(panel, -1, ' ', size=(225, -1))
-        wids['set_label'] = Button(panel, 'Update Label', size=(175, -1),
+        psizer = wx.BoxSizer(wx.HORIZONTAL)
+        psizer.Add( wids['plot_bline'], 0, 5)
+        psizer.Add( wids['plot_resid'], 0, 5)
+        psizer.Add(SimpleText(ppanel, 'Plot Window:'), 0, 5)
+        psizer.Add( wids['plot_win'], 0, 5)
+
+        pack(ppanel, psizer)
+
+        wids['load_model'] = Button(panel, 'Load this Model for Fitting',
+                                    size=(250, -1), action=self.onLoadModel)
+
+        wids['plot_choice'] = Button(panel, 'Plot This Fit',
+                                     size=(125, -1), action=self.onPlot)
+
+        wids['fit_label'] = wx.TextCtrl(panel, -1, ' ', size=(175, -1))
+        wids['set_label'] = Button(panel, 'Update Label', size=(150, -1),
                                    action=self.onUpdateLabel)
+        wids['del_fit'] = Button(panel, 'Remove from Fit History', size=(200, -1),
+                                 action=self.onRemoveFromHistory)
+
+
 
         irow = 0
         sizer.Add(title,              (irow, 0), (1, 1), LEFT)
@@ -226,22 +243,25 @@ class PrePeakFitResultFrame(wx.Frame):
         sizer.Add(wids['model_desc'],  (irow, 0), (1, 6), LEFT)
 
         irow += 1
-        # sizer.Add(SimpleText(panel, 'Plot: '), (irow, 0), (1, 1), LEFT)
-        sizer.Add(self.plot_choice,   (irow, 0), (1, 1), LEFT)
-        sizer.Add(wids['plot_bline'], (irow, 1), (1, 2), LEFT)
-        sizer.Add(wids['plot_resid'], (irow, 3), (1, 1), LEFT)
+        sizer.Add(wids['load_model'],(irow, 0), (1, 2), LEFT)
+
+        irow += 1
+        sizer.Add(wids['plot_choice'],(irow, 0), (1, 1), LEFT)
+        sizer.Add(ppanel, (irow, 1), (1, 4), LEFT)
+
 
         irow += 1
         sizer.Add(HLine(panel, size=(650, 3)), (irow, 0), (1, 5), LEFT)
 
         irow += 1
         sizer.Add(SimpleText(panel, 'Fit Label:', style=LEFT), (irow, 0), (1, 1), LEFT)
-        sizer.Add(wids['fit_label'], (irow, 1), (1, 2), LEFT)
-        sizer.Add(wids['set_label'], (irow, 3), (1, 1), LEFT)
+        sizer.Add(wids['fit_label'], (irow, 1), (1, 1), LEFT)
+        sizer.Add(wids['set_label'], (irow, 2), (1, 1), LEFT)
+        sizer.Add(wids['del_fit'], (irow, 3), (1, 2), LEFT)
 
         irow += 1
         title = SimpleText(panel, '[[Fit Statistics]]',  font=Font(FONTSIZE+2),
-                           colour=self.colors.title, style=LEFT)
+                           colour=COLORS['title'], style=LEFT)
         subtitle = SimpleText(panel, ' (most recent fit is at the top)',
                               font=Font(FONTSIZE+1),  style=LEFT)
 
@@ -249,6 +269,7 @@ class PrePeakFitResultFrame(wx.Frame):
         sizer.Add(subtitle, (irow, 1), (1, 1), LEFT)
 
         sview = self.wids['stats'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        sview.SetFont(self.font_fixedwidth)
         sview.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED, self.onSelectFit)
         sview.AppendTextColumn(' Label',  width=120)
         sview.AppendTextColumn(' N_data', width=75)
@@ -263,7 +284,7 @@ class PrePeakFitResultFrame(wx.Frame):
             this.Alignment = wx.ALIGN_RIGHT if col > 0 else wx.ALIGN_LEFT
             this.Renderer.Alignment = this.Alignment
 
-        sview.SetMinSize((700, 125))
+        sview.SetMinSize((725, 150))
 
         irow += 1
         sizer.Add(sview, (irow, 0), (1, 5), LEFT)
@@ -273,7 +294,7 @@ class PrePeakFitResultFrame(wx.Frame):
 
         irow += 1
         title = SimpleText(panel, '[[Variables]]',  font=Font(FONTSIZE+2),
-                           colour=self.colors.title, style=LEFT)
+                           colour=COLORS['title'], style=LEFT)
         sizer.Add(title, (irow, 0), (1, 1), LEFT)
 
         self.wids['copy_params'] = Button(panel, 'Update Model with these values',
@@ -282,6 +303,7 @@ class PrePeakFitResultFrame(wx.Frame):
         sizer.Add(self.wids['copy_params'], (irow, 1), (1, 3), LEFT)
 
         pview = self.wids['params'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        pview.SetFont(self.font_fixedwidth)
         self.wids['paramsdata'] = []
         pview.AppendTextColumn('Parameter',         width=150)
         pview.AppendTextColumn('Best-Fit Value',    width=125)
@@ -294,7 +316,7 @@ class PrePeakFitResultFrame(wx.Frame):
             this.Alignment = wx.ALIGN_RIGHT if col in (1, 2) else wx.ALIGN_LEFT
             this.Renderer.Alignment = this.Alignment
 
-        pview.SetMinSize((700, 200))
+        pview.SetMinSize((725, 200))
         pview.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED, self.onSelectParameter)
 
         irow += 1
@@ -305,7 +327,7 @@ class PrePeakFitResultFrame(wx.Frame):
 
         irow += 1
         title = SimpleText(panel, '[[Correlations]]',  font=Font(FONTSIZE+2),
-                           colour=self.colors.title, style=LEFT)
+                           colour=COLORS['title'], style=LEFT)
 
         self.wids['all_correl'] = Button(panel, 'Show All',
                                           size=(100, -1), action=self.onAllCorrel)
@@ -321,6 +343,8 @@ class PrePeakFitResultFrame(wx.Frame):
         sizer.Add(self.wids['all_correl'], (irow, 3), (1, 1), LEFT)
 
         cview = self.wids['correl'] = dv.DataViewListCtrl(panel, style=DVSTYLE)
+        cview.SetFont(self.font_fixedwidth)
+
 
         cview.AppendTextColumn('Parameter 1',    width=150)
         cview.AppendTextColumn('Parameter 2',    width=150)
@@ -333,7 +357,7 @@ class PrePeakFitResultFrame(wx.Frame):
             if col == 2:
                 align = wx.ALIGN_RIGHT
             this.Alignment = this.Renderer.Alignment = align
-        cview.SetMinSize((475, 200))
+        cview.SetMinSize((475, 150))
 
         irow += 1
         sizer.Add(cview, (irow, 0), (1, 5), LEFT)
@@ -341,7 +365,7 @@ class PrePeakFitResultFrame(wx.Frame):
         pack(panel, sizer)
         panel.SetupScrolling()
 
-        splitter.SplitVertically(self.datalistbox, panel, 1)
+        splitter.SplitVertically(self.filelist, panel, 1)
 
         mainsizer = wx.BoxSizer(wx.VERTICAL)
         mainsizer.Add(splitter, 1, wx.GROW|wx.ALL, 5)
@@ -356,11 +380,22 @@ class PrePeakFitResultFrame(wx.Frame):
         result.label = self.wids['fit_label'].GetValue()
         self.show_results()
 
+    def onRemoveFromHistory(self, event=None):
+        result = self.get_fitresult()
+        if wx.ID_YES != Popup(self,
+                              f"Remove fit '{result.label}' from history?\nThis cannot be undone.",
+                              "Remove fit?", style=wx.YES_NO):
+                return
+
+        self.datagroup.prepeaks.fit_history.pop(self.nfit)
+        self.nfit = 0
+        self.show_results()
+
 
     def onSaveAllStats(self, evt=None):
         "Save Parameters and Statistics to CSV"
         # get first dataset to extract fit parameter names
-        fnames = self.datalistbox.GetItems()
+        fnames = self.filelist.GetItems()
         if len(fnames) == 0:
             return
 
@@ -370,16 +405,17 @@ class PrePeakFitResultFrame(wx.Frame):
                         default_file=deffile, wildcard=wcards)
         if path is None:
             return
-        if os.path.exists(path):
+        if os.path.exists(path) and uname != 'darwin':  # darwin prompts in FileSave!
             if wx.ID_YES != Popup(self,
                                   "Overwrite existing Statistics File?",
                                   "Overwrite existing file?", style=wx.YES_NO):
                 return
 
         ppeaks_tmpl = self.datasets[fnames[0]].prepeaks
-        param_names = list(reversed(ppeaks_tmpl.fit_history[0].params.keys()))
+        res0 = ppeaks_tmpl.fit_history[0].result
+        param_names = list(reversed(res0.params.keys()))
         user_opts = ppeaks_tmpl.user_options
-        model_desc = self.get_model_desc(ppeaks_tmpl.fit_history[0].model).replace('\n', ' ')
+        model_desc = self.get_model_desc(res0.model).replace('\n', ' ')
         out = ['# Pre-edge Peak Fit Report %s' % time.ctime(),
                '# Fitted Array name: %s' %  user_opts['array_name'],
                '# Model form: %s' % model_desc,
@@ -396,7 +432,8 @@ class PrePeakFitResultFrame(wx.Frame):
             labels.append(pname+'_stderr')
         out.append('# %s' % (', '.join(labels)))
         for name, dgroup in self.datasets.items():
-            result = dgroup.prepeaks.fit_history[0]
+            pkfit = dgroup.prepeaks.fit_history[0]
+            result = pkfit.result
             label = dgroup.filename
             if len(label) < 25:
                 label = (label + ' '*25)[:25]
@@ -423,8 +460,8 @@ class PrePeakFitResultFrame(wx.Frame):
         sfile = FileSave(self, 'Save Fit Model', default_file=deffile,
                            wildcard=ModelWcards)
         if sfile is not None:
-            result = self.get_fitresult()
-            save_modelresult(result, sfile)
+            pkfit = self.get_fitresult()
+            save_groups(sfile, ['#peakfit 1.0', pkfit])
 
     def onExportFitResult(self, event=None):
         dgroup = self.datagroup
@@ -433,11 +470,12 @@ class PrePeakFitResultFrame(wx.Frame):
 
         outfile = FileSave(self, 'Export Fit Result', default_file=deffile)
 
-        result = self.get_fitresult()
+        pkfit = self.get_fitresult()
+        result = pkfit.result
         if outfile is not None:
             i1, i2 = get_xlims(dgroup.xdat,
-                               result.user_options['emin'],
-                               result.user_options['emax'])
+                               pkfit.user_options['emin'],
+                               pkfit.user_options['emax'])
             x = dgroup.xdat[i1:i2]
             y = dgroup.ydat[i1:i2]
             yerr = None
@@ -460,15 +498,17 @@ class PrePeakFitResultFrame(wx.Frame):
         self.nfit = max(0, nfit)
         if self.nfit > len(self.peakfit_history):
             self.nfit = 0
-        return self.peakfit_history[self.nfit]
+        if len(self.peakfit_history) > 0:
+            return self.peakfit_history[self.nfit]
 
     def onPlot(self, event=None):
         show_resid = self.wids['plot_resid'].IsChecked()
         sub_bline = self.wids['plot_bline'].IsChecked()
-        cmd = "plot_prepeaks_fit(%s, nfit=%i, show_residual=%s, subtract_baseline=%s)"
-        cmd = cmd % (self.datagroup.groupname, self.nfit, show_resid, sub_bline)
-
+        win  = int(self.wids['plot_win'].GetStringSelection())
+        cmd = "plot_prepeaks_fit(%s, nfit=%i, show_residual=%s, subtract_baseline=%s, win=%d)"
+        cmd = cmd % (self.datagroup.groupname, self.nfit, show_resid, sub_bline, win)
         self.peakframe.larch_eval(cmd)
+        self.peakframe.controller.set_focus(topwin=self)
 
     def onSelectFit(self, evt=None):
         if self.wids['stats'] is None:
@@ -489,7 +529,7 @@ class PrePeakFitResultFrame(wx.Frame):
         self.wids['correl'].DeleteAllItems()
 
         result = self.get_fitresult()
-        this = result.params[pname]
+        this = result.result.params[pname]
         if this.correl is not None:
             sort_correl = sorted(this.correl.items(), key=lambda it: abs(it[1]))
             for name, corval in reversed(sort_correl):
@@ -498,7 +538,7 @@ class PrePeakFitResultFrame(wx.Frame):
 
     def onAllCorrel(self, evt=None):
         result = self.get_fitresult()
-        params = result.params
+        params = result.result.params
         parnames = list(params.keys())
 
         cormin= self.wids['min_correl'].GetValue()
@@ -522,25 +562,28 @@ class PrePeakFitResultFrame(wx.Frame):
             name1, name2 = namepair.split('$$')
             self.wids['correl'].AppendItem((name1, name2, "% .4f" % corval))
 
+    def onLoadModel(self, event=None):
+        self.peakframe.use_modelresult(self.get_fitresult())
+
     def onCopyParams(self, evt=None):
         result = self.get_fitresult()
-        self.peakframe.update_start_values(result.params)
+        self.peakframe.update_start_values(result.result.params)
 
     def ShowDataSet(self, evt=None):
         dataset = evt.GetString()
         group = self.datasets.get(evt.GetString(), None)
         if group is not None:
-            self.show_results(datagroup=group)
+            self.show_results(datagroup=group, show_plot=True)
 
     def add_results(self, dgroup, form=None, larch_eval=None, show=True):
         name = dgroup.filename
-        if name not in self.datalistbox.GetItems():
-            self.datalistbox.Append(name)
+        if name not in self.filelist.GetItems():
+            self.filelist.Append(name)
         self.datasets[name] = dgroup
         if show:
             self.show_results(datagroup=dgroup, form=form, larch_eval=larch_eval)
 
-    def show_results(self, datagroup=None, form=None, larch_eval=None):
+    def show_results(self, datagroup=None, form=None, show_plot=False, larch_eval=None):
         if datagroup is not None:
             self.datagroup = datagroup
         if larch_eval is not None:
@@ -549,11 +592,10 @@ class PrePeakFitResultFrame(wx.Frame):
         datagroup = self.datagroup
         self.peakfit_history = getattr(self.datagroup.prepeaks, 'fit_history', [])
 
-        cur = self.get_fitresult()
+        # cur = self.get_fitresult()
         wids = self.wids
         wids['stats'].DeleteAllItems()
         for i, res in enumerate(self.peakfit_history):
-            # '%2.2d' % (i+1)
             args = [res.label]
             for attr in ('ndata', 'nvarys', 'chisqr', 'redchi', 'aic'):
                 val = getattr(res.result, attr)
@@ -566,6 +608,15 @@ class PrePeakFitResultFrame(wx.Frame):
         wids['data_title'].SetLabel(self.datagroup.filename)
         self.show_fitresult(nfit=0)
 
+        if show_plot:
+            show_resid= self.wids['plot_resid'].IsChecked()
+            sub_bline = self.wids['plot_bline'].IsChecked()
+            cmd = "plot_prepeaks_fit(%s, nfit=0, show_residual=%s, subtract_baseline=%s)"
+            cmd = cmd % (datagroup.groupname, show_resid, sub_bline)
+
+            self.peakframe.larch_eval(cmd)
+            self.peakframe.controller.set_focus(topwin=self)
+
     def get_model_desc(self, model):
         model_repr = model._reprstring(long=True)
         for word in ('Model(', ',', '(', ')', '+'):
@@ -592,10 +643,10 @@ class PrePeakFitResultFrame(wx.Frame):
         wids = self.wids
         wids['fit_label'].SetValue(result.label)
         wids['data_title'].SetLabel(self.datagroup.filename)
-        wids['model_desc'].SetLabel(self.get_model_desc(result.model))
+        wids['model_desc'].SetLabel(self.get_model_desc(result.result.model))
         wids['params'].DeleteAllItems()
         wids['paramsdata'] = []
-        for param in reversed(result.params.values()):
+        for param in reversed(result.result.params.values()):
             pname = param.name
             try:
                 val = gformat(param.value, 10)
@@ -616,14 +667,9 @@ class PrePeakFitResultFrame(wx.Frame):
             wids['paramsdata'].append(pname)
         self.Refresh()
 
-
 class PrePeakPanel(TaskPanel):
     def __init__(self, parent=None, controller=None, **kws):
-        TaskPanel.__init__(self, parent, controller,
-                           configname='prepeaks_config',
-                           title='Pre-edge Peak Analysis',
-                           config=defaults, **kws)
-
+        TaskPanel.__init__(self, parent, controller, panel='prepeaks', **kws)
         self.fit_components = {}
         self.user_added_params = None
 
@@ -646,12 +692,20 @@ class PrePeakPanel(TaskPanel):
             if not hasattr(dgroup, 'norm'):
                 self.xasmain.process_normalization(dgroup)
             self.fill_form(dgroup)
+
         except:
             pass # print(" Cannot Fill prepeak panel from group ")
 
+        pkfit = getattr(self.larch.symtable, 'peakresult', None)
+        if pkfit is not None:
+            self.showresults_btn.Enable()
+            self.use_modelresult(pkfit)
+
+    def onModelPanelExposed(self, event=None, **kws):
+        pass
+
     def build_display(self):
-        self.mod_nb = flatnotebook(self, {})
-        pan = self.panel = GridPanel(self, ncols=4, nrows=4, pad=2, itemstyle=LEFT)
+        pan = self.panel # = GridPanel(self, ncols=4, nrows=4, pad=2, itemstyle=LEFT)
 
         self.wids = {}
 
@@ -662,6 +716,10 @@ class PrePeakPanel(TaskPanel):
         ppeak_emin = self.add_floatspin('ppeak_emin', value=-20, **fsopts)
         ppeak_emax = self.add_floatspin('ppeak_emax', value=0, **fsopts)
 
+        self.showresults_btn = Button(pan, 'Show Fit Results',
+                                      action=self.onShowResults, size=(150, -1))
+        self.showresults_btn.Disable()
+
         self.fitbline_btn  = Button(pan,'Fit Baseline', action=self.onFitBaseline,
                                     size=(150, -1))
 
@@ -677,8 +735,8 @@ class PrePeakPanel(TaskPanel):
         self.fitmodel_btn.Disable()
 
         self.array_choice = Choice(pan, size=(175, -1),
-                                   choices=list(Array_Choices.keys()))
-        self.array_choice.SetSelection(1)
+                                   choices=list(PrePeak_ArrayChoices.keys()))
+        self.array_choice.SetSelection(0)
 
         self.bline_choice = Choice(pan, size=(175, -1),
                                    choices=BaselineFuncs)
@@ -713,7 +771,8 @@ class PrePeakPanel(TaskPanel):
 
         add_text('Array to fit: ')
         pan.Add(self.array_choice, dcol=3)
-
+        pan.Add((5,5))
+        pan.Add(self.showresults_btn)
         # add_text('E0: ', newrow=False)
         # pan.Add(ppeak_e0)
         # pan.Add(self.show_e0)
@@ -757,9 +816,14 @@ class PrePeakPanel(TaskPanel):
         pan.Add(self.message, dcol=4)
         pan.Add(self.fitselected_btn)
 
-        pan.Add(HLine(self, size=(600, 2)), dcol=6, newrow=True)
+        pan.Add(HLine(pan, size=(600, 2)), dcol=6, newrow=True)
         pan.pack()
 
+        self.mod_nb = flatnotebook(self, {}, on_change=self.onModelPanelExposed)
+        self.mod_nb_init = True
+        dummy_panel = wx.Panel(self.mod_nb)
+
+        self.mod_nb.AddPage(dummy_panel, 'Empty Model', True)
         sizer = wx.BoxSizer(wx.VERTICAL)
         sizer.Add((10, 10), 0, LEFT, 3)
         sizer.Add(pan, 0, LEFT, 3)
@@ -775,7 +839,7 @@ class PrePeakPanel(TaskPanel):
 
         conf = getattr(dgroup, 'prepeak_config', {})
         if 'e0' not in conf:
-            conf = defaults
+            conf = self.controller.get_defaultcfonfig()
             conf['e0'] = getattr(dgroup, 'e0', -1)
 
         dgroup.prepeak_config = conf
@@ -815,7 +879,7 @@ class PrePeakPanel(TaskPanel):
         form_opts = {'gname': dgroup.groupname,
                      'filename': dgroup.filename,
                      'array_desc': array_desc.lower(),
-                     'array_name': Array_Choices[array_desc],
+                     'array_name': PrePeak_ArrayChoices[array_desc],
                      'baseline_form': bline_form.lower(),
                      'bkg_components': []}
 
@@ -883,7 +947,6 @@ elo={elo:.3f}, ehi={ehi:.3f}, emin={emin:.3f}, emax={emax:.3f})"""
         i1, i2 = self.get_xranges(dgroup.energy)
         dgroup.yfit = dgroup.xfit = 0.0*dgroup.energy[i1:i2]
 
-        # self.plot_choice.SetStringSelection(PLOT_BASELINE)
         self.onPlot(baseline_only=True)
         # self.savebline_btn.Enable()
 
@@ -931,12 +994,12 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
                     wids.minval.SetValue(par.min)
                 if wids.maxval is not None:
                     wids.maxval.SetValue(par.max)
-                wids.value.SetValue(par.value)
                 varstr = 'vary' if par.vary else 'fix'
                 if par.expr is not None:
                     varstr = 'constrain'
                 if wids.vary is not None:
                     wids.vary.SetStringSelection(varstr)
+                wids.value.SetValue(par.value)
 
     def onPlotModel(self, evt=None):
         dgroup = self.controller.get_group()
@@ -949,6 +1012,11 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         opts['group'] = opts['gname']
         self.larch_eval(COMMANDS['prepeaks_setup'].format(**opts))
 
+        ppeaks_opts = dict(array=opts['array_name'], elo=opts['elo'],
+                           ehi=opts['ehi'], emin=opts['emin'],
+                           emax=opts['emax'])
+        dgroup.journal.add_ifnew('prepeaks_setup', ppeaks_opts)
+
         cmd = "plot_prepeaks_fit"
         args = ['{gname}']
         if baseline_only:
@@ -957,6 +1025,7 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
             args.append("show_init=%s" % (show_init))
         cmd = "%s(%s)" % (cmd, ', '.join(args))
         self.larch_eval(cmd.format(**opts))
+        self.controller.set_focus()
 
     def addModel(self, event=None, model=None, prefix=None, isbkg=False):
         if model is None and event is not None:
@@ -998,6 +1067,7 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
             minst = mclass(prefix=prefix)
 
         panel = GridPanel(self.mod_nb, ncols=2, nrows=5, pad=1, itemstyle=CEN)
+        panel.SetFont(Font(FONTSIZE))
 
         def SLabel(label, size=(80, -1), **kws):
             return  SimpleText(panel, label,
@@ -1089,6 +1159,9 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
 
         self.fit_components[prefix] = fgroup
         panel.pack()
+        if self.mod_nb_init:
+            self.mod_nb.DeletePage(0)
+            self.mod_nb_init = False
 
         self.mod_nb.AddPage(panel, title, True)
         sx,sy = self.GetSize()
@@ -1115,7 +1188,6 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
             self.fitmodel_btn.Disable()
             self.fitselected_btn.Enable()
 
-
     def onPick2EraseTimer(self, evt=None):
         """erases line trace showing automated 'Pick 2' guess """
         self.pick2erase_timer.Stop()
@@ -1213,14 +1285,22 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         if rfile is None:
             return
 
-        self.larch_eval("# peakmodel = lm_load_modelresult('%s')" %rfile)
+        self.larch_eval(f"# peakmodel = read_groups('{rfile}')[1]")
+        dat = read_groups(str(rfile))
+        if len(dat) != 2 or not dat[0].startswith('#peakfit'):
+            Popup(self, f" '{rfile}' is not a valid Peak Model file",
+                  "Invalid file")
 
-        result = load_modelresult(str(rfile))
+        self.use_modelresult(dat[1])
+
+    def use_modelresult(self, pkfit):
         for prefix in list(self.fit_components.keys()):
             self.onDeleteComponent(prefix=prefix)
 
+        result = pkfit.result
+        bkg_comps = pkfit.user_options['bkg_components']
         for comp in result.model.components:
-            isbkg = comp.prefix in result.user_options['bkg_components']
+            isbkg = comp.prefix in bkg_comps
             self.addModel(model=comp.func.__name__,
                           prefix=comp.prefix, isbkg=isbkg)
 
@@ -1229,39 +1309,16 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
             for pname, par in result.params.items():
                 if pname in parwids:
                     wids = parwids[pname]
-                    if wids.minval is not None:
-                        wids.minval.SetValue(par.min)
-                    if wids.maxval is not None:
-                        wids.maxval.SetValue(par.max)
-                    val = result.init_values.get(pname, par.value)
-                    wids.value.SetValue(val)
+                    wids.value.SetValue(result.init_values.get(pname, par.value))
                     varstr = 'vary' if par.vary else 'fix'
-                    if par.expr is not None:
-                        varstr = 'constrain'
-                    if wids.vary is not None:
-                        wids.vary.SetStringSelection(varstr)
+                    if par.expr is not None:   varstr = 'constrain'
+                    if wids.vary is not None:  wids.vary.SetStringSelection(varstr)
+                    if wids.minval is not None: wids.minval.SetValue(par.min)
+                    if wids.maxval is not None: wids.maxval.SetValue(par.max)
 
-        self.fill_form(result.user_options)
+        self.fill_form(pkfit.user_options)
 
 
-    def onSelPoint(self, evt=None, opt='__', relative_e0=False, win=None):
-        """
-        get last selected point from a specified plot window
-        and fill in the value for the widget defined by `opt`.
-
-        by default it finds the latest cursor position from the
-        cursor history of the first 20 plot windows.
-        """
-        if opt not in self.wids:
-            return None
-
-        _x, _y = last_cursor_pos(win=win, _larch=self.larch)
-
-        if _x is not None:
-            if relative_e0 and 'e0' in self.wids:
-                _x -= self.wids['e0'].GetValue()
-            self.wids[opt].SetValue(_x)
-
     def get_xranges(self, x):
         opts = self.read_form()
         dgroup = self.controller.get_group()
@@ -1286,6 +1343,12 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         dgroup = self.controller.get_group(groupname)
         self.larch_eval(COMMANDS['prepeaks_setup'].format(**opts))
 
+        ppeaks_opts = dict(array=opts['array_name'], elo=opts['elo'],
+                           ehi=opts['ehi'], emin=opts['emin'],
+                           emax=opts['emax'])
+        dgroup.journal.add_ifnew('prepeaks_setup', ppeaks_opts)
+
+
         for comp in self.fit_components.values():
             _cen, _amp = None, None
             if comp.usebox is not None and comp.usebox.IsChecked():
@@ -1344,6 +1407,11 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
             self.build_fitmodel(gname)
             opts['group'] = opts['gname']
             self.larch_eval(COMMANDS['prepeaks_setup'].format(**opts))
+
+            ppeaks_opts = dict(array=opts['array_name'], elo=opts['elo'],
+                               ehi=opts['ehi'], emin=opts['emin'],
+                               emax=opts['emax'])
+            dgroup.journal.add_ifnew('prepeaks_setup', ppeaks_opts)
             ppeaks = dgroup.prepeaks
 
             # add bkg_component to saved user options
@@ -1351,6 +1419,7 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
             for label, comp in self.fit_components.items():
                 if comp.bkgbox.IsChecked():
                     bkg_comps.append(label)
+
             opts['bkg_components'] = bkg_comps
             imin, imax = self.get_xranges(dgroup.xdat)
             cmds = ["## do peak fit for group %s / %s " % (gname, dgroup.filename) ]
@@ -1375,12 +1444,18 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
                                        imin=imin, imax=imax,
                                        user_opts=repr(opts)))
 
-            self.autosave_modelresult(self.larch_get("peakresult"))
+            pkfit = self.larch_get("peakresult")
+            jnl = {'label': pkfit.label, 'var_names': pkfit.result.var_names,
+                   'model': repr(pkfit.result.model)}
+            jnl.update(pkfit.user_options)
+            dgroup.journal.add('peakfit', jnl)
+            if igroup == 0:
+                self.autosave_modelresult(pkfit)
+
             self.subframes['prepeak_result'].add_results(dgroup, form=opts,
                                                          larch_eval=self.larch_eval,
                                                          show=igroup==ngroups-1)
 
-
     def onFitModel(self, event=None):
         dgroup = self.controller.get_group()
         if dgroup is None:
@@ -1392,6 +1467,11 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         opts['group'] = opts['gname']
         self.larch_eval(COMMANDS['prepeaks_setup'].format(**opts))
 
+        ppeaks_opts = dict(array=opts['array_name'], elo=opts['elo'],
+                           ehi=opts['ehi'], emin=opts['emin'],
+                           emax=opts['emax'])
+        dgroup.journal.add_ifnew('prepeaks_setup', ppeaks_opts)
+
         ppeaks = dgroup.prepeaks
 
         # add bkg_component to saved user options
@@ -1404,7 +1484,6 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         imin, imax = self.get_xranges(dgroup.xdat)
 
         cmds = ["## do peak fit: "]
-
         yerr_type = 'set_yerr_const'
         yerr = getattr(dgroup, 'yerr', None)
         if yerr is None:
@@ -1419,22 +1498,31 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         elif isinstance(dgroup.yerr, np.ndarray):
                 yerr_type = 'set_yerr_array'
 
-
         cmds.extend([COMMANDS[yerr_type], COMMANDS['dofit']])
-
         cmd = '\n'.join(cmds)
         self.larch_eval(cmd.format(group=dgroup.groupname,
                                    imin=imin, imax=imax,
                                    user_opts=repr(opts)))
 
-        self.autosave_modelresult(self.larch_get("peakresult"))
+        # journal about peakresult
+        pkfit = self.larch_get("peakresult")
+        jnl = {'label': pkfit.label, 'var_names': pkfit.result.var_names,
+               'model': repr(pkfit.result.model)}
+        jnl.update(pkfit.user_options)
+        dgroup.journal.add('peakfit', jnl)
 
+        self.autosave_modelresult(pkfit)
         self.onPlot()
-        self.show_subframe('prepeak_result', PrePeakFitResultFrame,
-                                  datagroup=dgroup, peakframe=self)
+        self.showresults_btn.Enable()
+        self.show_subframe('prepeak_result', PrePeakFitResultFrame, peakframe=self)
         self.subframes['prepeak_result'].add_results(dgroup, form=opts,
                                                      larch_eval=self.larch_eval)
 
+    def onShowResults(self, event=None):
+        self.show_subframe('prepeak_result',
+                           PrePeakFitResultFrame, peakframe=self)
+
+
     def update_start_values(self, params):
         """fill parameters with best fit values"""
         allparwids = {}
@@ -1457,5 +1545,5 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
                 print("Warning: cannot create XAS_Viewer user folder")
                 return
         if fname is None:
-            fname = 'autosave.fitmodel'
-        save_modelresult(result, os.path.join(confdir, fname))
+            fname = 'autosave_peakfile.modl'
+        save_groups(os.path.join(confdir, fname), ['#peakfit 1.0', result])
