@@ -9,7 +9,7 @@ import wx.grid as wxgrid
 import numpy as np
 import pickle
 import base64
-
+from copy import deepcopy
 from functools import partial
 
 from lmfit.printfuncs import gformat
@@ -17,33 +17,22 @@ from larch import Group
 from larch.math import index_of
 from larch.wxlib import (BitmapButton, TextCtrl, FloatCtrl, get_icon,
                          SimpleText, pack, Button, HLine, Choice, Check,
-                         NumericCombo, CEN, LEFT, Font, FileSave, FileOpen)
-from larch.io import read_csv
+                         NumericCombo, CEN, LEFT, Font, FileSave, FileOpen,
+                         DataTableGrid, Popup, FONTSIZE_FW)
+from larch.io import save_groups, read_groups, read_csv
 from larch.utils.strutils import fix_varname
+from larch.utils import get_cwd
 
-from .taskpanel import TaskPanel, DataTableGrid
-
-# plot options:
-norm   = 'Normalized \u03bC(E)'
-dmude  = 'd\u03bC(E)/dE'
-chik   = '\u03c7(k)'
-noplot = '<no plot>'
-noname = '<none>'
+from .taskpanel import TaskPanel
+from .config import Linear_ArrayChoices, Regress_Choices
 
 CSV_WILDCARDS = "CSV Files(*.csv,*.dat)|*.csv*;*.dat|All files (*.*)|*.*"
 MODEL_WILDCARDS = "Regression Model Files(*.regmod,*.dat)|*.regmod*;*.dat|All files (*.*)|*.*"
 
-FitSpace_Choices = [norm, dmude, chik]
 Plot_Choices = ['Mean Spectrum + Active Energies',
                 'Spectra Stack',
                 'Predicted External Varliable']
 
-Regress_Choices = ['Partial Least Squares', 'LassoLars']
-
-defaults = dict(fitspace=norm, varname='valence', xmin=-5.e5, xmax=5.e5,
-                scale=True, cv_folds=None, cv_repeats=3, fit_intercept=True,
-                use_lars=True, alpha=0.01)
-
 MAX_ROWS = 1000
 
 def make_steps(max=1, decades=8):
@@ -55,9 +44,7 @@ def make_steps(max=1, decades=8):
 class RegressionPanel(TaskPanel):
     """Regression Panel"""
     def __init__(self, parent, controller, **kws):
-        TaskPanel.__init__(self, parent, controller,
-                           configname='regression_config', config=defaults,
-                           title='Regression and Feature Selection', **kws)
+        TaskPanel.__init__(self, parent, controller, panel='regression', **kws)
         self.result = None
         self.save_csvfile   = 'RegressionData.csv'
         self.save_modelfile = 'Model.regmod'
@@ -74,20 +61,22 @@ class RegressionPanel(TaskPanel):
         wids = self.wids
         self.skip_process = True
 
-        wids['fitspace'] = Choice(panel, choices=FitSpace_Choices, size=(250, -1))
-        wids['fitspace'].SetStringSelection(norm)
+        wids['fitspace'] = Choice(panel, choices=list(Linear_ArrayChoices.keys()),
+                                  action=self.onFitSpace, size=(175, -1))
+        wids['fitspace'].SetSelection(0)
         # wids['plotchoice'] = Choice(panel, choices=Plot_Choices,
         #                           size=(250, -1), action=self.onPlot)
 
         wids['method'] = Choice(panel, choices=Regress_Choices, size=(250, -1),
                                 action=self.onRegressMethod)
-
+        wids['method'].SetSelection(1)
         add_text = self.add_text
 
         opts = dict(digits=2, increment=1.0)
+        defaults = self.get_defaultconfig()
+
+        self.make_fit_xspace_widgets(elo=defaults['elo_rel'], ehi=defaults['ehi_rel'])
 
-        w_xmin = self.add_floatspin('xmin', value=defaults['xmin'], **opts)
-        w_xmax = self.add_floatspin('xmax', value=defaults['xmax'], **opts)
         wids['alpha'] =  NumericCombo(panel, make_steps(), fmt='%.6g',
                                       default_val=0.01, width=100)
 
@@ -131,19 +120,21 @@ class RegressionPanel(TaskPanel):
         wids['stat2'] =  SimpleText(panel, ' - - - ')
 
 
-        collabels = [' File /Group Name ', 'External Value',
-                     'Predicted Value']
-        colsizes = [300, 120, 120]
-        coltypes = ['str', 'float:12,4', 'float:12,4']
-        coldefs  = ['', 0.0, 0.0]
+        collabels = [' File Group Name ', 'External Value',
+                     'Predicted Value', 'Training?']
+        colsizes = [325, 110, 110, 90]
+        coltypes = ['str', 'float:12,4', 'float:12,4', 'str']
+        coldefs  = ['', 0.0, 0.0, '']
+
+        self.font_fixedwidth = wx.Font(FONTSIZE_FW, wx.MODERN, wx.NORMAL, wx.BOLD)
 
         wids['table'] = DataTableGrid(panel, nrows=MAX_ROWS,
                                       collabels=collabels,
                                       datatypes=coltypes,
                                       defaults=coldefs,
                                       colsizes=colsizes)
-
-        wids['table'].SetMinSize((650, 225))
+        wids['table'].SetMinSize((700, 225))
+        wids['table'].SetFont(self.font_fixedwidth)
 
         wids['use_selected'] = Button(panel, 'Use Selected Groups',
                                       size=(150, -1),  action=self.onFillTable)
@@ -154,12 +145,10 @@ class RegressionPanel(TaskPanel):
         add_text('Array to Use: ', newrow=True)
         panel.Add(wids['fitspace'], dcol=4)
 
-        # add_text('Plot : ', newrow=True)
-        # panel.Add(wids['plotchoice'], dcol=3)
-        add_text('Fit Energy Range: ')
-        panel.Add(w_xmin)
+        panel.Add(wids['fitspace_label'], newrow=True)
+        panel.Add(self.elo_wids)
         add_text(' : ', newrow=False)
-        panel.Add(w_xmax, dcol=3)
+        panel.Add(self.ehi_wids, dcol=3)
         add_text('Regression Method:')
         panel.Add(wids['method'], dcol=4)
         add_text('PLS # components: ')
@@ -169,9 +158,6 @@ class RegressionPanel(TaskPanel):
         panel.Add(wids['alpha'])
         panel.Add(wids['auto_alpha'], dcol=2)
         panel.Add(wids['fit_intercept'])
-        wids['alpha'].Disable()
-        wids['auto_alpha'].Disable()
-        wids['fit_intercept'].Disable()
 
         add_text('Cross Validation: ')
         add_text(' # folds, # repeats: ', newrow=False)
@@ -210,6 +196,7 @@ class RegressionPanel(TaskPanel):
         sizer.Add((10, 10), 0, LEFT, 3)
         sizer.Add(panel, 1, LEFT, 3)
         pack(self, sizer)
+        self.onRegressMethod()
         self.skip_process = False
 
     def onRegressMethod(self, evt=None):
@@ -218,39 +205,82 @@ class RegressionPanel(TaskPanel):
         self.wids['alpha'].Enable(use_lasso)
         self.wids['auto_alpha'].Enable(use_lasso)
         self.wids['fit_intercept'].Enable(use_lasso)
-        self.wids['fit_intercept'].Enable(use_lasso)
         self.wids['auto_scale_pls'].Enable(not use_lasso)
         self.wids['ncomps'].Enable(not use_lasso)
 
+    def onFitSpace(self, evt=None):
+        fitspace = self.wids['fitspace'].GetStringSelection()
+        self.update_config(dict(fitspace=fitspace))
+        arrname = Linear_ArrayChoices.get(fitspace, 'norm')
+        self.update_fit_xspace(arrname)
 
-    def fill_form(self, dgroup):
-        opts = self.get_config(dgroup)
+
+    def fill_form(self, dgroup=None, opts=None):
+        conf = deepcopy(self.get_config(dgroup=dgroup, with_erange=True))
+        if opts is None:
+            opts = {}
+        conf.update(opts)
         self.dgroup = dgroup
-        if isinstance(dgroup, Group):
-            if not hasattr(dgroup, 'norm'):
-                self.xasmain.process_normalization(dgroup)
-            d_emin = min(dgroup.energy)
-            d_emax = max(dgroup.energy)
-            if opts['xmin'] < d_emin:
-                opts['xmin'] = -40 + int(dgroup.e0/10.0)*10
-            if opts['xmax'] > d_emax:
-                opts['xmax'] =  110 + int(dgroup.e0/10.0)*10
-
         self.skip_process = True
         wids = self.wids
-        for attr in ('xmin', 'xmax', 'alpha'):
-            val = opts.get(attr, None)
+
+        for attr in ('fitspace','method'):
+            if attr in conf:
+                wids[attr].SetStringSelection(conf[attr])
+
+        for attr in ('elo', 'ehi', 'alpha', 'varname', 'cv_folds', 'cv_repeats'):
+            val = conf.get(attr, None)
             if val is not None:
                 if attr == 'alpha':
-                    val = "%.6g" % val
-                wids[attr].SetValue(val)
+                    if val < 0:
+                        val = 0.001
+                        conf['auto_alpha'] = True
+                    val = '%.6g' % val
+                if attr in wids:
+                    wids[attr].SetValue(val)
 
-        for attr in ('fitspace',):
-            if attr in opts:
-                wids[attr].SetStringSelection(opts[attr])
+        use_lasso = conf['method'].lower().startswith('lasso')
+
+        for attr in ('auto_alpha', 'fit_intercept','auto_scale_pls'):
+            val = conf.get(attr, True)
+            if attr == 'auto_scale_pls':
+                val = val and not use_lasso
+            else:
+                val = val and use_lasso
+            wids[attr].SetValue(val)
+        self.onRegressMethod()
 
         self.skip_process = False
 
+    def read_form(self):
+        dgroup = self.controller.get_group()
+        form = {'groupname': getattr(dgroup, 'groupname', 'No Group')}
+
+        for k in ('fitspace', 'method'):
+            form[k] = self.wids[k].GetStringSelection()
+
+        for k in ('elo', 'ehi', 'alpha', 'cv_folds',
+                  'cv_repeats', 'ncomps', 'varname'):
+            form[k] = self.wids[k].GetValue()
+
+        form['alpha'] = float(form['alpha'])
+        if form['alpha'] < 0:
+            form['alpha'] = 1.e-3
+
+        for k in ('auto_scale_pls', 'auto_alpha',  'fit_intercept'):
+            form[k] = self.wids[k].IsChecked()
+
+        mname = form['method'].lower()
+        form['use_lars'] = 'lars' in mname
+        form['funcname'] = 'pls'
+        if mname.startswith('lasso'):
+            form['funcname'] = 'lasso'
+            if form['auto_alpha']:
+                form['alpha'] = None
+
+        return form
+
+
     def onFillTable(self, event=None):
         selected_groups = self.controller.filelist.GetCheckedStrings()
         varname = fix_varname(self.wids['varname'].GetValue())
@@ -260,19 +290,19 @@ class RegressionPanel(TaskPanel):
             gname = self.controller.file_groups[fname]
             grp = self.controller.get_group(gname)
             grid_data.append([fname, getattr(grp, varname, 0.0),
-                              getattr(grp, predname, 0.0)])
+                              getattr(grp, predname, 0.0), 'Yes'])
 
         self.wids['table'].table.data = grid_data
         self.wids['table'].table.View.Refresh()
 
     def onTrainModel(self, event=None):
-        opts = self.read_form()
-        varname = opts['varname']
+        form = self.read_form()
+        varname = form['varname']
         predname = varname + '_predicted'
 
         grid_data = self.wids['table'].table.data
         groups = []
-        for fname, yval, pval in grid_data:
+        for fname, yval, pval, istrain in grid_data:
             gname = self.controller.file_groups[fname]
             grp = self.controller.get_group(gname)
             setattr(grp, varname, yval)
@@ -282,73 +312,104 @@ class RegressionPanel(TaskPanel):
         cmds = ['# train linear regression model',
                'training_groups = [%s]' % ', '.join(groups)]
 
-        copts = ["varname='%s'" % varname,
-                 "xmin=%.4f" % opts['xmin'],
-                 "xmax=%.4f" % opts['xmax']]
+        copts = ["varname='%s'" % varname, "xmin=%.4f" % form['elo'],
+                 "xmax=%.4f" % form['ehi']]
 
-        arrname = 'norm'
-        if opts['fitspace'] == dmude:
-            arrname = 'dmude'
-        elif opts['fitspace'] == chik:
-            arrname = 'chi'
+        arrname = Linear_ArrayChoices.get(form['fitspace'], 'norm')
         copts.append("arrayname='%s'" % arrname)
 
-        self.method = 'pls'
-        if opts['method'].lower().startswith('lasso'):
-            self.method = 'lasso'
-            if opts['auto_alpha']:
+        if form['method'].lower().startswith('lasso'):
+            if form['auto_alpha']:
                 copts.append('alpha=None')
             else:
-                copts.append('alpha=%s' % opts['alpha'])
-            copts.append('use_lars=%s' % repr('lars' in opts['method'].lower()))
-            copts.append('fit_intercept=%s' % repr(opts['fit_intercept']))
+                copts.append('alpha=%.6g' % form['alpha'])
+            copts.append('use_lars=%s' % repr('lars' in form['method'].lower()))
+            copts.append('fit_intercept=%s' % repr(form['fit_intercept']))
         else:
-            copts.append('ncomps=%d' % opts['ncomps'])
-            copts.append('scale=%s' % repr(opts['auto_scale_pls']))
+            copts.append('ncomps=%d' % form['ncomps'])
+            copts.append('scale=%s' % repr(form['auto_scale_pls']))
+
+        callargs = ', '.join(copts)
 
-        copts = ', '.join(copts)
         cmds.append("reg_model = %s_train(training_groups, %s)" %
-                    (self.method, copts))
+                    (form['funcname'], callargs))
+
         self.larch_eval('\n'.join(cmds))
         reg_model = self.larch_get('reg_model')
+        reg_model.form = form
+        self.use_regmodel(reg_model)
+
+    def use_regmodel(self, reg_model):
+        if reg_model is None:
+            return
+        opts = self.read_form()
+
+        if hasattr(reg_model, 'form'):
+            opts.update(reg_model.form)
+
+        self.write_message('Regression Model trained: %s' % opts['method'])
+        rmse_cv = reg_model.rmse_cv
+        if rmse_cv is not None:
+            rmse_cv = "%.4f" % rmse_cv
+        stat = "RMSE_CV = %s, RMSE = %.4f" % (rmse_cv, reg_model.rmse)
+        self.wids['stat1'].SetLabel(stat)
+        if opts['funcname'].startswith('lasso'):
+            stat = "Alpha = %.4f, %d active components"
+            self.wids['stat2'].SetLabel(stat % (reg_model.alpha,
+                                                len(reg_model.active)))
+
+            if opts['auto_alpha']:
+                self.wids['alpha'].add_choice(reg_model.alpha)
+
+        else:
+            self.wids['stat2'].SetLabel('- - - ')
+        training_groups = reg_model.groupnames
+        ntrain = len(training_groups)
+        grid_data = self.wids['table'].table.data
+        grid_new = []
+        for i in range(ntrain): # min(ntrain, len(grid_data))):
+            fname = training_groups[i]
+            istrain = 'Yes' if fname in training_groups else 'No'
+            grid_new.append( [fname, reg_model.ydat[i], reg_model.ypred[i], istrain])
+        self.wids['table'].table.data = grid_new
+        self.wids['table'].table.View.Refresh()
+
+        if reg_model.cv_folds not in (0, None):
+            self.wids['cv_folds'].SetValue(reg_model.cv_folds)
+        if reg_model.cv_repeats not in (0, None):
+            self.wids['cv_repeats'].SetValue(reg_model.cv_repeats)
+
+        self.wids['save_model'].Enable()
+        self.wids['fit_group'].Enable()
+
+        wx.CallAfter(self.onPlotModel, model=reg_model)
+
+    def onPanelExposed(self, **kws):
+        # called when notebook is selected
+        try:
+            fname = self.controller.filelist.GetStringSelection()
+            gname = self.controller.file_groups[fname]
+            dgroup = self.controller.get_group(gname)
+            if not hasattr(dgroup, 'norm'):
+                self.xasmain.process_normalization(dgroup)
+            self.fill_form(dgroup)
+        except:
+            pass # print(" Cannot Fill prepeak panel from group ")
+
+        reg_model = getattr(self.larch.symtable, 'reg_model', None)
         if reg_model is not None:
-            self.write_message('Regression Model trained: %s' % opts['method'])
-            rmse_cv = reg_model.rmse_cv
-            if rmse_cv is not None:
-                rmse_cv = "%.4f" % rmse_cv
-            stat = "RMSE_CV = %s, RMSE = %.4f" % (rmse_cv, reg_model.rmse)
-            self.wids['stat1'].SetLabel(stat)
-            if self.method == 'lasso':
-                stat = "Alpha = %.4f, %d active components"
-                self.wids['stat2'].SetLabel(stat % (reg_model.alpha,
-                                                    len(reg_model.active)))
+            self.use_regmodel(reg_model)
 
-                if opts['auto_alpha']:
-                    self.wids['alpha'].add_choice(reg_model.alpha)
-
-            else:
-                self.wids['stat2'].SetLabel('- - - ')
-
-            for i, row in enumerate(grid_data):
-                grid_data[i] = [row[0], row[1], reg_model.ypred[i]]
-            self.wids['table'].table.data = grid_data
-            self.wids['table'].table.View.Refresh()
-
-            if reg_model.cv_folds not in (0, None):
-                self.wids['cv_folds'].SetValue(reg_model.cv_folds)
-            if reg_model.cv_repeats not in (0, None):
-                self.wids['cv_repeats'].SetValue(reg_model.cv_repeats)
-
-            self.wids['save_model'].Enable()
-            self.wids['fit_group'].Enable()
-
-            wx.CallAfter(self.onPlotModel, model=reg_model)
 
     def onPredictGroups(self, event=None):
         opts = self.read_form()
         varname = opts['varname'] + '_predicted'
 
+        reg_model = self.larch_get('reg_model')
+        training_groups = reg_model.groupnames
+
         grid_data = self.wids['table'].table.data
+
         gent = {}
         if len(grid_data[0][0].strip()) == 0:
             grid_data = []
@@ -361,13 +422,14 @@ class RegressionPanel(TaskPanel):
             grp   = self.controller.get_group(gname)
             extval = getattr(grp, opts['varname'], 0)
             cmd = "%s.%s = %s_predict(%s, reg_model)" % (gname, varname,
-                                                         self.method, gname)
+                                                         opts['funcname'], gname)
             self.larch_eval(cmd)
             val = self.larch_get('%s.%s' % (gname, varname))
             if fname in gent:
                 grid_data[gent[fname]][2] = val
             else:
-                grid_data.append([fname, extval, val])
+                istrain = 'Yes' if fname in training_groups else 'No'
+                grid_data.append([fname, extval, val, istrain])
             self.wids['table'].table.data = grid_data
             self.wids['table'].table.View.Refresh()
 
@@ -381,7 +443,7 @@ class RegressionPanel(TaskPanel):
             return
 
         dlg = wx.FileDialog(self, message="Save Regression Model",
-                            defaultDir=os.getcwd(),
+                            defaultDir=get_cwd(),
                             defaultFile=self.save_modelfile,
                             wildcard=MODEL_WILDCARDS,
                             style=wx.FD_SAVE)
@@ -391,17 +453,12 @@ class RegressionPanel(TaskPanel):
         dlg.Destroy()
         if fname is None:
             return
-        self.save_modelfile = os.path.split(fname)[1]
-        text = str(base64.b64encode(pickle.dumps(reg_model)), 'utf-8')
-        with open(fname, 'w') as fh:
-            fh.write("%s\n" % text)
-        fh.flush()
-        fh.close()
+        save_groups(fname, ['#regression model 1.0', reg_model])
         self.write_message('Wrote Regression Model to %s ' % fname)
 
     def onLoadModel(self, event=None):
         dlg = wx.FileDialog(self, message="Load Regression Model",
-                            defaultDir=os.getcwd(),
+                            defaultDir=get_cwd(),
                             wildcard=MODEL_WILDCARDS, style=wx.FD_OPEN)
 
         fname = None
@@ -410,19 +467,22 @@ class RegressionPanel(TaskPanel):
         dlg.Destroy()
         if fname is None:
             return
-        self.save_modelfile = os.path.split(fname)[1]
-        with open(fname, 'r') as fh:
-            text = fh.read()
+        dat = read_groups(fname)
+        if len(dat) != 2 or not dat[0].startswith('#regression model'):
+            Popup(self, f" '{rfile}' is not a valid Regression model file",
+                  "Invalid file")
 
-        reg_model = pickle.loads(base64.b64decode(bytes(text, 'utf-8')))
+        reg_model = dat[1]
         self.controller.symtable.reg_model = reg_model
+
         self.write_message('Read Regression Model from %s ' % fname)
         self.wids['fit_group'].Enable()
 
+        self.use_regmodel(reg_model)
 
     def onLoadCSV(self, event=None):
         dlg = wx.FileDialog(self, message="Load CSV Data File",
-                            defaultDir=os.getcwd(),
+                            defaultDir=get_cwd(),
                             wildcard=CSV_WILDCARDS, style=wx.FD_OPEN)
 
         fname = None
@@ -470,6 +530,7 @@ class RegressionPanel(TaskPanel):
         opts = self.read_form()
         if model is None:
             return
+        opts.update(model.form)
 
         ppanel = self.controller.get_display(win=1).panel
         viewlims = ppanel.get_viewlimits()
@@ -478,8 +539,10 @@ class RegressionPanel(TaskPanel):
         d_ave = model.spectra.mean(axis=0)
         d_std = model.spectra.std(axis=0)
         ymin, ymax = (d_ave-d_std).min(), (d_ave+d_std).max()
-        if self.method == 'lasso':
-            active_coefs = (model.coefs[model.active])
+
+        if opts['funcname'].startswith('lasso'):
+            active = [int(i) for i in model.active]
+            active_coefs = (model.coefs[active])
             active_coefs = active_coefs/max(abs(active_coefs))
             ymin = min(active_coefs.min(), ymin)
             ymax = max(active_coefs.max(), ymax)
@@ -492,7 +555,7 @@ class RegressionPanel(TaskPanel):
         ymax = ymax + 0.02*(ymax-ymin)
 
 
-        title = '%s Regression results' % (self.method.upper())
+        title = '%s Regression results' % (opts['method'])
 
         ppanel.plot(model.x, d_ave, win=1, title=title,
                     label='mean spectra', xlabel='Energy (eV)',
@@ -500,8 +563,8 @@ class RegressionPanel(TaskPanel):
                     ymin=ymin, ymax=ymax)
         ppanel.axes.fill_between(model.x, d_ave-d_std, d_ave+d_std,
                                  color='#1f77b433')
-        if self.method == 'lasso':
-            ppanel.axes.bar(model.x[model.active], active_coefs,
+        if opts['funcname'].startswith('lasso'):
+            ppanel.axes.bar(model.x[active], active_coefs,
                             1.0, color='#9f9f9f88',
                             label='coefficients')
         else:
@@ -523,7 +586,7 @@ class RegressionPanel(TaskPanel):
                     markersize=8, win=2, new=True, title=title)
 
         ppanel.oplot(model.ypred[sx], indices, label='predicted',
-                    labelfontsize=7, markersize=6, marker='o',
+                    labelxsxfontsize=7, markersize=6, marker='o',
                     linewidth=0, show_legend=True, new=False)
 
         ppanel.axes.barh(indices, diff[sx], 0.5, color='#9f9f9f88')
@@ -532,8 +595,8 @@ class RegressionPanel(TaskPanel):
         ppanel.conf.auto_margins = False
         ppanel.conf.set_margins(left=0.35, right=0.05, bottom=0.15, top=0.1)
         ppanel.canvas.draw()
+        self.controller.set_focus()
 
 
     def onCopyParam(self, name=None, evt=None):
-        print("on Copy Param")
         conf = self.get_config()
