@@ -8,12 +8,13 @@ reading and dealing with Feff.data files in larch:
 returns a Feff Group -- a special variation of a Group -- for
 the path represented by the feffNNNN.dat
 
-  group  = ff2chi(pathlist)
+  group  = ff2chi(paths)
 
 creates a group that contains the chi(k) for the sum of paths.
 """
+import os
 import numpy as np
-
+from copy import deepcopy
 from scipy.interpolate import UnivariateSpline
 from lmfit import Parameters, Parameter
 from lmfit.printfuncs import gformat
@@ -22,13 +23,12 @@ from xraydb import atomic_mass, atomic_symbol
 
 from larch import Group, isNamedClass
 from larch.utils.strutils import fix_varname, b32hash
-from larch.fitting import group2params, isParameter, param_value
+from larch.fitting import group2params, dict2params, isParameter, param_value
 
-from .xafsutils import ETOK, set_xafsGroup
+from .xafsutils import ETOK, ktoe, set_xafsGroup
 from .sigma2_models import add_sigma2funcs
 
-SMALL = 1.e-6
-
+SMALL_ENERGY = 1.e-6
 
 class FeffDatFile(Group):
     def __init__(self, filename=None,  **kws):
@@ -36,7 +36,7 @@ class FeffDatFile(Group):
         kwargs.update(kws)
         Group.__init__(self,  **kwargs)
         if filename is not None:
-            self.__read(filename)
+            self._read(filename)
 
     def __repr__(self):
         if self.filename is not None:
@@ -59,7 +59,7 @@ class FeffDatFile(Group):
     def nleg(self): return self.__nleg__
 
     @nleg.setter
-    def nleg(self, val):     pass
+    def nleg(self, val):    pass
 
     @property
     def rmass(self):
@@ -72,13 +72,44 @@ class FeffDatFile(Group):
         return self.__rmass
 
     @rmass.setter
-    def rmass(self, val):     pass
+    def rmass(self, val):  pass
 
-    def __read(self, filename):
+    def __setstate__(self, state):
+        (self.filename, self.title, self.version, self.shell,
+         self.absorber, self.degen, self.__reff__, self.__nleg__,
+         self.rnorman, self.edge, self.gam_ch, self.exch, self.vmu, self.vfermi,
+         self.vint, self.rs_int, self.potentials, self.geom, self.__rmass,
+         self.k, self.real_phc, self.mag_feff, self.pha_feff,
+         self.red_fact, self.lam, self.rep, self.pha, self.amp) = state
+
+        self.k = np.array(self.k)
+        self.real_phc = np.array(self.real_phc)
+        self.mag_feff = np.array(self.mag_feff)
+        self.pha_feff = np.array(self.pha_feff)
+        self.red_fact = np.array(self.red_fact)
+        self.lam = np.array(self.lam)
+        self.rep = np.array(self.rep)
+        self.pha = np.array(self.pha)
+        self.amp = np.array(self.amp)
+
+    def __getstate__(self):
+        return (self.filename, self.title, self.version, self.shell,
+                self.absorber, self.degen, self.__reff__, self.__nleg__,
+                self.rnorman, self.edge, self.gam_ch, self.exch, self.vmu,
+                self.vfermi, self.vint, self.rs_int, self.potentials,
+                self.geom, self.__rmass, self.k.tolist(),
+                self.real_phc.tolist(), self.mag_feff.tolist(),
+                self.pha_feff.tolist(), self.red_fact.tolist(),
+                self.lam.tolist(), self.rep.tolist(), self.pha.tolist(),
+                self.amp.tolist())
+
+
+    def _read(self, filename):
         try:
-            lines = open(filename, 'r').readlines()
+            with open(filename, 'r') as fh:
+                lines = fh.readlines()
         except:
-            print( 'Error reading file %s ' % filename)
+            print('Error reading file %s ' % filename)
             return
         self.filename = filename
         mode = 'header'
@@ -88,7 +119,7 @@ class FeffDatFile(Group):
         iline = 0
         for line in lines:
             iline += 1
-            line = line[:-1]
+            line = line[:-1].strip()
             if line.startswith('#'): line = line[1:]
             line = line.strip()
             if iline == 1:
@@ -112,15 +143,17 @@ class FeffDatFile(Group):
                 iz = int(words[1])
                 rmt = float(words[3])
                 rnm = float(words[5])
+                if line.startswith('Abs'):
+                    self.shell = words[6]
                 self.potentials.append((ipot, iz, rmt, rnm))
             elif mode == 'header' and line.startswith('Gam_ch'):
                 words  = line.replace('=', ' ').split(' ', 2)
                 self.gam_ch = float(words[1])
                 self.exch   = words[2]
             elif mode == 'header' and line.startswith('Mu'):
-                words  = line.replace('=', ' ').split()
-                self.mu = float(words[1])
-                self.kf = float(words[3])
+                words  = line.replace('=', ' ').replace('eV', ' ').split()
+                self.vmu = float(words[1])
+                self.vfermi = ktoe(float(words[3]))
                 self.vint = float(words[5])
                 self.rs_int= float(words[7])
             elif mode == 'path':
@@ -131,7 +164,7 @@ class FeffDatFile(Group):
                     self.degen, self.__reff__, self.rnorman, self.edge = w
                 elif pcounter > 2:
                     words = line.split()
-                    xyz = [float(x) for x in words[:3]]
+                    xyz = ["%7.4f" % float(x) for x in words[:3]]
                     ipot = int(words[3])
                     iz   = int(words[4])
                     if len(words) > 5:
@@ -140,6 +173,8 @@ class FeffDatFile(Group):
                         lab = atomic_symbol(iz)
                     amass = atomic_mass(iz)
                     geom = [lab, iz, ipot, amass] + xyz
+                    if len(self.geom) == 0:
+                        self.absorber = lab
                     self.geom.append(tuple(geom))
             elif mode == 'arrays':
                 d = np.array([float(x) for x in line.split()])
@@ -159,31 +194,63 @@ class FeffDatFile(Group):
 
 
 PATH_PARS = ('degen', 's02', 'e0', 'ei', 'deltar', 'sigma2', 'third', 'fourth')
-PATHPAR_FMT = "%s__%s"
 
 class FeffPathGroup(Group):
-    def __init__(self, filename=None, label=None, s02=None, degen=None,
+    def __init__(self, filename='', label='', feffrun='', s02=None, degen=None,
                  e0=None, ei=None, deltar=None, sigma2=None, third=None,
-                 fourth=None, _larch=None, **kws):
+                 fourth=None, use=True, **kws):
 
-        kwargs = dict(name='FeffPath: %s' % filename)
+        kwargs = dict(filename=filename)
         kwargs.update(kws)
         Group.__init__(self, **kwargs)
+
         self.filename = filename
-        self.params = None
+        self.feffrun = feffrun
         self.label = label
+        self.use = use
+        self.params = None
         self.spline_coefs = None
-        def_degen = 1
-
+        self.geom  = []
+        self.shell = 'K'
+        self.absorber = None
         self._feffdat = None
-        if filename is not None:
-            self._feffdat = FeffDatFile(filename=filename)
-            self.geom  = self._feffdat.geom
-            def_degen  = self._feffdat.degen
-            if self.label is None:
-                self.label = self.__geom2label()
 
-        self.degen = def_degen if degen  is None else degen
+        self.hashkey = 'p000'
+        self.k = None
+        self.chi = None
+
+        self.__def_degen = 1
+        if filename not in ('', None):
+            if not os.path.exists(filename):
+                raise ValueError(f"Feff Path file '{filename:s}' not found")
+            self._feffdat = FeffDatFile(filename=filename)
+            self.create_spline_coefs()
+
+            self.geom  = self._feffdat.geom
+            self.shell = self._feffdat.shell
+            self.absorber = self._feffdat.absorber
+            self.__def_degen  = self._feffdat.degen
+
+            self.hashkey = self.__geom2label()
+            if self.label in ('', None):
+                self.label = self.hashkey
+
+            if feffrun in ('',  None):
+                try:
+                    dirname, fpfile = os.path.split(filename)
+                    parent, folder = os.path.split(dirname)
+                    self.feffrun = folder
+                except:
+                    pass
+
+        self.init_path_params(degen=degen, s02=s02, e0=e0, ei=ei,
+                              deltar=deltar, sigma2=sigma2, third=third,
+                              fourth=fourth)
+
+    def init_path_params(self, degen=None, s02=None, e0=None, ei=None,
+                       deltar=None, sigma2=None, third=None, fourth=None):
+        """set inital values/expressions for path parameters for Feff Path"""
+        self.degen = self.__def_degen if degen  is None else degen
         self.s02    = 1.0      if s02    is None else s02
         self.e0     = 0.0      if e0     is None else e0
         self.ei     = 0.0      if ei     is None else ei
@@ -192,37 +259,71 @@ class FeffPathGroup(Group):
         self.third  = 0.0      if third  is None else third
         self.fourth = 0.0      if fourth is None else fourth
 
-        self.k = None
-        self.chi = None
-        if self._feffdat is not None:
-            self.create_spline_coefs()
+    def __repr__(self):
+        if self.filename is not None:
+            return 'feffpath((no_file)'
+        return f'feffpath({self.filename})'
+
+    def __getstate__(self):
+        _feffdat_state = self._feffdat.__getstate__()
+        return (self.filename, self.label, self.feffrun, self.degen,
+                self.s02, self.e0, self.ei, self.deltar, self.sigma2,
+                self.third, self.fourth, self.use, _feffdat_state)
+
+
+    def __setstate__(self, state):
+        self.params = self.spline_coefs = self.k = self.chi = None
+        self.use = True
+        if len(state) == 12:  # "use" was added after paths states were being saved
+            (self.filename, self.label, self.feffrun, self.degen,
+             self.s02, self.e0, self.ei, self.deltar, self.sigma2,
+             self.third, self.fourth, _feffdat_state) = state
+        elif len(state) == 13:
+            (self.filename, self.label, self.feffrun, self.degen,
+             self.s02, self.e0, self.ei, self.deltar, self.sigma2,
+             self.third, self.fourth, self.use, _feffdat_state) = state
+
+
+        self._feffdat = FeffDatFile()
+        self._feffdat.__setstate__(_feffdat_state)
+
+        self.create_spline_coefs()
+
+        self.geom  = self._feffdat.geom
+        self.shell = self._feffdat.shell
+        self.absorber = self._feffdat.absorber
+        def_degen  = self._feffdat.degen
+
+        self.hashkey = self.__geom2label()
+        if self.label in ('', None):
+            self.label = self.hashkey
+
 
     def __geom2label(self):
         """generate label by hashing path geometry"""
-        rep = []
-        if self.geom is not None:
-            for atom in self.geom:
-                rep.extend(atom)
-        if self._feffdat is not None:
-            rep.append(self._feffdat.degen)
-            rep.append(self._feffdat.reff)
-
-        for attr in ('s02', 'e0', 'ei', 'deltar', 'sigma2', 'third', 'fourth'):
-            rep.append(getattr(self, attr, '_'))
+        rep = [self._feffdat.degen, self._feffdat.shell, self.feffrun]
+        for atom in self.geom:
+            rep.extend(atom)
+        rep.append("%7.4f" % self._feffdat.reff)
         s = "|".join([str(i) for i in rep])
-        return "p%s" % (b32hash(s)[:8].lower())
+        return "p%s" % (b32hash(s)[:9].lower())
+
+    def pathpar_name(self, parname):
+        """
+        get internal name of lmfit Parameter for a path paramter, using Path's hashkey
+        """
+        return f'{parname}_{self.hashkey}'
 
     def __copy__(self):
-        return FeffPathGroup(filename=self.filename,
-                             s02=self.s02, degen=self.degen, e0=self.e0,
-                             ei=self.ei, deltar=self.deltar, sigma2=self.sigma2,
-                             third=self.third, fourth=self.fourth)
+        newpath = FeffPathGroup()
+        newpath.__setstate__(self.__getstate__())
+        return newpath
 
     def __deepcopy__(self, memo):
-        return FeffPathGroup(filename=self.filename,
-                             s02=self.s02, degen=self.degen, e0=self.e0,
-                             ei=self.ei, deltar=self.deltar, sigma2=self.sigma2,
-                             third=self.third, fourth=self.fourth)
+        newpath = FeffPathGroup()
+        newpath.__setstate__(self.__getstate__())
+        return newpath
+
 
     @property
     def reff(self): return self._feffdat.reff
@@ -243,18 +344,21 @@ class FeffPathGroup(Group):
     def rmass(self, val):  pass
 
     def __repr__(self):
-        if self.filename is not None:
-            return '<FeffPath Group %s>' % self.filename
-        return '<FeffPath Group (empty)>'
+        return f'<FeffPath Group label={self.label:s}, filename={self.filename:s}, use={self.use}>'
 
     def create_path_params(self, params=None):
         """
         create Path Parameters within the current lmfit.Parameters namespace
         """
         if params is not None:
-            self.params = params
+           self.params = params
         if self.params is None:
             self.params = Parameters()
+
+        if (not isinstance(self.params, Parameters) and
+            isinstance(self.params, dict)):
+            self.params = dict2params(self.params)
+
         if self.params._asteval.symtable.get('sigma2_debye', None) is None:
             add_sigma2funcs(self.params)
         if self.label is None:
@@ -266,8 +370,9 @@ class FeffPathGroup(Group):
             if isinstance(val, str):
                 attr = 'expr'
             kws =  {'vary': False, attr: val}
-            parname = fix_varname(PATHPAR_FMT % (pname, self.label))
+            parname = self.pathpar_name(pname)
             self.params.add(parname, **kws)
+            self.params[parname].is_pathparam = True
 
     def create_spline_coefs(self):
         """pre-calculate spline coefficients for feff data"""
@@ -282,9 +387,17 @@ class FeffPathGroup(Group):
         """stores data about this Feff path in the Parameters
         symbol table for use as `reff` and in sigma2 calcs
         """
+        if (not isinstance(self.params, Parameters) and
+            isinstance(self.params, dict)):
+            self.params = dict2params(self.params)
+
         symtab = self.params._asteval.symtable
         symtab['feffpath'] = self._feffdat
-        symtab['reff']  = self._feffdat.reff
+        symtab['reff'] = self._feffdat.reff
+        symtab['vint'] = self._feffdat.vint
+        symtab['vmu']  = self._feffdat.vmu
+        symtab['vfermi'] = self._feffdat.vfermi
+
 
     def __path_params(self, **kws):
         """evaluate path parameter value.  Returns
@@ -298,8 +411,8 @@ class FeffPathGroup(Group):
         out = []
         for pname in PATH_PARS:
             val = kws.get(pname, None)
-            parname = fix_varname(PATHPAR_FMT % (pname, self.label))
             if val is None:
+                parname = self.pathpar_name(pname)
                 val = self.params[parname]._getval()
             out.append(val)
         return out
@@ -315,13 +428,15 @@ class FeffPathGroup(Group):
         pathpars = {}
         for pname in ('degen', 's02', 'e0', 'deltar',
                       'sigma2', 'third', 'fourth', 'ei'):
-            parname = fix_varname(PATHPAR_FMT % (pname, self.label))
+            parname = self.pathpar_name(pname)
             if parname in self.params:
-                pathpars[pname] = (self.params[parname].value, self.params[parname].stderr)
+                pathpars[pname] = (self.params[parname].value,
+                                   self.params[parname].stderr)
 
-        geomlabel  = '     atom      x        y        z       ipot'
-        geomformat = '    %4s      % .4f, % .4f, % .4f  %i'
-        out = ['   Path %s, Feff.dat file = %s' % (self.label, self.filename)]
+        out = [f" = Path '{self.label}' = {self.absorber} {self.shell} Edge",
+               f"    feffdat file = {self.filename}, from feff run '{self.feffrun}'"]
+        geomlabel  = '    geometry  atom      x        y        z      ipot'
+        geomformat = '            %4s      %s, %s, %s  %d'
         out.append(geomlabel)
 
         for atsym, iz, ipot, amass, x, y, z in self.geom:
@@ -336,10 +451,10 @@ class FeffPathGroup(Group):
         for pname in ('degen', 's02', 'e0', 'r',
                       'deltar', 'sigma2', 'third', 'fourth', 'ei'):
             val = strval = getattr(self, pname, 0)
-            parname = fix_varname(PATHPAR_FMT % (pname, self.label))
+            parname = self.pathpar_name(pname)
             std = None
             if pname == 'r':
-                parname = fix_varname(PATHPAR_FMT % ('deltar', self.label))
+                parname = self.pathpar_name('deltar')
                 par = self.params.get(parname, None)
                 val = par.value + self._feffdat.reff
                 strval = 'reff + ' + getattr(self, 'deltar', 0)
@@ -362,13 +477,20 @@ class FeffPathGroup(Group):
 
             svalue = "     {:7s}= {:s}".format(pname, svalue)
             if isinstance(strval, str):
-                svalue = "{:s}  '{:s}'".format(svalue, strval)
+                svalue = "{:s}  := '{:s}'".format(svalue, strval)
 
             if val == 0 and pname in ('third', 'fourth', 'ei'):
                 continue
             out.append(svalue)
         return '\n'.join(out)
 
+    def calc_chi_from_params(self, params, **kws):
+        "calculate chi(k) from Parameters, ParameterGroup, and/or kws for path parameters"
+        if isinstance(params, Parameters):
+            self.create_path_params(params=params)
+        else:
+            self.create_path_params(params=group2params(params))
+        self._calc_chi(**kws)
 
     def _calc_chi(self, k=None, kmax=None, kstep=None, degen=None, s02=None,
                  e0=None, ei=None, deltar=None, sigma2=None,
@@ -385,7 +507,12 @@ class FeffPathGroup(Group):
             kmax = min(max(fdat.k), kmax)
             if kstep is None: kstep = 0.05
             k = kstep * np.arange(int(1.01 + kmax/kstep), dtype='float64')
-
+        if not self.use:
+            self.k = k
+            self.p = k
+            self.chi = 0.0 * k
+            self.chi_imag = 0.0 * k
+            return
         reff = fdat.reff
         # get values for all the path parameters
         (degen, s02, e0, ei, deltar, sigma2, third, fourth)  = \
@@ -395,9 +522,9 @@ class FeffPathGroup(Group):
 
         # create e0-shifted energy and k, careful to look for |e0| ~= 0.
         en = k*k - e0*ETOK
-        if min(abs(en)) < SMALL:
+        if min(abs(en)) < SMALL_ENERGY:
             try:
-                en[np.where(abs(en) < 2*SMALL)] = SMALL
+                en[np.where(abs(en) < 1.5*SMALL_ENERGY)] = SMALL_ENERGY
             except ValueError:
                 pass
         # q is the e0-shifted wavenumber
@@ -439,6 +566,8 @@ class FeffPathGroup(Group):
         self.chi = cchi.imag
         self.chi_imag = -cchi.real
 
+
+
 def path2chi(path, paramgroup=None, **kws):
     """calculate chi(k) for a Feff Path,
     optionally setting path parameter values
@@ -447,6 +576,7 @@ def path2chi(path, paramgroup=None, **kws):
     Parameters:
     ------------
       path:        a FeffPath Group
+      params:      lmfit Parameters or larch ParameterGroup
       kmax:        maximum k value for chi calculation [20].
       kstep:       step in k value for chi calculation [0.05].
       k:           explicit array of k values to calculate chi.
@@ -456,21 +586,19 @@ def path2chi(path, paramgroup=None, **kws):
       None - outputs are written to path group
 
     """
-    params = group2params(paramgroup)
-
     if not isNamedClass(path, FeffPathGroup):
         msg('%s is not a valid Feff Path' % path)
         return
-    path.create_path_params(params=params)
-    path._calc_chi(**kws)
+    path.calc_chi_from_params(paramgroup, **kws)
 
-def ff2chi(pathlist, group=None, paramgroup=None, 
-            k=None, kmax=None, kstep=0.05, _larch=None, **kws):
+
+def ff2chi(paths, group=None, paramgroup=None, k=None, kmax=None,
+            kstep=0.05,  **kws):
     """sum chi(k) for a list of FeffPath Groups.
 
     Parameters:
     ------------
-      pathlist:    a list of FeffPath Groups
+      paths:       a list of FeffPath Groups or dict of {label: FeffPathGroups}
       paramgroup:  a Parameter Group for calculating Path Parameters [None]
       kmax:        maximum k value for chi calculation [20].
       kstep:       step in k value for chi calculation [0.05].
@@ -480,10 +608,25 @@ def ff2chi(pathlist, group=None, paramgroup=None,
        group contain arrays for k and chi
 
     This essentially calls path2chi() for each of the paths in the
-    pathlist and writes the resulting arrays to group.k and group.chi.
+    `paths` and writes the resulting arrays to group.k and group.chi.
 
     """
-    params = group2params(paramgroup)
+    if isinstance(paramgroup, Parameters):
+        params = paramgroup
+    else:
+        params = group2params(paramgroup)
+
+
+    if isinstance(paths, (list, tuple)):
+        pathlist = paths
+    elif isinstance(paths, dict):
+        pathlist = list(paths.values())
+    else:
+        raise ValueErrror('paths must be list, tuple, or dict')
+
+    if len(pathlist) == 0:
+        return Group(k=np.linspace(0, 20, 401),
+                     chi=np.zeros(401, dtype='float64'))
 
     for path in pathlist:
         if not isNamedClass(path, FeffPathGroup):
@@ -498,15 +641,13 @@ def ff2chi(pathlist, group=None, paramgroup=None,
 
     if group is None:
         group = Group()
-    else:
-        group = set_xafsGroup(group, _larch=_larch)
     group.k = k
     group.chi = out
     return group
 
-def feffpath(filename=None, label=None, s02=None, degen=None,
+def feffpath(filename='', label='', feffrun='', s02=None, degen=None,
              e0=None,ei=None, deltar=None, sigma2=None, third=None,
-             fourth=None, _larch=None, **kws):
+             fourth=None, use=True, **kws):
     """create a Feff Path Group from a *feffNNNN.dat* file.
 
     Parameters:
@@ -521,6 +662,8 @@ def feffpath(filename=None, label=None, s02=None, degen=None,
       third:     c_3      value or parameter [0.0]
       fourth:    c_4      value or parameter [0.0]
       ei:        E_i      value or parameter [0.0]
+      feffrun:   label for Feff run          [parent folder of Feff.dat file]
+      use :      use in sum of paths         [True]
 
     For all the options described as **value or parameter** either a
     numerical value or a Parameter (as created by param()) can be given.
@@ -528,8 +671,33 @@ def feffpath(filename=None, label=None, s02=None, degen=None,
     Returns:
     ---------
         a FeffPath Group.
-
     """
-    return FeffPathGroup(filename=filename, label=label, s02=s02,
-                         degen=degen, e0=e0, ei=ei, deltar=deltar,
-                         sigma2=sigma2, third=third, fourth=fourth)
+    return FeffPathGroup(filename=filename, label=label, feffrun=feffrun,
+                         s02=s02, degen=degen, e0=e0, ei=ei, deltar=deltar,
+                         sigma2=sigma2, third=third, fourth=fourth, use=use)
+
+def use_feffpath(pathcache, label, degen=None, s02=None,  e0=None,ei=None,
+                 deltar=None, sigma2=None, third=None, fourth=None, use=True):
+    """use a copy of a Feff Path from a cache of feff paths - a simply dictionary
+    keyed by the path label, and to support in-memory paths, not read from feff.dat files
+
+    Parameters:
+    -----------
+      pathcache: dictionary of feff paths
+      label:     label for path -- the dictionary key
+      degen:     path degeneracy, N [taken from file]
+      s02:       S_0^2    value or parameter [1.0]
+      e0:        E_0      value or parameter [0.0]
+      deltar:    delta_R  value or parameter [0.0]
+      sigma2:    sigma^2  value or parameter [0.0]
+      third:     c_3      value or parameter [0.0]
+      fourth:    c_4      value or parameter [0.0]
+      ei:        E_i      value or parameter [0.0]
+      use:       whether to use path in sum [True]
+      """
+    path = deepcopy(pathcache[label])
+    path.use = use
+    path.init_path_params(s02=s02, degen=degen, e0=e0, ei=ei,
+                         deltar=deltar, sigma2=sigma2, third=third,
+                         fourth=fourth)
+    return path
