@@ -2,7 +2,10 @@
 """
    feffit sums Feff paths to match xafs data
 """
-from collections import Iterable
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable
 from copy import copy, deepcopy
 from functools import partial
 import numpy as np
@@ -21,7 +24,6 @@ from ..fitting import (correlated_values, eval_stderr, ParameterGroup,
 
 from .xafsutils import set_xafsGroup
 from .xafsft import xftf_fast, xftr_fast, ftwindow
-from .sigma2_models import sigma2_correldebye, sigma2_debye
 from .feffdat import FeffPathGroup, ff2chi
 
 
@@ -130,6 +132,10 @@ class TransformGroup(Group):
         group.chir_pha =  complex_phase(out[:irmax])
         group.chir_re  =  out.real[:irmax]
         group.chir_im  =  out.imag[:irmax]
+        if self.rwin is None:
+            self.rwin = ftwindow(self.r_, xmin=self.rmin, xmax=self.rmax,
+                                 dx=self.dr, dx2=self.dr2, window=self.rwindow)
+        group.rwin = self.rwin[:irmax]
 
     def get_kweight(self):
         "if kweight is a list/tuple, use only the first one here"
@@ -157,7 +163,6 @@ class TransformGroup(Group):
         if self.rwin is None:
             self.rwin = ftwindow(self.r_, xmin=self.rmin, xmax=self.rmax,
                                  dx=self.dr, dx2=self.dr2, window=self.rwindow)
-
         cx = chir * self.rwin[:len(chir)]
         return xftr_fast(cx, kstep=self.kstep, nfft=self.nfft)
 
@@ -240,16 +245,24 @@ class FeffitDataSet(Group):
         else:
             self.paths = {}
 
-        self.data = data
+        # make datagroup from passed in data: copy of k, chi, delta_chi, epsilon_k
+        self.data = Group(__name__='Feffit DatasSet from %s' % repr(data),
+                          groupname=getattr(data, 'groupname', repr(data)),
+                          filename=getattr(data, 'filename', repr(data)),
+                          k=data.k[:], chi=data.chi[:])
+        self.data.delta_chi = getattr(data, 'delta_chi', None)
+
+        self.data.epsilon_k = getattr(data, 'epsilon_k', epsilon_k)
+        if epsilon_k is not None:
+            self.data.epsilon_k = epsilon_k
+
         if transform is None:
             transform = TransformGroup()
         else:
             trasform = copy(transform)
         self.transform = transform
-        if epsilon_k is not None:
-            self.data.epsilon_k = epsilon_k
 
-        self.model = Group()
+        self.model = Group(__name__='Feffit Model for %s' % repr(data))
         self.model.k = None
         self.__chi = None
         self.__prepared = False
@@ -277,6 +290,7 @@ class FeffitDataSet(Group):
 
         # ikmax = index_of(trans.k_, max(self.data.k))
         self.model.k = trans.k_[:ikmax]
+        self.model.chi = np.zeros(len(self.model.k), dtype='float64')
         self.__chi = interp(self.model.k, self.data.k, self.data.chi)
         self.n_idp = 1 + 2*(trans.rmax-trans.rmin)*(trans.kmax-trans.kmin)/pi
 
@@ -287,19 +301,26 @@ class FeffitDataSet(Group):
             self.set_epsilon_k(eps_k)
         else:
             self.estimate_noise(chi=self.__chi, rmin=15.0, rmax=30.0)
-            # uncertainty in chi(k) from autobk or other source
-            if hasattr(self.data, 'delta_chi'):
-                if isinstance(self.epsilon_k, (list, tuple)):
+            # if delta_chi (uncertainty in chi(k) from autobk or other source)
+            # exists, add it in quadrature to high-k noise estimate, and
+            # update epsilon_k to be this combination.
+            if getattr(self.data, 'delta_chi', None) is not None:
+                cur_eps_k = getattr(self, 'epsilon_k', 0.0)
+                if isinstance(cur_eps_k, (list, tuple)):
                     eps_ave = 0.
-                    for eps in self.epsilon_k:
+                    for eps in cur_eps_k:
                         eps_ave += eps
-                    self.epsilon_k = eps_ave/len(self.epsilon_k)
-                _dchi = interp(self.model.k, self.data.k, self.data.delta_chi)
-                eps_k = np.sqrt(_dchi**2 + self.epsilon_k**2)
-                self.set_epsilon_k(eps_k)
+                    cur_eps_k = eps_ave/len(cur_eps_k)
+
+                _dchi = self.data.delta_chi
+                if isinstance(_dchi, np.ndarray):
+                    _dchi = interp(self.model.k, self.data.k, _dchi)
+                self.set_epsilon_k(np.sqrt(_dchi**2 + cur_eps_k**2))
 
         # for each path in the list of paths, setup the Path Parameters
         # to use the current Parameters namespace
+        if isinstance(params, Group):
+            params = group2params(params)
         for label, path in self.paths.items():
             path.create_path_params(params=params)
             if path.spline_coefs is None:
@@ -312,7 +333,8 @@ class FeffitDataSet(Group):
         """estimage noise in a chi spectrum from its high r components"""
         trans = self.transform
         trans.make_karrays()
-        if chi is None: chi = self.__chi
+        if chi is None:
+            chi = self.__chi
 
         save = trans.rmin, trans.rmax, trans.fitspace
 
@@ -383,7 +405,7 @@ class FeffitDataSet(Group):
         if not isNamedClass(self.transform, TransformGroup):
             return
         if not self.__prepared:
-            self.prepare_fit()
+            self.prepare_fit(paramgroup)
 
         ff2chi(self.paths, paramgroup=paramgroup, k=self.model.k,
                 _larch=self._larch, group=self.model)
@@ -529,14 +551,25 @@ def feffit(paramgroup, datasets, rmax_out=10, path_outputs=True, _larch=None, **
         chir_re      real part of chi(R).
         chir_im      imaginary part of chi(R).
     """
+    fit_kws = dict(gtol=1.e-6, ftol=1.e-6, xtol=1.e-6)
+    if 'tol' in kws:
+        tol = kws.pop('tol')
+        fit_kws['gtol'] = fit_kws['ftol'] = fit_kws['xtol'] = tol
+
+    fit_kws.update(kws)
 
     work_paramgroup = deepcopy(paramgroup)
+    for pname in dir(paramgroup):  # explicitly copy 'skip'!
+        wpar = getattr(work_paramgroup, pname)
+        opar = getattr(paramgroup, pname)
+        if isinstance(wpar, Parameter):
+            setattr(wpar, 'skip', getattr(opar, 'skip', False))
+
     params = group2params(work_paramgroup)
 
     def _resid(params, datasets=None, pargroup=None, **kwargs):
         """ this is the residual function"""
-        params2group(params, pargroup)
-        return concatenate([d._residual(pargroup) for d in datasets])
+        return concatenate([d._residual(params) for d in datasets])
 
     if isNamedClass(datasets, FeffitDataSet):
         datasets = [datasets]
@@ -549,7 +582,7 @@ def feffit(paramgroup, datasets, rmax_out=10, path_outputs=True, _larch=None, **
 
     fit = Minimizer(_resid, params,
                     fcn_kws=dict(datasets=datasets, pargroup=work_paramgroup),
-                    scale_covar=True, **kws)
+                    scale_covar=True, **fit_kws)
 
     result = fit.leastsq()
     params2group(result.params, work_paramgroup)
@@ -578,7 +611,6 @@ def feffit(paramgroup, datasets, rmax_out=10, path_outputs=True, _larch=None, **
     # so we rescale uncertainties here.
 
     covar = getattr(result, 'covar', None)
-    # print("COVAR " , covar)
     if covar is not None:
         err_scale = (result.nfree / (n_idp - result.nvarys))
         for name in result.var_names:
@@ -626,9 +658,11 @@ def feffit(paramgroup, datasets, rmax_out=10, path_outputs=True, _larch=None, **
     for ds in datasets:
         ds.save_ffts(rmax_out=rmax_out, path_outputs=path_outputs)
 
-    out = Group(name='feffit results', datasets=datasets,
+    out = Group(name='feffit results',
                 paramgroup=work_paramgroup,
-                fitter=fit, fit_details=result, chi_square=chi_square,
+                datasets=datasets,
+                # fitter=fit,
+                fit_details=result, chi_square=chi_square,
                 n_independent=n_idp, chi2_reduced=chi2_reduced,
                 rfactor=rfactor, aic=aic, bic=bic, covar=covar)
 
@@ -688,6 +722,8 @@ def feffit_report(result, min_correl=0.1, with_paths=True, _larch=None):
     else:
         out.append(header % 'Datasets (%i)' % len(datasets))
     for i, ds in enumerate(datasets):
+        if not hasattr(ds, 'epsilon_k'):
+            ds.prepare_fit(params)
         tr = ds.transform
         if len(datasets) > 1:
             out.append(' dataset %i:' % (i+1))
