@@ -59,9 +59,9 @@ CifPublication = namedtuple('CifPublication', ('id', 'journalname', 'year',
 
 
 _CIFDB = None
-AMSCIF_FULL = 'amscif_full.db'
-AMSCIF_TRIM = 'amscif_trim.db'
-SOURCE_URLS = ('https://docs.xrayabsorption.org/',
+AMSCIF_TRIM = 'amcsd_cif1.db'
+AMSCIF_FULL = 'amcsd_cif2.db'
+SOURCE_URLS = ('https://docs.xrayabsorption.org/databases/',
                'https://millenia.cars.aps.anl.gov/xraylarch/downloads/')
 
 def get_nonzero(thing):
@@ -239,7 +239,8 @@ class CifStructure():
                                          energy=energy, qmin=qmin,
                                          qmax=qmax)
 
-    def get_feffinp(self, absorber, edge=None, cluster_size=8.0, absorber_site=1, version8=True):
+    def get_feffinp(self, absorber, edge=None, cluster_size=8.0, absorber_site=1,
+                    with_h=False, version8=True):
         pub = self.publication
         journal = f"{pub.journalname} {pub.volume}, pp. {pub.page_first}-{pub.page_last} ({pub.year:d})"
         authors = ', '.join(pub.authors)
@@ -254,8 +255,9 @@ class CifStructure():
             for i, line in enumerate(self.pub_title.split('\n')):
                 titles.append(f'Title{i+1:d}: {line}')
 
+        print(" get feffinp -> " )
         return cif2feffinp(self.ciftext, absorber, edge=edge,
-                           cluster_size=cluster_size,
+                           cluster_size=cluster_size, with_h=with_h,
                            absorber_site=absorber_site,
                            extra_titles=titles, version8=version8)
 
@@ -310,10 +312,9 @@ class AMSCIFDB():
         else:
             self.session = sessionmaker(bind=self.engine)()
 
-        self.metadata = MetaData(self.engine)
-        self.metadata.reflect()
+        self.metadata = MetaData()
+        self.metadata.reflect(self.engine)
         self.tables = self.metadata.tables
-        elems = self.tables['elements'].select().execute()
         self.cif_elems = None
 
     def close(self):
@@ -325,6 +326,19 @@ class AMSCIFDB():
         "generic query"
         return self.session.query(*args, **kws)
 
+    def execall(self, query):
+        return self.session.execute(query).fetchall()
+
+    def execone(self, query):
+        results = self.session.execute(query).fetchone()
+        if results is None or len(results) < 1:
+            return None
+        return results
+
+    def get_all(self, tablename):
+        return self.execall(self.tables[tablename].select())
+
+
     def get_version(self, long=False, with_history=False):
         """
         return sqlite3 database and python library version numbers
@@ -337,7 +351,7 @@ class AMSCIFDB():
             string: version information
         """
         out = []
-        rows = self.tables['Version'].select().execute().fetchall()
+        rows = self.get_all('version')
         if not with_history:
             rows = rows[-1:]
         if long or with_history:
@@ -345,6 +359,8 @@ class AMSCIFDB():
                 out.append(f"AMSCIF DB Version: {row.tag} [{row.date}] '{row.notes}'")
             out.append(f"Python Version: {__version__}")
             out = "\n".join(out)
+        elif row is None:
+            out = f"AMSCIF DB Version: unknown, Python Version: {__version__}"
         else:
             out = f"AMSCIF DB Version: {rows[0].tag}, Python Version: {__version__}"
         return out
@@ -353,12 +369,12 @@ class AMSCIFDB():
         tab = self.tables[table]
         if '"' in name:
             name = name.replace('"', '\"')
-        rows = tab.select(tab.c.name==name).execute().fetchall()
+        rows = self.execall(tab.select(tab.c.name==name))
         if len(rows) == 0:
             if not add:
                 return None
             tab.insert().execute(name=name)
-            rows = tab.select(tab.c.name==name).execute().fetchall()
+            rows = self.execall(tab.select(tab.c.name==name))
         return rows[0]
 
 
@@ -366,7 +382,7 @@ class AMSCIFDB():
         """get row from spacegroups table by HM notation.  See add_spacegroup()
         """
         tab = self.tables['spacegroups']
-        rows = tab.select(tab.c.hm_notation==hm_name).execute().fetchall()
+        rows = self.execall(tab.select(tab.c.hm_notation==hm_name))
         if len(rows) >0:
             return rows[0]
         return None
@@ -407,7 +423,7 @@ class AMSCIFDB():
         if id is not None:
             args.append(tab.c.id==id)
 
-        rows = tab.select(and_(*args)).execute().fetchall()
+        rows = self.execall(tab.select(and_(*args)))
         if len(rows) > 0:
             out = []
             authtab = self.tables['authors']
@@ -415,7 +431,7 @@ class AMSCIFDB():
             for row in rows:
                 q = select(authtab.c.name).where(and_(authtab.c.id==patab.c.author_id,
                                                       patab.c.publication_id==row.id))
-                authors = tuple([i[0] for i in self.conn.execute(q).fetchall()])
+                authors = tuple([i[0] for i in self.execall(q)])
                 out.append(CifPublication(row.id, row.journalname, row.year,
                                           row.volume, row.page_first,
                                           row.page_last, authors))
@@ -498,6 +514,7 @@ class AMSCIFDB():
         cif = CifParser(filename)
         cifkey = list(cif._cif.data.keys())[0]
         dat = cif._cif.data[cifkey].data
+
         formula = None
         for formname in ('_chemical_formula_sum', '_chemical_formula_moiety'):
             if formname in dat:
@@ -515,13 +532,20 @@ class AMSCIFDB():
             raise ValueError(f'Cannot read chemical formula from file {filename:s}')
 
         compound = '<missing>'
-        for compname in ('_chemical_compound_source', '_chemical_name_systematic'):
+        for compname in ('_chemical_compound_source',
+                         '_chemical_name_systematic',
+                         '_chemical_name_common'):
             if compname in dat:
                 compound = dat[compname]
 
 
         # get spacegroup and symmetry
-        sgroup_name = dat['_symmetry_space_group_name_H-M']
+        sgroup_name = dat.get('_symmetry_space_group_name_H-M', None)
+        if sgroup_name is None:
+            for key, val in dat.items():
+                if 'space_group' in key and 'H-M' in key:
+                    sgroup_name = val
+
         symm_xyz = dat.get('_space_group_symop_operation_xyz', None)
         if symm_xyz is None:
             symm_xyz = dat.get('_symmetry_equiv_pos_as_xyz', None)
@@ -541,22 +565,41 @@ class AMSCIFDB():
         if sgroup is None:
             sgroup = self.add_spacegroup(sgroup_name, symm_xyz)
 
-        min_name = dat.get('_chemical_name_mineral', '<missing>')
+        min_name = '<missing>'
+        for mname in ('_chemical_name_mineral',
+                       '_chemical_name_common'):
+            if mname in dat:
+                min_name = dat[mname]
         mineral = self._get_tablerow('minerals', min_name)
 
-        pubs = self.get_publications(journalname=dat.get('_journal_name_full', 'nojournal'),
-                                    year=dat.get('_journal_year', 00),
-                                    volume=dat.get('_journal_volume', ''),
-                                    page_first=dat.get('_journal_page_first', ''),
-                                    page_last=dat.get('_journal_page_last', ''))
+        # get publication data (including ISCD style of 'citation' in place of 'journal' )
+        pubdict = dict(journalname=dat.get('_journal_name_full', None),
+                       year=dat.get('_journal_year', None),
+                       volume=dat.get('_journal_volume', None),
+                       page_first=dat.get('_journal_page_first', None),
+                       page_last=dat.get('_journal_page_last', None))
 
+        for key, alt, dval in (('journalname', 'journal_full', 'No Journal'),
+                               ('year', None, -1),
+                               ('volume', 'journal_volume', 0),
+                               ('page_first', None, 0),
+                               ('page_last', None, 0)):
+            if pubdict[key] is None:
+                if alt is None:
+                    alt = key
+                alt = '_citation_%s' % alt
+                pubdict[key] = dat.get(alt, [dval])[0]
+        authors = dat.get('_publ_author_name', None)
+        if authors is None:
+            authors = dat.get('_citation_author_name', ['Anonymous'])
+
+        pubs = self.get_publications(**pubdict)
         if pubs is None:
-            pub = self.add_publication(dat.get('_journal_name_full', 'nojournal'),
-                                       dat.get('_journal_year', 00),
-                                       dat.get('_publ_author_name', ['noauthor']),
-                                       volume=dat.get('_journal_volume', ''),
-                                       page_first=dat.get('_journal_page_first', ''),
-                                       page_last=dat.get('_journal_page_last', ''))
+            pub = self.add_publication(pubdict['journalname'],
+                                       pubdict['year'], authors,
+                                       volume=pubdict['volume'],
+                                       page_first=pubdict['page_first'],
+                                       page_last=pubdict['page_last'])
         else:
             pub = pubs[0]
 
@@ -574,7 +617,7 @@ class AMSCIFDB():
 
         # check again for this cif id (must match CIF AMS id and formula
         tabcif = self.tables['cif']
-        this = select(tabcif.c.id, tabcif.c.formula).where(tabcif.c.id==int(cif_id)).execute().fetchone()
+        this = self.execone(select(tabcif.c.id, tabcif.c.formula).where(tabcif.c.id==int(cif_id)))
         if this is not None:
             _cid, _formula = this
             if formula.replace(' ', '') == _formula.replace(' ', ''):
@@ -582,6 +625,34 @@ class AMSCIFDB():
             else:
                 cif_id = self.next_cif_id()
 
+        print("Would Add Cif Data !" )
+        print(cif_id, mineral.id, pub.id, sgroup.id,
+              formula)
+        print(compound,
+              dat.get('_amcsd_formula_title', '<missing>'),
+              dat.get('_publ_section_title', '<missing>'),
+              json.dumps(dat['_atom_site_label']))
+        print(put_optarray(dat, '_atom_site_fract_x'),
+              put_optarray(dat, '_atom_site_fract_y'),
+              put_optarray(dat, '_atom_site_fract_z'),
+              put_optarray(dat, '_atom_site_occupancy'),
+              put_optarray(dat, '_atom_site_U_iso_or_equiv'))
+        print( json.dumps(dat.get('_atom_site_aniso_label', '<missing>')))
+        print(put_optarray(dat, '_atom_site_aniso_U_11'),
+              put_optarray(dat, '_atom_site_aniso_U_22'),
+              put_optarray(dat, '_atom_site_aniso_U_33'),
+              put_optarray(dat, '_atom_site_aniso_U_12'),
+              put_optarray(dat, '_atom_site_aniso_U_13'),
+              put_optarray(dat, '_atom_site_aniso_U_23'))
+        print('cell ', dat['_cell_length_a'],
+              dat['_cell_length_b'],
+              dat['_cell_length_c'],
+              dat['_cell_angle_alpha'],
+              dat['_cell_angle_beta'],
+              dat['_cell_angle_gamma'],
+              dat.get('_cell_volume', -1),
+              density,
+              url)
 
         self.add_cifdata(cif_id, mineral.id, pub.id, sgroup.id,
                          formula=formula, compound=compound,
@@ -614,7 +685,7 @@ class AMSCIFDB():
     def get_cif(self, cif_id, as_strings=False):
         """get Cif Structure object """
         tab = self.tables['cif']
-        cif = tab.select(tab.c.id==cif_id).execute().fetchone()
+        cif = self.execone(tab.select(tab.c.id==cif_id))
         if cif is None:
             return
 
@@ -623,8 +694,8 @@ class AMSCIFDB():
         tab_pa   = self.tables['publication_authors']
         tab_min  = self.tables['minerals']
         tab_sp   = self.tables['spacegroups']
-        mineral  = tab_min.select(tab_min.c.id==cif.mineral_id).execute().fetchone()
-        sgroup   = tab_sp.select(tab_sp.c.id==cif.spacegroup_id).execute().fetchone()
+        mineral  = self.execone(tab_min.select(tab_min.c.id==cif.mineral_id))
+        sgroup   = self.execone(tab_sp.select(tab_sp.c.id==cif.spacegroup_id))
         hm_symbol = sgroup.hm_notation
         if '%var' in hm_symbol:
             hm_symbol = hm_symbol.split('%var')[0]
@@ -663,12 +734,23 @@ class AMSCIFDB():
                          'atoms_u_iso', 'atoms_aniso_u11', 'atoms_aniso_u22',
                          'atoms_aniso_u33', 'atoms_aniso_u12',
                          'atoms_aniso_u13', 'atoms_aniso_u23'):
-                val =  get_optarray(getattr(cif, attr))
-                if val == '0':
-                    val = None
-                elif not as_strings and '?' not in val:
-                    val = np.array([float(v) for v in val])
-                setattr(out, attr, val)
+                try:
+                    val =  get_optarray(getattr(cif, attr))
+                    if val == '0':
+                        val = None
+                    elif not as_strings:
+                        tmp = []
+                        for i in range(len(val)):
+                            v = val[i]
+                            if v in ('?', '.'):
+                                v = 2.
+                            else:
+                                v = float(v)
+                            tmp.append(v)
+                        val = tmp
+                    setattr(out, attr, val)
+                except:
+                    print(f"could not parse CIF entry for {cif_id} '{attr}': {val} ")
 
         out.qval = None
         if cif.qdat is not None:
@@ -681,7 +763,7 @@ class AMSCIFDB():
         """next available CIF ID > 200000 that is not in current table"""
         max_id = 200_000
         tabcif = self.tables['cif']
-        for row in select(tabcif.c.id).where(tabcif.c.id>200000).execute().fetchall():
+        for row in self.execall(select(tabcif.c.id).where(tabcif.c.id>200000)):
             if row[0] > max_id:
                 max_id = row[0]
         return max_id + 1
@@ -689,21 +771,21 @@ class AMSCIFDB():
 
     def all_minerals(self):
         names = []
-        for row in self.tables['minerals'].select().execute().fetchall():
+        for row in self.get_all('minerals'):
             if row.name not in names:
                 names.append(row.name)
         return names
 
     def all_authors(self):
         names = []
-        for row in self.tables['authors'].select().execute().fetchall():
+        for row in self.get_all('authors'):
             if row.name not in names:
                 names.append(row.name)
         return names
 
     def all_journals(self):
         names = []
-        for row in self.tables['publications'].select().execute().fetchall():
+        for row in self.get_all('publications'):
             if row.journalname not in names:
                 names.append(row.journalname)
         return names
@@ -711,7 +793,7 @@ class AMSCIFDB():
     def get_cif_elems(self):
         if self.cif_elems is None:
             out = {}
-            for row in self.tables['cif_elements'].select().execute().fetchall():
+            for row in self.get_all('cif_elements'):
                 cifid = int(row.cif_id)
                 if cifid not in out:
                     out[cifid] = []
@@ -750,10 +832,10 @@ class AMSCIFDB():
                                                '$' in mineral_name):
             pattern = mineral_name.replace('*', '.*').replace('..*', '.*')
             matches = []
-            for row in self.tables['minerals'].select().execute().fetchall():
+            for row in self.get_all('minerals'):
                 if re.search(pattern, row.name, flags=re.IGNORECASE) is not None:
                     query = select(tabcif.c.id).where(tabcif.c.mineral_id==row.id)
-                    for m in [row[0] for row in query.execute().fetchall()]:
+                    for m in [row[0] for row in self.execall(query)]:
                         if m not in matches:
                            matches.append(m)
 
@@ -761,8 +843,8 @@ class AMSCIFDB():
                 pattern = journal_name.replace('*', '.*').replace('..*', '.*')
                 new_matches = []
                 for c in matches:
-                    pub_id = select(tabcif.c.publication_id).where(tabcif.c.id==c).execute().fetchone()[0]
-                    this_journal = select(tabpub.c.journalname).where(tabpub.c.id==pub_id).execute().fetchone()[0]
+                    pub_id = self.execone(select(tabcif.c.publication_id).where(tabcif.c.id==c))
+                    this_journal = self.execone(select(tabpub.c.journalname).where(tabpub.c.id==pub_id))
                     if re.search(pattern,  this_journal, flags=re.IGNORECASE) is not None:
                         new_matches.append[c]
                 matches = new_matches
@@ -786,7 +868,7 @@ class AMSCIFDB():
             query = select(tabcif.c.id)
             if len(args) > 0:
                 query = select(tabcif.c.id).where(and_(*args))
-            matches = [row[0] for row in self.conn.execute(query).fetchall()]
+            matches = [row[0] for row in self.execall(query)]
             matches = list(set(matches))
         #
         cif_elems = self.get_cif_elems()
@@ -794,7 +876,7 @@ class AMSCIFDB():
             for el in contains_elements:
                 new_matches = []
                 for row in matches:
-                    if el in cif_elems[row]:
+                    if row in cif_elems and el in cif_elems[row]:
                         new_matches.append(row)
                 matches = new_matches
 
@@ -816,7 +898,7 @@ class AMSCIFDB():
         if full_occupancy:
             good = []
             for cif_id in matches:
-                cif = tabcif.select(tabcif.c.id==cif_id).execute().fetchone()
+                cif = self.execone(tabcif.select(tabcif.c.id==cif_id))
                 occ = get_optarray(getattr(cif, 'atoms_occupancy'))
                 if occ in ('0', 0, None):
                     good.append(cif_id)
